<?php

/**
  CLASE PARA SET/GET LOS VALORES DE LA FACTURA Y COEXION CON HACIENDA
  BY: JONATHAN SANCHEZ
  DATE: JULIO, 2018
 * */
class WC_Efactura_hacienda_connect {

  private $IDP_URI = "https://idp.comprobanteselectronicos.go.cr/auth/realms/rut-stag/protocol/openid-connect";
  private $RECEPCION_URI = "https://api.comprobanteselectronicos.go.cr/recepcion-sandbox/v1";
  private $IDP_CLIENT_ID = "api-stag"; //Test: 'api-stag' Production: 'api-prod'
  private $usuario;
  private $password;
  private $pin;
  private $consecutive;
  private $idEmisor;
  private $situacionComprobante;
  private $codSeguridad; //  Codigo de segurirdad de 8 digitos
  private $clave;
  private $idReceptor;
  private $typeReceptor = "01";
  private $typeEmisor;
  private $fecha; //Obligatorio
  private $varMedioPago;

  function __construct() {    
    //setting paths to store information
    $base_dir = WP_CONTENT_DIR . '/uploads/fefiles';
    $this->xmlPath = "{$base_dir}/xml/";
    $this->xmlPathSigned = "{$base_dir}/xml_signed/";
    $this->certsPath = "{$base_dir}/certs/";
    $this->xmlConfirm = "{$base_dir}/xml_confirm/";
    $this->creditPath = "{$base_dir}/xml_nota_credito/";
    $this->acceptPath = "{$base_dir}/xml_accept/";
  }

  function setTypeReceptor($typeReceptor) {
    $this->typeReceptor = $typeReceptor;
  }

  function setTypeEmisor($typeEmisor) {
    $this->typeEmisor = $typeEmisor;
  }

  /**
    @param string $pin
    - Pin es la contraseña para accesar a la llave criptografica
   * */
  function setPin($pin) {
    $this->pin = $pin;
  }

  public function setConsecutive($consecutive) {
    $this->consecutive = $consecutive;
  }

  /**
    @param string $idEmisor
    - Numero de identificacion del emisor para formar la clave
   * */
  public function setIdEmisor($idEmisor) {
    $this->idEmisor = $idEmisor;
  }

  function getIdReceptor() {
    return $this->idReceptor;
  }

  function getTypeReceptor() {
    return $this->typeReceptor;
  }

  function getTypeEmisor() {
    return $this->typeEmisor;
  }

  /**
    @param string $idReceptor
    - Numero de identificacion del receptor
   * */
  public function setIdReceptor($idReceptor) {
    $type = $this->getTypeReceptor();
    $max = 9;
    if ($type == "02" || $type == "04") {
      $max = 10;
    }

    if (strlen($idReceptor) < $max && $type != "03") {
      $idReceptor = str_pad($idReceptor, $max, '0', STR_PAD_LEFT);
    }
    $this->idReceptor = $idReceptor;
  }

  /**
    @param string $situacionComprobante
    - Codigo de la situacion del comprobante para formar la clave
    - 1 - Normal
    - 2 - Contingencia
    - 3 - Sin Internet
   * */
  public function setSituacionComprobante($situacionComprobante) {
    $this->situacionComprobante = $situacionComprobante;
  }

  /**
    @param string $codSeguridad
    - Codigo de segurirdad para formar la clave
   * */
  public function setCodSeguridad($codSeguridad) {
    $this->codSeguridad = $codSeguridad;
  }

  public function setClave($clave) {
    $this->clave = $clave;
  }

  /**
    @param string $varMedioPago
    - 01 - Efectivo
    - 02 - Tarjeta
    - 03 - Cheque
    - 04 - Transferencia - Deposito Bancario
    - 05 - Recaudado por terceros
    - 99 - Otros (indicar)
   * */
  public function setVarMedioPago($varMedioPago) {
    $this->varMedioPago = $varMedioPago;
  }

  /**
   * Change the default value from staging to production enviroment
   */
  public function setReceptionUriProd() {
    $this->RECEPCION_URI = 'https://api.comprobanteselectronicos.go.cr/recepcion/v1';
  }

  /**
   * Change the default value from staging to production enviroment
   */
  public function setIdpClientIDProd() {
    $this->IDP_CLIENT_ID = 'api-prod';
  }

  /**
   * Change the default value from staging to production enviroment
   */
  public function setIdpUriProd() {
    $this->IDP_URI = 'https://idp.comprobanteselectronicos.go.cr/auth/realms/rut/protocol/openid-connect';
  }

  public function getConsecutive() {
    return $this->consecutive;
  }

  /**
    @return string
    - Numerode identificacion del emisor para formar la clave
   * */
  public function getIdEmisor() {
    return $this->idEmisor;
  }

  /**
    @return string
    - Codigo de la situacion del comprobante para formar la clave
   * */
  public function getSituacionComprobante() {
    return $this->situacionComprobante;
  }

  /**
    @return string
    - Codigo de seguridad para formar la clave
   * */
  public function getCodSeguridad() {
    return $this->codSeguridad;
  }

  /**
    @return string
    - Numero de clave del comprobante electronico
   * */
  public function getClave() {
    return $this->clave;
  }

  /**
    @return string
    - valor del tipo de medio de pago
   * */
  public function getVarMedioPago() {
    return $this->varMedioPago;
  }

  /**
    @return string
    - Contraseña del usuario de ATV
   * */
  function getPass() {
    return $this->password;
  }

  /**
    @return string
    - Contraseña del usuario de ATV
   * */
  function setPass($pass) {
    $this->password = $pass;
  }

  /**
    @return string
    - URL de API
   * */
  function getIdpUri() {
    return $this->IDP_URI;
  }

  function getReceptionUri() {
    return $this->RECEPCION_URI;
  }

  /**
    @return string
    - Cliente de API
   * */
  function getIdpClient() {
    return $this->IDP_CLIENT_ID;
  }

  public function getXmlPathSigned() {
    return $this->xmlPathSigned;
  }

  /**
    @return string
    - Usuario de ATV
   * */
  function getUser() {
    return $this->usuario;
  }

  /**
    @return string
    - Usuario de ATV
   * */
  function setUser($usuario) {
    $this->usuario = $usuario;
  }

  /**
    @return string
    - Pin de segurirdad de la llavecriptografica
   * */
  function getPin() {
    return $this->pin;
  }

  /**
    @return string
    - Fecha de creacion de la factura
   * */
  function getFecha() {
    return $this->fecha;
  }

  function setFecha($fecha) {
    $this->fecha = $fecha;
  }

  /**
    @return string
    - Generacion del access token con el API de MH
    - Acces Token es necesario para el envio o consulta al MH
   * */
  public function tokenAuth() {
    $url = $this->getIdpUri() . '/token'; //access token url
    $data = array('client_id' => $this->getIdpClient(),
      'client_secret' => '', //always empty
      'grant_type' => 'password', //always 'password'
      //go to https://www.hacienda.go.cr/ATV/login.aspx to generate a username and password credentials
      'username' => $this->getUser(),
      'password' => $this->getPass(),
      'scope' => ''); //always empty
    // use key 'http' even if you send the request to https://...
    $options = array(
      'http' => array(
        'header' => "Content-type: application/x-www-form-urlencoded\r\n",
        'method' => 'POST',
        'content' => http_build_query($data)
      )
    );
    $context = stream_context_create($options);
    $result = @file_get_contents($url, false, $context);
    if ($result === FALSE) {
      return $result;
    }
    $token = json_decode($result); //get a token object
    $objtoken = get_object_vars($token);
    $access = $objtoken['access_token'];
    return $access;
  }

  /**
    @param string $token
    @param string $namefact
    @return array
    - Encriptacion, Preparacion formato Json y Envio de la factura al MH y recepcion de la respuesta
    - DATOS DEL ENVIO DE LA FACTURA AL MH
    $estadoAPI= $arrayResp['0'];
    $estadoRecepcion= $arrayResp['2']; 200= aceptado 400= error
    $error=$response['17'];  si el estado de recepcion es 400 aparece el detalle del error, EJEMPLO:
    X-Error-Cause: El comprobante [50616071800310167282500100001020000000209122999922] ya fue recibido anteriormente.
    Sino el X-Error-Cause no aparece,en ves aparece: X-Ratelimit-Limit: 1000
    echo "\n Estado API: ".$estadoAPI;
    echo "\n Estado Recepcion: ".$estadoRecepcion;
    echo "\n Error: ".$error;
   * */
  public function sendInvoice($token, $namefact, $nota_credito = FALSE) {
    $invoicename = $this->xmlPathSigned . $namefact;
    if($nota_credito){
      $invoicename = $this->creditPath . $namefact;
      }
    
    $key = $this->getClave();
    $date = $this->getFecha();

    if (file_exists($invoicename)) {
      // Get file xml signed.
      $comprobante = file_get_contents($invoicename);
      // Encode from base64
      $comprobanteEncode = base64_encode($comprobante);

      $data = array(
        "clave" => $key,
        "fecha" => $date,
        "emisor" => array(
          "tipoIdentificacion" => $this->getTypeEmisor(),
          "numeroIdentificacion" => $this->getIdEmisor()
        ),
        "receptor" => array(
          "tipoIdentificacion" => $this->getTypeReceptor(),
          "numeroIdentificacion" => $this->getIdReceptor()
        ),
        "comprobanteXml" => $comprobanteEncode
      );
      $datajson = json_encode($data);

      try {
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => $this->getReceptionUri() . "/recepcion",
          CURLOPT_RETURNTRANSFER => TRUE,
          CURLOPT_HEADER => TRUE,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_HTTPHEADER => array(
            "Authorization: Bearer " . $token,
            "Content-Type: application/json",
            "Cache-Control: no-cache",
          ),
          CURLOPT_POSTFIELDS => $datajson,
        ));

        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        
        if ($status == 400) {
          preg_match_all("~X-Error-Cause:([^#]+)x-amz-apigw-id~", $response, $result);          
          return preg_replace('/[^A-Za-z0-9\-]/', ' ', $result[1][0]);
        }
        elseif ($status == 202) {
          $response = $status;
        }
      }
      catch (\Exception $e) {
        
      }

      return $response;
    }
  }

  public function sendMensajeReceptor($token, $namefact) {
    $invoicename = $this->acceptPath . $namefact;
    
    $key = $this->getClave();
    $date = $this->getFecha();

    if (file_exists($invoicename)) {
      // Get file xml signed.
      $comprobante = file_get_contents($invoicename);
      // Encode from base64
      $comprobanteEncode = base64_encode($comprobante);

      $data = array(
        "clave" => $key,
        "fecha" => $date,
        "emisor" => array(
          "tipoIdentificacion" => $this->getTypeEmisor(),
          "numeroIdentificacion" => $this->getIdEmisor()
        ),
        "receptor" => array(
          "tipoIdentificacion" => $this->getTypeReceptor(),
          "numeroIdentificacion" => $this->getIdReceptor()
        ),
        "consecutivoReceptor" => $this->getConsecutive(),
        "comprobanteXml" => $comprobanteEncode
      );
      
      $datajson = json_encode($data);

      try {
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => $this->getReceptionUri() . "/recepcion",
          CURLOPT_RETURNTRANSFER => TRUE,
          CURLOPT_HEADER => TRUE,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_HTTPHEADER => array(
            "Authorization: Bearer " . $token,
            "Content-Type: application/json",
            "Cache-Control: no-cache",
          ),
          CURLOPT_POSTFIELDS => $datajson,
        ));

        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);
        
        if ($status == 400) {
          preg_match_all("~X-Error-Cause:([^#]+)X-Ratelimit-Limit~", $response, $result);          
          return preg_replace('/[^A-Za-z0-9\-]/', ' ', $result[1][0]);
        }
        elseif ($status == 202) {
          $response = $status;
        }
      }
      catch (\Exception $e) {
        
      }

      return $response;
    }
  }
  /**
   * 
   * @param <string> $token
   * @return 
   *   - bool, true if the connection was close properly
   */
  public function closeConnection($token) {
    $url = $this->getIdpUri() . '/logout'; //access token url
    $data = http_build_query(
        array('client_id' => $this->getIdpClient(),
          'refresh_token' => $token)
    );

    try {
      $curl = curl_init();
      curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_HTTPHEADER => array(
          "Content-Type: application/x-www-form-urlencoded; charset=utf-8",
          "Cache-Control: no-cache",
        ),
        CURLOPT_POSTFIELDS => $data,
      ));

      curl_exec($curl);
      $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
      curl_close($curl);
    }
    catch (\Exception $e) {
      
    }

    return (is_numeric($status) && $status == 204) ? TRUE : FALSE;
  }

  public function consultar($token) {
    $curl = curl_init();
    $clave = $this->getClave();
    if ($clave == "" && strlen($clave) == 0) {
      return "La clave no puede ser en blanco";
    }

    $url = $this->getReceptionUri() . "/recepcion/{$clave}";
    
    curl_setopt_array($curl, array(
      CURLOPT_URL => $url,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => "",
      CURLOPT_HEADER => CURL_HTTP_VERSION_1_1,
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 30,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "GET",
      CURLOPT_HTTPHEADER => array(
        "Authorization: Bearer " . $token,
        "Cache-Control: no-cache",
        "Content-Type: application/x-www-form-urlencoded"
      ),
    ));

    $response = explode("\n", curl_exec($curl));
    
    foreach ($response as $value) {
      $json = json_decode($value, true);
      if (json_last_error() == JSON_ERROR_NONE) {
        $j_response = $json;
        break;
      }
    }
    
    $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);
    $xml = '';    
    if ($status !== 200) {
      return array(
        'mensaje' => 'Error Revisando Estado'
      );
    }
    
    if (is_array($j_response)) {
      $mensaje = $j_response['ind-estado'];
      $xml = $j_response['respuesta-xml'];
    }
    else {
      $mensaje = 'Error comunicando con hacienda';
    }

    return array(
      'mensaje' => $mensaje,
      'xml' => $xml
    );
  }

}
