<?php

use Spipu\Html2Pdf\Html2Pdf;
define('FE_FILES', 'fefiles');
define('FE_PDF', 'fepdf');
define('XML_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/xml/');
define('XML_SIGNED_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/xml_signed/');
define('CERTS_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/certs/');
define('QR_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/qr/');
define('CONFIRM_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/xml_confirm/');
define('CREDIT_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/xml_nota_credito/');
define('ACCEPTED_PATH', WP_CONTENT_DIR . '/uploads/' . FE_FILES . '/xml_accept/');
define('PDF_REPORT', WP_CONTENT_DIR . '/uploads/' . FE_PDF . '/pdf_report/');

/**
 * Check if WooCommerce is active
 * */
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
  if (!class_exists('WC_Efactura')) {
    /**
     * Localisation
     * */
    load_plugin_textdomain('wc_efactura', false, dirname(plugin_basename(__FILE__)) . '/');

    class WC_Efactura {

      public function __construct() {
        // called only after woocommerce has finished loading
        add_action('woocommerce_init', array(&$this, 'woocommerce_loaded'));

        add_filter('http_request_timeout', function($timeout) {
          return 40;
        });

        // called after all plugins have loaded
        add_action('plugins_loaded', array(&$this, 'plugins_loaded'));

        // called just before the woocommerce template functions are included
        add_action('init', array(&$this, 'set_setting'), 20);

        //hook to add metaboxes to an order
        add_action('add_meta_boxes', __CLASS__ . '::efactura_add_meta_boxes');
        //action to save post information
        add_action('save_post', __CLASS__ . '::save_efactura_fields');
        // hook to add extra field in billing information
        add_filter('woocommerce_billing_fields', __CLASS__ . '::custom_woocommerce_billing_fields');

        // Hook thankyou page..
        add_action('woocommerce_thankyou', __CLASS__ . '::save_efactura_component', 10, 1);
        add_action('rest_pre_echo_response', __CLASS__ . '::rest_api_procesar_orden', 10, 1);
        add_action('woocommerce_admin_field_file', __CLASS__ . '::add_admin_field_file');
        add_action('upload_mimes', __CLASS__ . '::efactura_mime_types');

        add_action('admin_enqueue_scripts', __CLASS__ . '::admin_scripts');

        add_action('wp_enqueue_scripts', __CLASS__ . '::fe_scripts');

        add_action('save_post', __CLASS__ . '::fe_save_order_details', 10, 1);

        add_action('woocommerce_admin_order_data_after_billing_address', __CLASS__ . '::fe_checkout_field_display_admin_order_meta', 10, 1);

        add_filter('woocommerce_email_attachments', __CLASS__ . '::attach_hacienda_to_mail', 99, 3);

        add_action('template_redirect', __CLASS__ . '::fe_download_file_manager');

        add_filter('wpo_wcpdf_template_paths', __CLASS__ . '::fe_wpo_wcpdf_template_paths', 10, 1);

        add_action('woocommerce_after_checkout_validation', __CLASS__ . '::fe_checkout_validate', 20, 2);

        // indicates we are running the admin
        if (is_admin()) {
          add_action('wp_ajax_enviar_nota_credito', __CLASS__ . '::ajax_callback_enviar_nota_credito');
          add_action('wp_ajax_firmar_mensaje_hacienda', __CLASS__ . '::ajax_callback_firmar_mensaje_hacienda');
          add_action('wp_ajax_enviar_hacienda', __CLASS__ . '::ajax_callback_enviar_hacienda');
          add_action('wp_ajax_procesar_orden', __CLASS__ . '::ajax_callback_procesar_orden');
          add_action('wp_ajax_cargar_actividades', __CLASS__ . '::ajax_callback_cargar_actividades');
          add_action('wp_ajax_consultar_reporte', __CLASS__ . '::ajax_callback_consultar_reporte');
          add_action('wp_ajax_obtener_codigo_catalogo', __CLASS__ . '::ajax_callback_obtener_codigo_catalogo');

          add_filter('upload_dir', __CLASS__ . '::llave_criptografica_upload_dir');


        }

        // indicates we are being served over ssl
        if (is_ssl()) {
          // ...
        }

        self::includes();
      }

      //validates dynamically the fields when EI
      static function fe_checkout_validate($data, $errors) {
        if ($data['tipofelectronico'] == '01') {
          if(!$data['fenumerocedula']){
            wc_add_notice(__("Número de cédula es un campo requerido."), 'error');
            
          }
        }
      }

      static function fe_checkout_field_display_admin_order_meta($order) {

        $ticket = !get_post_meta($order->get_id(), '_billing_is_fe', true);
        $option = get_post_meta($order->get_id(), '_billing_id_type', true);
        $number = get_post_meta($order->get_id(), '_billing_id_number', true);
        $doc_electronico = get_post_meta($order->get_id(), '_billing_type_doc', true);
        $tipos_doc_electronicos = array(
          '04' => __('Tíquete Electrónico', 'woocommerce'),
          '01' => __('Factura Electrónica', 'woocommerce'),
          '09' => __('Factura Exportación', 'woocommerce'),
          '08' => __('Factura de compra',   'woocommerce'),
          );
        echo '<br><p><strong class="form-field form-field-wide">' . __('Tipo de documento electrónico') . ':</strong></p><p><select name="tipofelectronico" id="tipo-fe-lectronico">';
          foreach ($tipos_doc_electronicos as $id => $text) {
            $default = $id == $doc_electronico ? 'selected="selected"' : '';
            echo '<option value="' . $id . '"' . " {$default}>" . $text . '</option>';
          }
        echo '</select></p>';
        $values = array(
          '01' => __('Cédula Física', 'woocommerce'),
          '02' => __('Cédula Jurídica', 'woocommerce'),
          '03' => __('DIMEX', 'woocommerce'),
          '04' => __('NITE', 'woocommerce'),
          '05' => __('Cliente extranjero', 'woocommerce'),
        );

        echo '<article class="factura-electronica"><p><strong>' . __('Tipo de identificación') . ':</strong></p><p><select name="fetipocedula">';
        foreach ($values as $id => $text) {
          $default = $id == $option ? 'selected="selected"' : '';
          echo '<option value="' . $id . '"' . " {$default}>" . $text . '</option>';
        }
        echo '</select></p>';
        echo '<p><strong>' . __('Número de Identificación') . ':</strong></p><p><input type="text" name="fenumerocedula" id="fenumerocedula" value="' . get_post_meta($order->get_id(), '_billing_id_number', true) . '"></p></article>';
        echo '<article class="article-actividad-economica">';
        echo '<p><strong>' . __('Actividad económica') . ':</strong></p><p><select name="feCompra" id="fe-compra">';
        echo '<option value="0">'."Selecione".'</option>';
        echo '</select>&nbsp;<button class="button-primary" id="cargar-actividades">Cargar Actividades</button></p>';
        echo '<p><strong>' . __('Código de actividad seleccionado') . ':</strong><span class="codigo-actividad-selected">'.get_post_meta($order->get_id(), '_billing_cod_actividad', true).'</span></p> ';
        echo '</article>';
        if($doc_electronico == '01' ) {

          echo '<p><strong>' . __('Procesado como Factura electrónica.') . '</strong></p>';
        }
        if($doc_electronico == '08' ) {

          echo '<p><strong>' . __('Procesado como Factura de compra electrónica.') . '</strong></p>';
        }
        if($doc_electronico == '04' ) {

          echo '<p><strong>' . __('Procesado como tíquete Electrónico') . '</strong></p>';
        }
        if ($doc_electronico == '09' ) {

          echo '<p><strong>' . __('Procesado como factura de exportación') . '</strong></p>';
        }
        echo '<p><strong>' . __('Base imponible') . ':</strong></p><p><input type="text" name="feBaseImponible" id="feBaseImponible" value="' . get_post_meta($order->get_id(), '_billing_base_imponible', true) . '"></p>';

      }

      static function fe_save_order_details($post_id) {
        // We need to verify this with the proper authorization (security stuff).
        // Check if our nonce is set.

        $fetipocedula = $_REQUEST['fetipocedula'] != null ? $_REQUEST['fetipocedula'] : get_post_meta($post_id, '_billing_id_type', true);
        $fenumerocedula = $_REQUEST['fenumerocedula'] != null ? $_REQUEST['fenumerocedula'] : get_post_meta($post_id, '_billing_id_number', true);
        $tipofelectronico = $_REQUEST['tipofelectronico'] != null ? $_REQUEST['tipofelectronico'] : get_post_meta($post_id, '_billing_type_doc', true);
        $base_imponible = $_REQUEST['feBaseImponible'] != null ? $_REQUEST['feBaseImponible'] : get_post_meta($post_id, '_billing_base_imponible', true);      
        //se elimino codigo ya que cuando entraba a crear la orden y existia una cedula vacia, creaba la orden como unt tiquete electronico y entonces si iba a pasar cada ves que se recargara por el mismo motivo de la cedula que es vacia.
        //get codigo de actividad selecionado
        if (get_post_meta($post_id,'_billing_cod_actividad', true)) {
          //si existe lo consulta y lo guarda el codigo seleccionado.
          $codigo_actividad = get_post_meta($post_id,'_billing_cod_actividad', true);

        }
        else {

          $codigo_actividad = $_REQUEST['feCompra'] != null ? $_REQUEST['feCompra'] : get_post_meta($post_id, '_billing_cod_actividad', true);
        }
        if (empty($fenumerocedula)) {
          $tipofelectronico = "04";
        }
        if ($codigo_actividad == '0') $codigo_actividad = "";
        // se cambia de posicion ya que habia un bug cuando un usuario tiene rol de subcritor y hacia una compra no guardaba la cedula como debia de ser , existias retuns antes de setearlos.
        // Update the meta field in the database.
        update_post_meta($post_id, '_billing_is_fe', strlen($fenumerocedula));
        if(!empty($fetipocedula) && !empty($fenumerocedula)){
          update_post_meta($post_id, '_billing_id_type', $fetipocedula);
          update_post_meta($post_id, '_billing_id_number', $fenumerocedula);
        }
        if(!empty($tipofelectronico)){
          //setear el tipo de documento de la base de datos
          update_post_meta($post_id, '_billing_type_doc', $tipofelectronico);
        }
        //seter el codigo de actividad ala cual se le va asignar ala factura de compra.
        update_post_meta($post_id,'_billing_cod_actividad', $codigo_actividad);
        //setear la base imponible si algunos de los item fue seleccionado con el iva calculo especial.
        update_post_meta($post_id, '_billing_base_imponible', $base_imponible);
        
        // If this is an autosave, our form has not been submitted, so we don't want to do anything.
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
          return $post_id;
        }

        // Check the user's permissions.
        if ('page' == $_POST['post_type']) {

          if (!current_user_can('edit_page', $post_id)) {
            return $post_id;
          }
        }
        else {

          if (!current_user_can('edit_post', $post_id)) {
            return $post_id;
          }
        }

      }

      static function includes() {
        include_once plugin_dir_path(__FILE__) . 'class-hacienda-connect.php';
        include_once plugin_dir_path(__FILE__) . 'felectronica-enviar-hacienda.php';
        include_once plugin_dir_path(__FILE__) . 'efactura-placeholders.php';
        include_once plugin_dir_path(__FILE__) . 'phpqrcode/qrlib.php';
        include_once plugin_dir_path(__FILE__) . 'efactura-settings.php';
        include_once plugin_dir_path(__FILE__) . 'vendor/autoload.php';
        include_once plugin_dir_path(__FILE__) . 'vendor/spipu/html2pdf/src/Html2Pdf.php';

      }
      static function efactura_mime_types($mimes = array()) {

        $mimes['p12'] = "application/x-pkcs12";

        return $mimes;
      }

      static function llave_criptografica_upload_dir($dir) {
        if ('/9999/99' === $dir['subdir']) {
          return array(
            'path' => $dir['basedir'] . '/' . FE_FILES . '/certs',
            'url' => $dir['baseurl'] . '/' . FE_FILES . '/certs',
            'subdir' => '/' . FE_FILES . '/certs',
              ) + $dir;
        }

        return $dir;
      }

      static function errormsg($text = '') {
        $class = 'notice notice-error';
        $message = __($text);
        die(printf('<div class="%1$s"><p>%2$s</p></div>', $class, $message));
      }

      static function add_efactura_fields() {
        global $post;

        //load default behavior
        $nfile = file_exists(get_template_directory() . '/woocommerce/emails/fe-template-noinfo.php') ? get_template_directory() . '/woocommerce/emails/fe-template-noinfo.php' : dirname(WC_FE_FILE) . '/templates/fe-template-noinfo.php';
        ob_start();
        load_template($nfile, TRUE);
        $html = ob_get_contents();
        ob_end_clean();

        $meta_field_data = get_post_meta($post->ID, '_xml_signed_file', true);
        if (strlen($meta_field_data)) {
          $tfile = file_exists(get_template_directory() . '/woocommerce/emails/fe-template-invoice-link.php') ? get_template_directory() . '/woocommerce/emails/fe-template-invoice-link.php' : dirname(WC_FE_FILE) . '/templates/fe-template-invoice-link.php';
          ob_start();
          load_template($tfile, TRUE);
          $html = ob_get_contents();
          ob_end_clean();
        }

        echo $html;
      }

      static function save_efactura_fields($post_id) {
        // We need to verify this with the proper authorization (security stuff).
        // Check if our nonce is set.
        if (!isset($_POST['mv_other_meta_field_nonce'])) {
          return $post_id;
        }
        $nonce = $_REQUEST['mv_other_meta_field_nonce'];

        //Verify that the nonce is valid.
        if (!wp_verify_nonce($nonce)) {
          return $post_id;
        }


        // If this is an autosave, our form has not been submitted, so we don't want to do anything.
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
          return $post_id;
        }

        // Check the user's permissions.
        if ('page' == $_POST['post_type']) {

          if (!current_user_can('edit_page', $post_id)) {
            return $post_id;
          }
        }
        else {

          if (!current_user_can('edit_post', $post_id)) {
            return $post_id;
          }
        }
      }

      static function efactura_add_meta_boxes() {
        add_meta_box('mv_other_fields', __('INFORMACION DE HACIENDA', 'woocommerce'), __CLASS__ . '::add_efactura_fields', 'shop_order', 'side', 'core');
      }

      /**
       * Take care of anything that needs woocommerce to be loaded.
       * For instance, if you need access to the $woocommerce global
       */
      static function woocommerce_loaded() {
        // ...
      }

      /**
       * Take care of anything that needs all plugins to be loaded
       */
      static function plugins_loaded() {
        // ...
      }

      /**
       * Override any of the template functions from woocommerce/woocommerce-template.php
       * with our own template functions file
       */
      static function set_setting() {
        register_setting(
            'order_consecutive_group', 'order_consecutive'
        );

        new WP_Enviar_Hacienda();
        new WC_Fe_Placeholders();
      }

      static function fe_download_file_manager() {
        global $wp_query;
        //xml_download        
        if (isset($wp_query->query_vars['pagename']) && $wp_query->query_vars['pagename'] == 'fe-order' && $hash = $_GET['hash']) {
          $order_id = $wp_query->query_vars['page'];
          if ($url_name = get_post_meta($order_id, '_xml_signed_file', true)) {
            if (strstr($url_name, $hash)) {
              header('Content-disposition: attachment; filename="Factura.xml"');
              header('Content-type: "text/xml"; charset="utf8"', true, 200);
              readfile(XML_SIGNED_PATH . $url_name);
              exit;
            }
          }
        }

        if (isset($wp_query->query_vars['pagename']) && $wp_query->query_vars['pagename'] == 'fe-confirmacion' && $hash = $_GET['hash']) {
          $order_id = $wp_query->query_vars['page'];
          if ($url_name = get_post_meta($order_id, '_xml_signed_file', true)) {
            if (strstr($url_name, $hash)) {
              header('Content-disposition: attachment; filename="Confirmacion.xml"');
              header('Content-type: "text/xml"; charset="utf8"', true, 200);
              readfile(CONFIRM_PATH . $url_name);
              exit;
            }
          }
        }

        if (isset($wp_query->query_vars['pagename']) && $wp_query->query_vars['pagename'] == 'fe-nota-credito' && $hash = $_GET['hash']) {
          $order_id = $wp_query->query_vars['page'];
          if ($url_name = get_post_meta($order_id, '_xml_signed_file', true)) {
            if (strstr($url_name, $hash)) {
              header('Content-disposition: attachment; filename="nota-credito.xml"');
              header('Content-type: "text/xml"; charset="utf8"', true, 200);
              readfile(CREDIT_PATH . $url_name);
              exit;
            }
          }
        }
      }

      //we use the same name of the file every time, so wordpress by default will
      //add a suffix number for repited names, but we only want one name.
      static function fe_unique_name($dir, $name, $ext) {
        return 'cert.p12';
      }

      static function add_admin_field_file($value) {
        $option_value = (array) WC_Admin_Settings::get_option($value['id']);
        $description = WC_Admin_Settings::get_field_description($value);

        echo '<tr valign="top">
		            <th scope="row" class="titledesc">
		                <label for="' . esc_attr($value['id']) . '">' . esc_html($value['title']) . '</label>' .
        $description['tooltip_html'] . '
		            </th>
		            <td class="forminp forminp-' . sanitize_title($value['type']) . '">
		                 <input
		                        name = "' . esc_attr($value['name']) . '"
		                        id   ="' . esc_attr($value['id']) . '"
		                        type ="file"
		                        style="' . esc_attr($value['css']) . '"
		                        value=""
		                        class="' . esc_attr($value['class']) . '"
		                /> ' .
        $description['description'] . '
		            </td>
		        </tr>';
      }

      /**
       * Hook add more fields to billing form.
       */
      static function custom_woocommerce_billing_fields($fields) {
         $fields['tipofelectronico'] = array(
          'label' => __('Tipo de documento electrónico', 'woocommerce-settings-tab-efactura'),
          'type' => 'select',
          'required' => true,
          'options' => array(
            '04' => __('Tíquete Electrónico', 'woocommerce'),
            '01' => __('Factura Electrónica', 'woocommerce'),
            '09' => __('Factura Exportación', 'woocommerce'),
          ),
          'class' => array('form-fe-dependent'),
          'priority' => 1
        );

        $fields['fetipocedula'] = array(
          'label' => __('Tipo de identificación', 'woocommerce-settings-tab-efactura'),
          'type' => 'select',
          'required' => true,
          'options' => array(
            '01' => __('Cédula Física', 'woocommerce'),
            '02' => __('Cédula Jurídica', 'woocommerce'),
            '03' => __('DIMEX', 'woocommerce'),
            '04' => __('NITE', 'woocommerce'),
            '05' => __('Cliente extranjero', 'woocommerce'),

          ),
          'class' => array('form-fe-dependent-field'),
          'priority' => 2
        );

        $fields['fenumerocedula'] = array(
          'label' => __('Número de Identificación', 'woocommerce'), // Add custom field label
          'placeholder' => _x('Identificación', 'placeholder', 'woocommerce'), // Add custom field placeholder
          'clear' => false, // add clear or not
          'type' => 'text', // add field type
          'class' => array('form-fe-dependent-field  validate-required woocommerce-invalid woocommerce-invalid-required-field'),
          'priority' => 2
        );
        return $fields;
      }

      /**
       * Save the efactura component.
       */
      static function save_efactura_component($order_id, $ajax = FALSE, $nota_credito = FALSE) {
        if (!$order_id) {
          return;
        }
        $download_url = '';
        $download_link = '<div class="download-invoice">' . __('Descargar Factura Electronica: ' . "<a href='%s' target='_blank'>aqui</a>") . '</div>';
        $url_name = get_post_meta($order_id, '_xml_signed_file', true);
        if (strlen($url_name) && !$ajax) {
          $download_url = self::getSignedFileUrl($order_id, $url_name);
          echo sprintf($download_link, $download_url);
          return;
        }
        $econnect_data = self::get_connection_data();
        if ($econnect_data['cod_actividad'] == null) {
          $message = __('Código de actividad requerido, vaya a hacienda settings y configure su código de actividad comercial.');
          self::notify_email($message, $order_id );
          return $message;
        }

        // Getting an instance of the order object
        $order = wc_get_order($order_id);
        if (!is_null($order) && !$order instanceof WC_Order) {
         return __('La order no pertenece a la instancia de woocommerce llamada WC_Order. Contacte a soporte tecnico');
        }
        //total de impuesto
        $monto_impuesto = 0;
        //total de linea 
        $monto_linea = 0;
        //total de servicios gravados
        $total_serv_gravados = 0;
        //total de servicios exentos
        $total_serv_exentos = 0;
        //total de mercancias gravadas.
        $total_merc_gravadas = 0;
        //total de mercancias exentas.
        $total_merc_exentas = 0;
        //total exento 
        $total_exento = 0;
        //total amount of the order to be taxed
        $total_taxed = 0;
        //descuento
        $descuento = false;
        $tiquet = false;
        $exportacion  = false;
        $compra = false;
        //base imponible
        $base_imponible = get_post_meta($order->get_id(), '_billing_base_imponible', true);
        //is tiquet
        $tipo_doc = get_post_meta($order->get_id(), '_billing_type_doc', true);
        if ($tipo_doc == '04') {
          $tiquet = true;
        }
        else  if ($tipo_doc == '09') {
          $exportacion = true;
        }
        else if ($tipo_doc == '08') {
          $compra = true;
        }
        //calculate dollar exchange
        $exchange = get_option('wc_settings_tab_efactura_dollar_custom');
        $tipo = get_option('wc_settings_tab_efactura_tipo_cambio');
        if ('texto' !== $tipo) {
          $fecha = date('d/m/Y');
          $dollar = self::tipo_cambio($fecha);
        }

        $order_data = array('order_id' => $order_id);

        // Get Receiver data.
        $order_data['receiver_data'] = self::getReceiverData($order);
        if (isset($order_data['receiver_data']['error']) && !$tiquet && !$exportacion) {
          return $order_data['receiver_data']['message'];
        }

        //validacion del codigo postal
        if(strlen($order_data['receiver_data']['postcode']) != 5 && !$tiquet && !$exportacion) {
          $message = __('El formato del codigó postal debe de cumplir con los requisitos de hacienda con un formato de 5 dígitos.');
          self::notify_email($message, $order_id );
          return $message;
        }

        //validaciones de cedulas por tipo.
        if ($order_data['receiver_data']['id_type'] == "01") {
          if (strlen(str_replace("-","",$order_data['receiver_data']['id'])) != 9) {
            $message = __('El formato de la cédula para el tipo fisica es incorrecto. Comuniquese con el cliente.');
            self::notify_email($message, $order_id );
            return $message;
          }
        }

        if ($order_data['receiver_data']['id_type'] == "02") {
          if (strlen(str_replace("-","",$order_data['receiver_data']['id'])) != 10) {
            $message = __('El formato de la cédula para el tipo jurídica es incorrecto. Comuniquese con el cliente.');
            self::notify_email($message, $order_id );
            return $message;
          }
        }

        if ($order_data['receiver_data']['id_type'] == "03") {
          if (strlen(str_replace("-","",$order_data['receiver_data']['id'])) != 11 && strlen(str_replace("-","",$order_data['receiver_data']['id'])) != 12){
            $message = __('El formato de la cédula para el tipo DIMEX es incorrecto. Comuniquese con el cliente.');
            self::notify_email($message, $order_id );
            return $message;
          }       
        }

        if ($order_data['receiver_data']['id_type'] == "04") {
          if (strlen(str_replace("-","",$order_data['receiver_data']['id'])) != 10) {
            $message = __('El formato de la cédula para el tipo NITE es incorrecto. Comuniquese con el cliente.');
            self::notify_email($message, $order_id );
            return $message;
          }
        }

        //validacion del numero de telefono ,formato de 8 digitos, puede ser con guiones para efectos del plugin de correos de costa rica, backend lo formatea.
        //se estaba dando un error cuando se procesa el documento como ticket, estaba entrando forzado a esta validacion.
        if ( strlen(str_replace("-","",$order_data['receiver_data']['phone'])) != 8 && !$tiquet && !$exportacion) {
          $message = __('El formato del teléfono debe de cumplir con los requisitos de hacienda con un formato de 8 dígitos sin guiones.');
          self::notify_email($message, $order_id );
          return $message;
        }
        if ( $order_data['receiver_data']['id_type'] == '05' ) {
          $tiquet = true;
        }

        // Get Transmitter data.
        $order_data['transmitter_data'] = self::getTransmitterData();
        if (isset($order_data['transmitter_data']['error']) && $compra  == true) {
          return $order_data['transmitter_data']['message'];
        }
        //validar si es compra que es necesario el codigo de actividad.
        if ($compra && !get_post_meta($order_id,"_billing_cod_actividad",true)) {
          $message = __('El código de actividad es requerido para la factura de compra');
          self::notify_email($message, $order_id );
          return $message;
        }
        // iterating through each order items (getting product ID and the product object)
        // (work for simple and variable products)

        $products = array();
        foreach ($order->get_items() as $item_id => $item) {
          //unidad de medida siempre va ser UNID
          $unidad_medida = 'Unid';
          //codigo del impuesto
          $codigo_impuesto = '01';
          $partida_arancelaria = "";
          if ($item['variation_id'] > 0) {
            $product_id = $item['variation_id']; // variable product
          }
          else {
            $product_id = $item['product_id']; // simple product
          }
          //si pose una atributo extra con partida o partida arancelaria
          $product = wc_get_product($item['product_id']);
          //factura de exportacion.
          if ($exportacion) {
            $atributes_partida_arancelaria = $product->get_attributes();
            if (isset($atributes_partida_arancelaria)) {
              if (isset($atributes_partida_arancelaria['partida-arancelaria'])) {
                $partida_arancelaria = $atributes_partida_arancelaria['partida-arancelaria']['data']['options'][0];
                //validar que la partida arancelaria tengo la cantidad de 12 digitos.
                if (strlen($partida_arancelaria) != 12) {
                  $message = __("La partida arancelaria debe de contener 12 digítos.");
                  self::notify_email($message, $order_id );
                  return $message;
                }
              }
              else {
                //si no trae partida arancelaria lo convertimos en servicios sujeto alas factura de exportacion
                $unidad_medida = 'Sp';
              }
            }
          }
          //validar si en las campos personalizados de cada producto existe una unidade de medida para Servicios y en general.
          $unid_attribute = '';
          $unid_attribute = $product->get_attributes();

          if (isset($unid_attribute)) {
            if (isset($unid_attribute['unidad-medida'])) {
              $unidad_medida = $unid_attribute['unidad-medida']['data']['options'][0];
            }
          }
          $has_tax = (bool) $order->get_line_tax($item);
          // Get the product object
          $flag_cod = false;
          $tax_class = $order->get_items_tax_classes();
         
          //valida si existe un impuesto 
          if ($has_tax || $item['tax_class'] != "" ) {
            if ( $item->get_subtotal_tax() == 0 ) {
              //validacion si existe una class tax pero no un total de impuesto en la linea cal.
              if ( $unidad_medida == 'Sp' or $unidad_medida == 'Spe' or $unidad_medida == 'St') {
                //servicios profesionales.
                $total_serv_exentos += $order->get_line_subtotal($item);
              }
              else {
                $total_merc_exentas += $order->get_line_subtotal($item);
              }    
            } 
            elseif($item['tax_class'] == 'impuesto-selectivo-consumo'){
              $codigo_impuesto = '02';  
              if ( $exportacion ) {
                if ( $unidad_medida == 'Unid' ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                elseif ( $order->get_line_tax($item) != 0 ){
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }
            }
            elseif ($item['tax_class'] == 'impuesto-unico-combustibles') {
              $codigo_impuesto = '03';
              if ( $exportacion ) {
                //cuando es exportacion , hacienda recibe a exportacion con los servicios o mercancias 
                if ( $unidad_medida == 'Unid' && $order->get_line_tax($item) != 0 ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                elseif ( $order->get_line_tax($item) != 0 ){
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }  
            } 
            elseif ($item['tax_class'] == 'impuesto-especifico-bebidas-alcoholicas') {
              $codigo_impuesto = '04';
              if ( $exportacion ) {
                if ( $unidad_medida == 'Unid' && $order->get_line_tax($item) != 0 ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                elseif ( $order->get_line_tax($item) != 0 ) {
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }
            }
            elseif ($item['tax_class'] == 'impuesto-especifico-bebidas-sin-alcohol-y-jabones-tocador') {
              $codigo_impuesto = '05';
              if ($exportacion) {
                if ( $unidad_medida == 'Unid' && $order->get_line_tax($item) != 0 ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                elseif ( $order->get_line_tax($item) != 0 ){
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }
            }
            elseif ($item['tax_class'] == 'impuesto-a-los-productos-de-tabaco') {
              $codigo_impuesto = '06';
              if ( $exportacion ) {
                if ( $unidad_medida == 'Unid' && $order->get_line_tax($item) != 0 ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                else if ( $order->get_line_tax($item) != 0 ) {
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }
            }
            elseif($item['tax_class'] == 'otros'){
              $codigo_impuesto = '99';
              if ( $exportacion ) {
                if ( $unidad_medida == 'Unid' && $order->get_line_tax($item) != 0 ) {
                  $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
                else if ( $order->get_line_tax($item) != 0 ) {
                  $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;
                }
              }
            }
            else {
              //sino , es iva.
                $has_tax = true;
              //devuelve el tax_class de cada producto en codigo
              foreach (self::cargar_codigos_tarifas_impuestos() as $cod => $value) {
                foreach ($value as $key => $codigos) {
                  if ($item['tax_class'] == $codigos) {
                    $flag_cod = true;
                    $cod_tarifa = $key;
                    $class_tax = substr($codigos, 0, 11);
                    if ($class_tax == "iva-calculo") {
                      if ($base_imponible == '') {
                        $message = __("La base imponible es requerido para el IVA (calculo especial).");
                        self::notify_email($message, $order_id );
                        return $message;
                      }
                      $codigo_impuesto = "07";
  
                    } else if ($class_tax == "iva-regimen") {
  
                      $codigo_impuesto = "08";
                    }
                    break;
                  }
                }
              }
 
              if ( $unidad_medida == 'Sp' or $unidad_medida == 'Spe' or $unidad_medida == 'St' ) {
                $total_serv_gravados += $has_tax ? $order->get_line_subtotal($item) : 0;

              }
              else {
                $total_merc_gravadas += $has_tax ? $order->get_line_subtotal($item) : 0;

              }
            }
         }
          else {
            if ( $unidad_medida == 'Sp' or $unidad_medida == 'Spe' or $unidad_medida == 'St' ) {
              //servicios profesionales.
              $total_serv_exentos += $order->get_line_subtotal($item);
            
            }
            else {
              //Mercancias
              $total_merc_exentas += $order->get_line_subtotal($item);
            }
 
          }
          // valida que exista algun tipo de cupon.
          foreach ($order->get_items('coupon') as $_ => $coupon_item) {
            $descuento = true;
            $coupon = $coupon_item->get_data();
            $naturaleza_descuento = $coupon['code'];
          }
          //si son mas de un impuesto poner el texto descuento aplicados.
          if(count($order->get_items('coupon')) > 1){
            $naturaleza_descuento = __('Descuento Aplicados');
          }
          if ($codigo_impuesto == '07') {
            //calcula el monto del impuesto sobre la base imponible
            $monto_impuesto = ($base_imponible * round((($item->get_subtotal_tax() / $order->get_line_subtotal($item)) * 100)) / 100);
            //calcula el monto toal de linea sobre la base imponible
            $monto_linea = round($order->get_line_subtotal($item) + $monto_impuesto);
            //sumatoria de total a impuestos a pagar.
            $total_impuesto =  $total_impuesto + $monto_impuesto;
          } else {
            $monto_impuesto = $order->get_line_tax($item);
            $monto_linea = $order->get_line_subtotal($item, TRUE);
            $total_impuesto = $total_impuesto + $monto_impuesto;

          }
          $product = wc_get_product($product_id);
          $sku = $product->get_sku();
                    $cabys_codigo = "";
          

          $codigo = strlen($sku) ? $sku : $product_id;
          if (strlen($codigo) > 20) {
            $message = __("El código o sku del producto de ser menor o igual a 20 caracteres");
            self::notify_email($message, $order_id );
            return $message;
          }
          
          $number_decimal = wc_get_price_decimals();
          $sub_total = round($item->get_total(), $number_decimal);
          $montodesc = round(($item->get_subtotal() - $item->get_total()),$number_decimal);
          
          if($product->get_parent_id()){
            $parent_product = wc_get_product($product->get_parent_id());
            $cabys_codigo = $parent_product->get_meta( 'product_cabys_cod_field' );
          }else{
            $cabys_codigo = $product->get_meta( 'product_cabys_cod_field' );
           
          }
          
          $cabys_codigo_part = explode("-", $cabys_codigo);
          if(is_numeric($cabys_codigo_part[0])){
             $cabys_codigo = trim($cabys_codigo_part[0]);
          }else{
            foreach ($cabys_codigo_part as $value) {
              if(is_numeric($value)){
                $cabys_codigo = trim($value);
              }
            }
          }
          $impuesto_total = round($item->get_subtotal_tax() / $order->get_line_subtotal($item) * 100);
         
          if(!$cod_tarifa){
            $impuestos = [
              '0' => '01',
              '1' => '02',
              '2' => '03',
              '4' => '04',
              '8' => '07',
              '13' => '08'
            ];
            if($impuestos[$impuesto_total]){
              $flag_cod = TRUE;
              $cod_tarifa = $impuestos[$impuesto_total];
            }else{
              $cod_tarifa  = 01;
            }
              
          }
          $products[$item_id] = array(
            "tipocod" => "01", // Codigo del producto del vendedor
            "codigo_cabys" => $cabys_codigo,
            "codigo" => $codigo,
            "partida_arancelaria" => $partida_arancelaria,
            "cantidad" => $item->get_quantity(),
            "unimedida" => $unidad_medida,
            "detalle" => $product->get_title(), // detalle del producto.
            "preciounit" => $order->get_item_subtotal($item), // precio unitario.
            "montototal" => $order->get_line_subtotal($item), // monto total con descuento (preciounitario * cantidad),
            "montodesc" => $descuento ?  ( $montodesc ) : 0,
            "natdesc" => $descuento ? $naturaleza_descuento : "",
            "subtotal" => $sub_total, // monto total menos descuento
            "tipodoc" => '01',
            "numdoc" => "", //solo en caso de exoneracion
            "nominsti" => "", //solo en caso de exoneracion
            "fechaemision" => "", //solo en caso de exoneracion
            "montoimpexo" => "", //solo en caso de exoneracion
            "porccompra" => "", //solo en caso de exoneracion
            "codigoimpuesto" => $has_tax ? $codigo_impuesto : '', //impuesto de venta
            "codigoTarifa" =>  $cod_tarifa ,//tarifa de impuesto.
            "tarifaimpuesto" => $impuesto_total,
            "montoimpuesto" => $monto_impuesto,
            "montolinea" => $monto_linea,
            "base_imponible" => $base_imponible,
          );
    
        }
        //devuelve los shipping o envios.
        $line_items_shipping = $order->get_items('shipping');
        //crear items si existe uno o mas envios
        $monto_total_shipping = 0;
        foreach ($line_items_shipping as $item_id => $item) {
          //si existe lo recorre si no, no los crea.
          $shiping = $item->get_data(); // The Order data
          //se suma los totales del shipping + el total de impuesto si fuera necesario.
          $monto_total_shipping = $shiping['total'] + $shiping['total_tax'];
          //se crea item para los medios de envio cobrados.
          $products[$item_id] = array(
            "tipocod" => "01", // Codigo del producto del vendedor
            "codigo_cabys" => $cabys_codigo,
            "codigo" => $item_id,
            "cantidad" => 1,
            "unimedida" => 'Unid',
            "detalle" => $shiping['name'], // detalle del producto.
            "preciounit" => $monto_total_shipping, // precio unitario.
            "montototal" => $monto_total_shipping, // monto total con descuento (preciounitario * cantidad),
            "montodesc" => 0,
            "natdesc" => "",
            "subtotal" => $monto_total_shipping, // monto total menos descuento
            "tipodoc" => '01',
            "numdoc" => "", //solo en caso de exoneracion
            "nominsti" => "", //solo en caso de exoneracion
            "fechaemision" => "", //solo en caso de exoneracion
            "montoimpexo" => "", //solo en caso de exoneracion
            "porccompra" => "", //solo en caso de exoneracion
            "codigoimpuesto" => '', //impuesto de venta
            "codigoTarifa" => '', //tarifa de impuesto.
            "tarifaimpuesto" => '0',
            "montoimpuesto" => '0',
            "montolinea" => $monto_total_shipping,
          );
          //total del item de envio
          $total_merc_exentas += $monto_total_shipping;
        }
        if (count($products) === 0) {
          return 'Error, al menos un producto es necesario para firmar el documento.';
          
        }

        //consulta la orden
        $hacienda = get_post_meta($order->get_id(), '_xml_hacienda', true);        //SI EXISTE EN LA ORDEN NO EJECUTAR LA SIGUIENTE LINEA
        //valida si es una nota de credito
        if ($nota_credito == true) {
          //si el atributo nota_credito no se ha creado en la orden, se crea uno nuevo
          if (isset($hacienda['nota_credito'])) {
            //si la orden ya fue creada y por x motivo existio un error de conexion, asiga el consecutivo ya creado.
            $order_data['consecutive_number'] = $hacienda['nota_credito'];
          }
          else {
            $order_data['consecutive_number'] = self::generateConsecutive($order, $nota_credito);
          }
          //valida que  no es nota de credito y que el consecutivo no exista en la orden
        }
        elseif (isset($hacienda['consecutivo'])) {
          $order_data['consecutive_number'] = $hacienda['consecutivo'];
        }
        else {
          //genera un nuevo consecutivo del ticket o FE.
          $order_data['consecutive_number'] = self::generateConsecutive($order, FALSE, $tiquet, $exportacion, $compra);
        }
        //total de los servicios y mercancias gravadas
        $total_taxed = $total_merc_gravadas + $total_serv_gravados;
        //total de los servicios y mercancias exentas.
        $total_exento = $total_merc_exentas + $total_serv_exentos;

        $number_decimal = wc_get_price_decimals();
        $total_descuento = 0;
        $total_comprobante = 0;
        $total_venta_neta = 0;

        if($number_decimal == 0){
          if ($descuento) {
            $total_descuento = round($order->get_discount_total());
          }
          $total_venta_neta = round((($total_taxed + $total_exento) - $total_descuento));
          $total_comprobante = round(($total_taxed + $total_exento + $total_impuesto - $order->get_discount_total()));
        }
        else {
          if($descuento) {
            $total_descuento = $order->get_discount_total();
          }
          $total_venta_neta = (($total_taxed + $total_exento) - $total_descuento);
          $total_comprobante = ($total_taxed + $total_exento + $total_impuesto - $total_descuento);

        }
        $order_data['resumen_factura'] = array(
          "total_serv_gravados" => $total_serv_gravados, //si hablamos de productos o servicios, esto es un servicio
          "total_serv_exentos" => $total_serv_exentos,
          "total_mercancias_gravadas" => $total_merc_gravadas, //si hablamos de productos o servicios, esto es un producto
          "total_mercancias_exentas" => $total_merc_exentas,
          "total_gravado" => $total_taxed, //total de servicios y mercancias gravadas
          "total_excento" => $total_exento, //total de servicio0s y mercancias exentas
          "total_venta" => (float) ($total_taxed + $total_exento), //total_gravado + total_excento
          "total_descuentos" => $total_descuento,
          "total_venta_neta" => $total_venta_neta,//total_venta - total_descuentos
          "total_impuesto" => $total_impuesto,
          "total_comprobante" => $total_comprobante, //total_venta_neta+total_impuesto
          "codigo_moneda" => get_woocommerce_currency(),
          "tipo_cambio" => 'texto' === $tipo ? $exchange : $dollar[$tipo],
        );


        $order_data['tipo_comprobante'] = $tipo_doc;
        $order_data['products'] = $products;
        $order_data['license'] = woocommerce_factura_electronica_get_option();
        $order_data['domain'] = strstr($_SERVER['SERVER_SOFTWARE'], 'nginx') ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
        $order_data['nota_credito'] = $nota_credito;
        $order_data['ticket'] = $tiquet;
        $order_data['exportacion'] = $exportacion;
        $order_data['compra'] = $compra;
        $order_data['order_timestamp'] = $order->get_date_created()->getTimestamp();
        $order_data['flag_wp'] = true;
        if ($compra) {
          $order_data['cod_actividad'] = get_post_meta($order_id, '_billing_cod_actividad', true);
        }
        else {
          $order_data['cod_actividad'] = $econnect_data['cod_actividad'];
        }
        if ($nota_credito) {
          $order_data['tipo_comprobante_nc'] = $hacienda['tipo_comprobante'];
          $order_data['clave_nc'] = $hacienda['clave'];
          $order_data['clave_nc'] = $hacienda['clave'];
          $order_data['fecha_nc'] = $hacienda['fecha'];


        }
        $array = array(
          'order_data' => $order_data,
          'nota_credito' => $nota_credito,
          'url_name' => $url_name,
          'download_link' => $download_link,
          'download_url' => $download_url,
          'order_id' => $order_id,
          'ajax' => $ajax
        );
        $firmar_xml_api_ewa = "";
       
        // se va ejecutar un hilo 
        if ($ajax == true) {
          $firmar_xml_api_ewa = self::firmar_xml_api_ewa($array);
        }
        else {
          $enviar_hacienda = new WP_Enviar_Hacienda();
          $enviar_hacienda->push_to_queue(array('save_efactura_component' => $array));
          $enviar_hacienda->save()->dispatch();
        }
        return $firmar_xml_api_ewa;
      }

      /**
       * Type Arra
       * return string
       * @metodo que conecta al api de facturacion ewa para procesar los docuemtos electronicos 
       */
      static function firmar_xml_api_ewa($array) {
        $order_data = array();
        $order_data = isset($array['order_data']) ? $array['order_data'] : $array['save_efactura_component']['order_data'] ;
        $order_id = $order_data['order_id'];
        $api_response = wp_remote_post(FE_SERVER . '/wp-json/facturaelectronica/v1/procesa-factura/', array(
          'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
          'body' => json_encode($order_data)
            )
        );

        if (is_array($api_response)) {
          $body = json_decode($api_response['body'], TRUE);
          if ($body) {
            $xml_signed = json_decode($body, TRUE);
              if (isset($xml_signed['file'])) {
              if ($xml_signed['file'] != null) {
                if ($array['nota_credito']) {
                  if ($xml_signed['error_hacienda'] != '' && $xml_signed['estado_hacienda'] == "") {

                    // no permite que aumente el consecutivo si ocurre un error
                    self::set_fe_error_values($order_data);
                    self::notify_email($xml_signed['error_hacienda'], $order_id);
                    return $xml_signed['error_hacienda'];
                    
                  }
                  //si devuelve firmador fue que se ejecuto una mala accion en el servidor de hacienda y por tal motivo no se puedo enviar la factura
                  if ($xml_signed['estado_hacienda'] == 'firmado' ) {

                    // no permite que aumente el consecutivo si ocurre un error
                    self::set_fe_error_values($order_data);
                    self::notify_email($xml_signed['error_hacienda'], $order_id);
                    return $xml_signed['error_hacienda'];
                  }
                  file_put_contents(CREDIT_PATH . '/' . $array['url_name'], base64_decode($xml_signed['file']));
                  self::set_nc_values($order_data, $xml_signed);
                  update_post_meta($order_id, '_nc_remote_status', $xml_signed['estado_hacienda']);
                  $return['status'] = 'success';
                  $return['text'] = 'Nota De Credito recibida con exito';
                  if ($xml_signed['estado_hacienda'] == 'rechazado') {
                    //notificar el error por correo y procesa a la verificacion del problema
                    self::notify_email($xml_signed['error_hacienda'], $order_id);

                  }
                  return $return['text'];
                } else {

                  if ($xml_signed['error_hacienda'] != '' && $xml_signed['estado_hacienda'] == "") {

                    // no permite que aumente el consecutivo si ocurre un error
                    self::set_fe_error_values($order_data);
                    //notificar el error por correo y procesa a la verificacion del problema
                    self::notify_email($xml_signed['error_hacienda'], $order_id);
                    return $xml_signed['error_hacienda'];
                  }
                  
                  if ($xml_signed['estado_hacienda'] == 'firmado' ) {
                    // no permite que aumente el consecutivo si ocurre un error
                    self::set_fe_error_values($order_data);
                    //notificar el error por correo y procesa a la verificacion del problema
                    self::notify_email($xml_signed['error_hacienda'], $order_id);
                    return $xml_signed['error_hacienda'];
                  }                  
                  //save file
                  file_put_contents(XML_SIGNED_PATH . '/' . $xml_signed['file_name'], base64_decode($xml_signed['file']));
                  self::set_fe_values($order_data, $xml_signed);
                  //save the filename on content
                  $download_url = self::getSignedFileUrl($order_id);
                  self::salvar_respuesta_xml($xml_signed['xml_acuse'], $order_id);
                  $url = self::get_confirmation_url($order_id);
                  if ($xml_signed['estado_hacienda'] == 'rechazado') {
                    self::notify_email($xml_signed['error_hacienda'], $order_id);

                  }
                  
                  if (!$array['ajax']) {
                    echo sprintf($array['download_link'], $array['download_url']);
                  }

                  update_post_meta($order_id, '_xml_remote_status', 'Recibido');
                  update_post_meta($order_id, '_nc_remote_status', 'Pendiente');
                  update_post_meta($order_id, '_xml_remote_status', 'Nota');
                  update_post_meta($order_id, '_estado_hacienda', $xml_signed['estado_hacienda']);
                    
                }
                return true;
              } else {

                if ($array['nota_credito']) {
                  /* en caso que ocurra un  error se va guardar la orden para 
                    que cuando realice la peticion por segunda vez , no aumente el consecutivo
                    sino que consulte la orden y asigne el consecutivo ya creado.
                   */
                  self::set_nc_error_values($order_data);
                }
                else {

                  self::set_fe_error_values($order_data);
                }

                return 'Error al procesar el documento, vuelva intentarlo de nuevo.';
              }
            }
            else if ($xml_signed['error'] == null) {

              return 'Xml vacio, verifique sus datos.';
            }
            else {
              //se agrego esta validacion ya que se produce un error que no estaba capturado , cuando la llave criptografica no existia dentro la licencia.
              if ($array['nota_credito']) {
                /* en caso  que ocurra un  error se va guardar la orden para 
                  que cuando realice la peticion por segunda vez , no aumente el consecutivo
                  sino que consulte la orden y asigne el consecutivo ya creado.
                 */
                self::set_nc_error_values($order_data);
              }
              else {
                self::set_fe_error_values($order_data);
              }
              return $xml_signed['error'];
            }
          }
        }      
        if ($array['nota_credito']) {
          /* en caso que ocurra un  error se va guardar la orden para 
            que cuando realice la peticion por segunda vez , no aumente el consecutivo
            sino que consulte la orden y asigne el consecutivo ya creado.
           */
          self::set_nc_error_values($order_data);
        }
        else {
          self::set_fe_error_values($order_data);
        }

        return 'Error conectando con el servidor';
      }
      /**
       * return array
       * @metodo que carga los codigos de las tarifas de impuestos que se pueden usar en el sistama.
       */
      static function cargar_codigos_tarifas_impuestos() {
        $array = array(
          '01' => array('01' => 'tarifa-0-exento'),
          '02' => array('02' =>'tarifa-reducida-1'),
          '03' => array('03' =>'tarifa-reducida-2'),
          '04' => array('04' =>'tarifa-reducida-4'),
          '05' => array('05' =>'transitorio-0'),
          '06' => array('06' =>'ransitorio-4'),
          '07' => array('07' =>'transitorio-8'),
          '08' => array('08' =>'tarifa-general-13'),
          '09' => array('01' =>'iva-calculo-especial-tarifa-0-exento'),
          '10' => array('02' =>'iva-calculo-especial-tarifa-reducida-1'),
          '11' => array('03' =>'iva-calculo-especial-tarifa-reducida-2'),
          '12' => array('04' =>'iva-calculo-especial-tarifa-reducida-4'),
          '13' => array('05' =>'iva-calculo-especial-transitorio-0'),
          '14' => array('06' =>'iva-calculo-especial-transitorio-4'),
          '15' => array('07' =>'iva-calculo-especial-transitorio-8'),
          '16' => array('08' =>'iva-calculo-especial-tarifa-general-13'),
          '17' => array('01' =>'iva-regimen-bienes-usados-tarifa-0-exento'),
          '18' => array('02' =>'iva-regimen-bienes-usados-tarifa-reducida-1'),
          '19' => array('03' =>'iva-regimen-bienes-usados-tarifa-reducida-2'),
          '20' => array('04' =>'iva-regimen-bienes-usados-tarifa-reducida-4'),
          '21' => array('05' =>'iva-regimen-bienes-usados-transitorio-0'),
          '22' => array('06' =>'iva-regimen-bienes-usados-transitorio-4'),
          '23' => array('07' =>'iva-regimen-bienes-usados-transitorio-8'),
          '24' => array('08' =>'iva-regimen-bienes-usados-tarifa-general-13'),
        );
        return $array;
      }

     

      /**
       * 
       * @param $order_id
       *   - the woocomerce order id
       * @param  $filename
       *   - the name of the invoice file
       * @return mixed
       *   - the url to download the invoice or false
       */
      static function getSignedFileUrl($order_id, $filename = FALSE) {
        if ($filename) {
 
          return get_home_url() . '/wp-content/uploads/fefiles/xml_signed/'. "$filename";
        }
        else {
          $factura = get_post_meta($order_id, '_xml_signed_file', true);
          if ($factura && file_exists(XML_SIGNED_PATH . $factura)) {
          return get_home_url() . '/wp-content/uploads/fefiles/xml_signed/'. "$factura";
          }
        }

        return false;
      }

      /**
       * Generate consecutive number and add it to the order.
        @return string / consecutive
        --Este metodo resive por parametros una FE un ticket o una nota de credito
       */
      static function generateConsecutive($order, $nota_credito = false, $ticket = false, $exportacion = false, $compra = false) {
        if ($compra == true) {
          $consecutive_number = substr('0000000000' . (get_option('order_consecutive_compra') + 1), -10);
          update_option('order_consecutive_compra', $consecutive_number);
          return $consecutive_number;
        }
        if ($exportacion == true) {
          $consecutive_number = substr('0000000000' . (get_option('order_consecutive_exportacion') + 1), -10);
          update_option('order_consecutive_exportacion', $consecutive_number);
          return $consecutive_number;
        }
        if ($nota_credito == true) {
          $consecutive_number = substr('0000000000' . (get_option('order_consecutive_nota_credito') + 1), -10);
          update_option('order_consecutive_nota_credito', $consecutive_number);
          return $consecutive_number;
        }
        else if ($ticket == true) {
          $consecutive_number = substr('0000000000' . (get_option('order_consecutive_ticket') + 1), -10);
          update_option('order_consecutive_ticket', $consecutive_number);
          return $consecutive_number;
        }
        else {
          if (!is_null($order)) {
            $consecutive_number = substr('0000000000' . (get_option('order_consecutive_fe') + 1), -10);
            update_option('order_consecutive_fe', $consecutive_number);
            return $consecutive_number;
          }
        }

        if (empty(get_option('order_consecutive'))) {
          update_option('order_consecutive', '0000000000');
        }


        if (!is_null($order)) {
          // Add number to the order.
          update_post_meta($order->get_id(), '_efactura_consecutive_number', $consecutive_number);
        }

        return $consecutive_number;
      }

      /**
       * Get Receiver data from the billing information form.
       */
      static function getReceiverData($order) {
        $zip_code = substr($order->get_billing_postcode(), 0, 5);
        $id = get_post_meta($order->get_id(), '_billing_id_number', true);
        $data = [
          'id_type' => get_post_meta($order->get_id(), '_billing_id_type', true),
          'id' => strlen($id) ? $id : '',
          'first_name' => $order->get_billing_first_name(),
          'last_name' => $order->get_billing_last_name(),
          'phone' => $order->get_billing_phone(),
          'state' => $order->get_billing_state() ? $order->get_billing_state() : 'N/A',
          'city' => $order->get_billing_city(),
          'address1' => $order->get_billing_address_1(),
          'country' => $order->get_billing_country(),
          'email' => $order->get_billing_email(),
          'postcode' => $order->get_billing_postcode(),
          'postcode' => $zip_code,
          'provincia' => strlen($zip_code > 5) ? substr($zip_code, 0, 1) : '1',
          'canton' => strlen($zip_code > 5) ? substr($zip_code, 1, 2) : '01',
          'distrito' => strlen($zip_code > 5) ? substr($zip_code, 3, 2) : '01'
        ];
        foreach ($data as $key => $value) {
          if (!strlen($value)) {
            $field_name = $key;
            if ($key == 'id') {
              $field_name = 'Cédula';
            }
            return [
              'error' => '401',
              'message' => 'Receptor Incompleto: ' . "Falta el campo: {$field_name}"
            ];
          }
        }
        return $data;
      }

      /**
       * Get Transmitter data from woocommerce settings form.
       */
      static function getTransmitterData() {
        $zip_code = substr(get_option('woocommerce_store_postcode', true), 0, 5);
        $data = [
          'name' => get_option('wc_settings_tab_efactura_full_name', true),
          'id_type' => get_option('wc_settings_tab_efactura_id_type', true),
          'id' => str_pad(get_option('wc_settings_tab_efactura_id_number', true), 10, '0', STR_PAD_LEFT),
          'phone' => get_option('wc_settings_tab_efactura_store_phone', true),
          'state' => get_option('woocommerce_store_state', true),
          'city' => get_option('woocommerce_store_city', true),
          'address1' => get_option('woocommerce_store_address', true),
          'country' => get_option('woocommerce_default_country', true),
          'email' => get_option('wc_settings_tab_efactura_store_email', true),
          'postcode' => $zip_code,
          'provincia' => strlen($zip_code > 5) ? substr($zip_code, 0, 1) : '1',
          'canton' => strlen($zip_code > 5) ? substr($zip_code, 1, 2) : '01',
          'distrito' => strlen($zip_code > 5) ? substr($zip_code, 3, 2) : '01'
        ];

        foreach ($data as $key => $value) {
          if (!strlen($value)) {
            return [
              'error' => '401',
              'message' => 'Emisor Incompleto: ' . "Falta el campo: {$key}"
            ];
          }
        }
        return $data;
      }

      static function get_total_tax_amount($order) {
        $total_tax = 0.0;
        foreach ($order->get_items('tax') as $item_id => $item_tax) {
          $total_tax += (float) $item_tax->get_tax_total();
        }

        return $total_tax;
      }

      static function etiqueta_final($parser, $name) {
        global $info, $datos, $contador;
        $info[$name][] = $datos;
      }

      static function extractor_datos($parser, $data) {
        global $datos;
        $datos = $data;
      }

      static function extractor_datos_tags($parser, $data) {
        global $datos;
        $datos .= $data;
      }

      static function parser_extractor($cadena, $tags = true) {

        // Definiendo variables 
        global $info, $datos, $contador;
        $info = array();
        $datos = "";
        $contador = 0;

        // Creando el parser
        $xml_parser = xml_parser_create();

        // Definiendo la funciones controladoras
        xml_set_character_data_handler($xml_parser, ($tags ? __CLASS__ . '::extractor_datos' : __CLASS__ . '::extractor_datos_tags'));
        xml_set_element_handler($xml_parser, "", __CLASS__ . '::etiqueta_final');

        // Procesando el archivo
        if (!xml_parse($xml_parser, $cadena)) {
          die(sprintf("XML error: %s at line %d", xml_error_string(xml_get_error_code($xml_parser)), xml_get_current_line_number($xml_parser)));
        }

        // Liberando recursos
        xml_parser_free($xml_parser);
        return $info;
      }

      /*
        La siguiente Funcion debe recibir por parametro la fecha en formato dd/mm/YYYY
       */

      static function tipo_cambio($fecha) {
       
        $curl = curl_init();
        //api del ministerio de hacienda
        curl_setopt_array($curl, array(
          CURLOPT_URL => "https://api.hacienda.go.cr/indicadores/tc/dolar",
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "postman-token: 4856d147-a8f2-68a4-d61b-a6583442ee7a"
          ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
          $res = json_decode($response);
          //get valor de compra
          $tipo['compra'] = $res->compra->valor;
          //get valor de venta
          $tipo['venta'] = $res->venta->valor;
          //devuelve los tipos
          return $tipo;
        }
      }

      /**
       * Get Conncetion data from woocomerce settings form.
       */
      static function get_connection_data() {
        $data = [
          'username' => get_option('wc_settings_tab_efactura_username', true),
          'password' => get_option('wc_settings_tab_efactura_password', true),
          'cert_file' => get_option('wc_settings_tab_efactura_cert_file', true),
          'pin' => get_option('wc_settings_tab_efactura_pin', true),
          'office' => get_option('wc_settings_tab_efactura_office', true),
          'point_of_sale' => get_option('wc_settings_tab_efactura_pointsale', true),
          'store_enviroment' => get_option('wc_settings_tab_efactura_store_enviroment', true),
          'sec_code' => get_option('wc_settings_tab_efactura_sec_code', true),
          //consecutivo por orden, no es necesario a nivel global
          //'consecutivo' => get_option('wc_settings_tab_efactura_consecutivo_fe', true),
          'cod_actividad' => get_option('wc_settings_tab_efactura_cod_actividad', true),
        ];
        return $data;
      }

      static function check_status($order_id) {
        $status = self::envio_hacienda($order_id);
        return is_numeric($status) && $status == 202;
      }

      //metodo que va permitir notificar al cliente en caso de un error en hacienda
      static function notify_email($message, $order_id) {

        $headers[] = 'From: EWA Facturación reporte de error <ewa@outerspacecoders.com>';
        $headers[] = 'Content-Type: text/html; charset=UTF-8';
        $email = get_option('wc_settings_tab_efactura_store_email', true);
        $response = wp_mail( $email, 'Error al enviar hacienda ', $message . '<br> '.'Verifique la orden # &nbsp;' . $order_id . '&nbsp; y intente  procesarla de nuevo.', $headers, null );
        
      }
  
      static function envio_hacienda($order_id, $confirmacion = FALSE) {   
        $order_data[] = NULL;
        if ($license = woocommerce_factura_electronica_get_option() && $hacienda = self::get_hacienda_info($order_id)) {

          $server = get_option('wc_settings_tab_efactura_store_enviroment', true);
          $econnect_data = self::get_connection_data();
          $order = wc_get_order($order_id);
          $econnect = new WC_Efactura_hacienda_connect();
          $econnect->setUser($econnect_data['username']);
          $econnect->setPass($econnect_data['password']);
          $econnect->setPin($econnect_data['pin']);
          $econnect->setTypeReceptor($receptor['id_type']);
          $econnect->setIdReceptor($receptor['id']);
          $econnect->setTypeEmisor($emisor['id_type']);
          $econnect->setIdEmisor($emisor['id']);
          $econnect->setConsecutive($hacienda['consecutivo']);
          $econnect->setClave($hacienda['clave']);
          $econnect->setFecha($hacienda['fecha']);

          if ($server == 'api-prod') {
            $econnect->setReceptionUriProd();
            $econnect->setIdpUriProd();
            $econnect->setIdpClientIDProd();
          }
          if ($tokenAuth = $econnect->tokenAuth()) {

            $order_data[] = NULL;
            $order_data['clave'] = $hacienda['clave'];
            $order_data['token'] = $tokenAuth;

            if ($confirmacion) {
              //consulta al server el comprobante y devuelve un estado con el xml de rechazo o aceptado.
              $api_response = wp_remote_post(FE_SERVER . '/wp-json/consultarEstadoHacienda/v1/procesa-estado/', array(
                'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
                'body' => json_encode($order_data)
                  )
              );

              if (is_array($api_response)) {
                $body = json_decode($api_response['body'], TRUE);
                if ($body) {
                  $data = $body;
                  if (strlen($data['xml_acuse']) && self::salvar_respuesta_xml($data['xml_acuse'], $order_id)) {
                    if ($data['estado_hacienda'] == 'rechazado') {
                      self::notify_email($data['error_hacienda'], $order_id);
      
                    }
                    $url = self::get_confirmation_url($order_id);
                    $message = sprintf('<a href= "%s" target="_blank" >Descargar Confirmacion</a>', $url);
                    update_post_meta($order_id, '_xml_remote_status', 'Recibido');
                    update_post_meta($order_id, '_nc_remote_status', 'Pendiente');
                    update_post_meta($order_id, '_xml_remote_status', 'Nota');
                    update_post_meta($order_id, '_estado_hacienda', $data['estado_hacienda']);
                  
                  } 
                }
              }
            }
            else {
              $message = $econnect->sendInvoice($tokenAuth, get_post_meta($order->get_id(), '_xml_signed_file', true));
            }
          }
          $econnect->closeConnection($tokenAuth);
       
        }
        else{
          $message = __('Datos insuficientes en factura');
        }
        return $message;
      }

      static function envio_nota_credito($order_id) {

        if ($license = woocommerce_factura_electronica_get_option() && $hacienda = self::get_hacienda_info($order_id)) {
          $server = get_option('wc_settings_tab_efactura_store_enviroment', true);
          $emisor = $hacienda['emisor'];
          $receptor = $hacienda['receptor'];

          $econnect_data = self::get_connection_data();
          $order = wc_get_order($order_id);
          $econnect = new WC_Efactura_hacienda_connect();
          $econnect->setUser($econnect_data['username']);
          $econnect->setPass($econnect_data['password']);
          $econnect->setPin($econnect_data['pin']);
          $econnect->setTypeEmisor($emisor['id_type']);
          $econnect->setTypeReceptor($receptor['id_type']);
          $econnect->setIdEmisor($emisor['id']);
          $econnect->setIdReceptor($receptor['id']);
          $econnect->setConsecutive($hacienda['consecutivo']);
          $econnect->setClave($hacienda['nc_clave']);
          $econnect->setFecha($hacienda['nc_fecha']);
          if ($server == 'api-prod') {
            $econnect->setReceptionUriProd();
            $econnect->setIdpUriProd();
            $econnect->setIdpClientIDProd();
          }
          if ($tokenAuth = $econnect->tokenAuth()) {
            $message = $econnect->sendInvoice($tokenAuth, get_post_meta($order->get_id(), '_xml_signed_file', true), TRUE);
            $econnect->closeConnection($tokenAuth);
          }
          else {
            $message = __('Token Invalido');
          }
        }
        else {
          $message = __('Datos insuficientes en Nota Credito');
        }
        return $message;
      }

      function ajax_callback_enviar_nota_credito() {
        $return = array();
        $return['status'] = 'ERROR';
        $return['text'] = 'Parece que la orden NO existe en hacienda';

        if ($order = $_POST['order']) {
          if (!get_post_meta($order, '_nc_remote_status', TRUE) == 'Recibido') {
            $return['status'] = 'ERROR';
            $return['text'] = __('La Nota de Credito ya fue registrada en hacienda');
          }
          else {
            $status = self::envio_nota_credito($order);
            if (is_numeric($status) && $status == 202) {
              update_post_meta($order, '_nc_remote_status', 'Recibido');
              $return['status'] = 'success';
              $return['text'] = 'Nota De Credito recibida con exito';
            }
            else {
              $return['status'] = 'ERROR';
              $return['text'] = $status;
            }
          }
        }

        die(json_encode($return));
      }

      static function ajax_callback_firmar_mensaje_hacienda() {
        global $wpdb;
        $clave = $_POST['clave'];
        $action = $_POST['my_action'];
        $return['status'] = 'ERROR';
        $return['text'] = __('uoh! algo salio mal');
        $table_name = $wpdb->prefix . 'fe_aceptadas';


        if ($action == 'Firmar') {
          $mensaje = self::firmar_mensaje_hacienda($clave);
          if ($mensaje == "true") {
            $updated = $wpdb->update($table_name, array('hacienda_respuesta' => 'FIRMADA'), array('clave' => $clave));
            if ($updated) {
              $return['status'] = 'success';
              $return['text'] = __('Factura Firmada Con Exito');
            }
          }
          else if ($mensaje == null) {
            $return['status'] = 'ERROR';
            $return['text'] = 'Error conectando al servidor';
          }
          else {
            $return['status'] = 'ERROR';
            $return['text'] = $mensaje;
          }
        }
        elseif ($action == 'Eliminar') {
          $wpdb->query("DELETE FROM $table_name WHERE clave IN($clave)");
          $return['status'] = 'success';
          $return['text'] = __('Factura Eliminada Con Exito');
        }
        elseif ($action == 'Confirmar') {
          $mensaje = self::mensaje_aceptar_rechazar($clave);
          $return['text'] = $mensaje;
          if (is_int($mensaje)) {
            $updated = $wpdb->update($table_name, array('hacienda_respuesta' => 'ACEPTADA'), array('clave' => $clave));
            $return['status'] = 'success';
            $return['file'] = get_site_url() . '/' . str_replace(ABSPATH, '', ACCEPTED_PATH . "Aceptacion-{$clave}.xml");
          }
        }
        elseif ($action == 'Rechazar') {
          $mensaje = self::mensaje_aceptar_rechazar($clave, 'Rechazo');
          $return['text'] = $mensaje;
          if (is_int($mensaje)) {
            $updated = $wpdb->update($table_name, array('hacienda_respuesta' => 'RECHAZADA'), array('clave' => $clave));
            $return['status'] = 'success';
            $return['file'] = get_site_url() . '/' . str_replace(ABSPATH, '', ACCEPTED_PATH . "Rechazo-{$clave}.xml");
          }
        }
        elseif ($action == 'Rechazado-parcial') {
          $mensaje = self::mensaje_aceptar_rechazar($clave, 'Rechazado-parcial');
          $return['text'] = $mensaje;
          if (is_int($mensaje)) {
            $updated = $wpdb->update($table_name, array('hacienda_respuesta' => 'Rechazado-parcial'), array('clave' => $clave));
            $return['status'] = 'success';
            $return['file'] = get_site_url() . '/' . str_replace(ABSPATH, '', ACCEPTED_PATH . "Rechazado-parcial-{$clave}.xml");
          }
        }

        die(json_encode($return));
      }

      function mensaje_aceptar_rechazar($clave, $type = 'Aceptacion') {
        if ($license = woocommerce_factura_electronica_get_option()) {
          $server = get_option('wc_settings_tab_efactura_store_enviroment', true);
          global $wpdb;
          $table_name = $wpdb->prefix . 'fe_aceptadas';
          $data = array_pop($wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name WHERE clave=%s", $clave), ARRAY_A));
          $xml = simplexml_load_file(ACCEPTED_PATH . "{$type}-{$clave}.xml");
          $consecutivo = (string) $xml->NumeroConsecutivoReceptor;

          $econnect_data = self::get_connection_data();
          $econnect = new WC_Efactura_hacienda_connect();
          $econnect->setUser($econnect_data['username']);
          $econnect->setPass($econnect_data['password']);
          $econnect->setPin($econnect_data['pin']);
          $econnect->setTypeReceptor($data['tipo_receptor']);
          $econnect->setIdReceptor($data['cedula_receptor']);
          $econnect->setTypeEmisor($data['tipo_emisor']);
          $econnect->setIdEmisor($data['cedula_emisor']);
          $econnect->setConsecutive($consecutivo);
          $econnect->setClave($clave);
          $econnect->setFecha($data['fecha']);
          if ($server == 'api-prod') {
            $econnect->setReceptionUriProd();
            $econnect->setIdpUriProd();
            $econnect->setIdpClientIDProd();
          }
          if ($tokenAuth = $econnect->tokenAuth()) {
            
            if ($data['hacienda_respuesta'] == 'FIRMADA') {
              $message = $econnect->sendMensajeReceptor($tokenAuth, "{$type}-{$clave}.xml");
            }
            //Este 1 se va utilizar cuando ya la factura ha sido enviada la primera vez envia hacienda, luego solo se consulta el estado a obtener un resultado.
            if($data['hacienda_respuesta'] == 'ENVIADA') $message = 1;
            if (is_int($message)) {
              //hacienda no es el mas rapido, esperemoslo un momentito
              $data = $econnect->consultar($tokenAuth);
              while ($data['mensaje'] === 'procesando') {
                sleep(2);
                $data = $econnect->consultar($tokenAuth);
              }

              if (strlen($data['xml']) && $data['mensaje'] != 'rechazado') {
                //everthing is working fine
              }
              else {
                $mensaje = base64_decode($data['xml']);
                $pattern = "#<\s*?DetalleMensaje\b[^>]*>(.*?)</DetalleMensaje\b[^>]*>#s";
                preg_match($pattern, $mensaje, $matches);
                $errors = $matches[1];
                preg_match('/"([^"]+)"/', $errors, $matches);
                $mensaje = isset($matches[1]) ? $matches[1] : $errors;
                $message = empty($mensaje) ? __('<strong>Error: Intente de nuevo en unos segundos</strong>') : __('<strong>Error: </strong>' . $mensaje);
              }
            }
            else {
              $wpdb->update($table_name, array('hacienda_respuesta' => 'ENVIADA'), array('clave' => $clave));
            }

            $econnect->closeConnection($tokenAuth);
          }
          else {
            $message = __('<strong>Token Invalido:</strong> Revise que su usuario,conrtaseña,pin y clave criptografica estan correctos.');
          }
          return $message;
        }
      }

      function firmar_mensaje_hacienda($clave) {
        global $wpdb;
        $table_name = $wpdb->prefix . 'fe_aceptadas';
        $data = array_pop($wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name WHERE clave=%s", $clave), ARRAY_A));
        $xml = simplexml_load_file(ACCEPTED_PATH . $data['file_name']);
        $resumen_xml = $xml->ResumenFactura;
        $order_data = array('order_id' => '');
        $order_data['receiver_data'] = array('id' => $data['cedula_receptor']);
        // Get Transmitter data.
        $order_data['transmitter_data'] = array('id' => $data['cedula_emisor']);
        $order_data['consecutive_number'] = self::generateConsecutiveMensaje();
        $order_data['resumen_factura'] = array(
          'total_venta' => $resumen_xml->TotalComprobante,
          'total_impuesto' => $resumen_xml->TotalImpuesto
        );
        $econnect_data = self::get_connection_data();

        $order_data['products'] = '';
        $order_data['license'] = woocommerce_factura_electronica_get_option();
        $order_data['domain'] = strstr($_SERVER['SERVER_SOFTWARE'], 'nginx') ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
        $order_data['order_timestamp'] = '';
        $order_data['fecha_emision'] = $data['fecha'];
        $order_data['clave'] = $clave;
        $order_data['mensaje'] = 'TRUE';
        $order_data['cod_actividad'] = $econnect_data['cod_actividad'];
        
        $api_response = wp_remote_post(FE_SERVER . '/wp-json/mensajeelectronico/v1/procesa-mensaje/', array(
          'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
          'body' => json_encode($order_data)
            )
        );

        if (is_array($api_response)) {
          $body = json_decode($api_response['body'], TRUE);
          if ($body) {
            $xml_signed = json_decode($body, TRUE);
            if (isset($xml_signed['acepta_file'])) {
              file_put_contents(ACCEPTED_PATH . '/' . $xml_signed['acepta_file_name'], base64_decode($xml_signed['acepta_file']));
              file_put_contents(ACCEPTED_PATH . '/' . $xml_signed['rechaza_file_name'], base64_decode($xml_signed['rechaza_file']));
              file_put_contents(ACCEPTED_PATH . '/' . $xml_signed['acepta_partial_file_name'], base64_decode($xml_signed['acepta_file_partial']));
              //creó los xmls necesario y fue succed la respuesta.
              return __("true");
            }
            else {
              //error de licencia o de credenciales de hacienda.
              return __($body);
            }
          }
        }
        //when return mull es un error 500 del servidor.
      }

      /**
        @return string
        --Este metodo a diferencia ,genera el consecutivo especifico para el mensaje de acpetacion de fe
       */
      static function generateConsecutiveMensaje() {
        // Get consecutive number and sum 1
        $consecutive_number = substr('0000000000' . (get_option('order_consecutive_mensaje') + 1), -10);
        update_option('order_consecutive_mensaje', $consecutive_number);
        return $consecutive_number;
      }

      static function ajax_callback_enviar_hacienda() {
        $return = array();
        $return['status'] = 'ERROR';
        $return['text'] = 'Parece que la orden NO existe en hacienda';
        $confirmacion = isset($_POST['confirmacion']);
        if ($order = $_POST['order']) {
          if (!get_post_meta($order, '_xml_remote_status', TRUE) == 'Recibido' && !$confirmacion) {
            $return['status'] = 'ERROR';
            $return['text'] = 'La orden ya fue registrada en hacienda';
          }
          else {
            $status = self::envio_hacienda($order, $confirmacion);
            if (is_numeric($status) && $status == 202) {
              update_post_meta($order, '_xml_remote_status', 'Nota');
              $return['status'] = 'success';
              $return['text'] = 'orden recibida con exito';
            }
            elseif ($confirmacion) {
              $return['status'] = 'success';
              $return['text'] = $status;
            }
            else {
              $return['status'] = 'ERROR';
              $return['text'] = $status;
            }
          }
        }

        die(json_encode($return));
      }
      /**
       * Metodo que cuando la orden se procesa, envia automaticamente desde el rest API de woo
       */
      static function rest_api_procesar_orden($order) {
        
        self::save_efactura_component($order['id'], false, $cancelar);
        return $order;
        
      }
      /**
       * Return $test
       * Metodo funciona para crear un reporte segun rango de fechas
       */
      static function ajax_callback_consultar_reporte(){
        $date_from = new DateTime();
        $date_from->setTimezone(new DateTimeZone('America/Costa_Rica'));
        $date_from->setTimestamp($_POST['from']);
        $date_to = new DateTime();
        $date_to->setTimezone(new DateTimeZone('America/Costa_Rica'));
        $date_to->setTimestamp($_POST['to']);
        $order_data['desde'] = $date_from->getTimestamp();
        $order_data['hasta'] = $date_to->getTimestamp();
        $order_data['license'] = woocommerce_factura_electronica_get_option();
        $order_data['domain'] = strstr($_SERVER['SERVER_SOFTWARE'], 'nginx') ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'];
        $text['ok'] = "Exitoso";
        $api_response = wp_remote_post(FE_SERVER . '/wp-json/facturaelectronica/v1/consultar-facturas/', array(
          'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
          'body' => json_encode($order_data)
            )
        );
        //validar con el servidor:
        if(is_array($api_response)){
          $body = json_decode($api_response['body'], TRUE);
          if ($body) {

            if(is_array($body['respuesta']) && count($body['respuesta'] ) > 0  ){
              $return['status'] = 'success';
              $body['date_from'] = $_POST['from'];
              $body['date_to'] = $_POST['to'];
              //recorrer las ordenes y guardar los xml por timestamp
              $array_invoices = [];
              $contador = 0;
              foreach($body['respuesta'] as $invoices) {
                $invoices['total_venta'] = 0;
                $invoices['total_impuesto'] = 0;
                
                if ($invoices['xml_firmado'] != NULL) {                
                  $xml = base64_decode($invoices['xml_firmado']);
                  file_put_contents(CONFIRM_PATH . "firmado". $invoices['timestamp'].".xml", $xml);
                  $xml = simplexml_load_string(base64_decode($invoices['xml_firmado']));
                  $invoices["xml_firmado"] = get_bloginfo('url'). "/" .  str_replace(ABSPATH, '',CONFIRM_PATH . "firmado". $invoices['timestamp'].".xml"); 
                  $totales = $xml->ResumenFactura;
                  $invoices['total_venta'] = (float) $totales->TotalComprobante[0];
                  $invoices['total_impuesto'] = (float) $totales->TotalImpuesto[0];
                }
                if ($invoices['xml_acuse'] != NULL) {                
                  $xml = base64_decode($invoices['xml_acuse']);
                  file_put_contents(CONFIRM_PATH . "acuse". $invoices['timestamp'].".xml", $xml);
                  $invoices["xml_acuse"] = get_bloginfo('url') . "/" .  str_replace(ABSPATH, '', CONFIRM_PATH . "acuse". $invoices['timestamp'].".xml"); 
                }
                if($invoices['order_id'] != NULL){
                  $invoices["order"] = $invoices['order_id'];
                  $invoices["order_id"] = get_bloginfo('url') . "/" .  str_replace(ABSPATH, '', "wp-admin/post.php?post={$invoices['order_id']}&action=edit"); 
                }else{
                  $invoices["order"] = "N/A";
                }
                $array_invoices[$contador] = $invoices;
                $contador++;
                
              }
              $body['respuesta'] = NULL;
              $body['respuesta'] = $array_invoices;
              
               //limpiar folder
			  $files = glob(PDF_REPORT.'/*');  
			   
			  foreach($files as $file) { 			   
			    if(is_file($file))  
			        unlink($file);  
			  } 
              
              
              //guarda los options para el reporte
              update_option('array_report', $body);
              $tfile = dirname(WC_FE_FILE) . '/templates/fe-report-info.php';
              ob_start();
                
              load_template($tfile, TRUE);
              //carga el html del template para el pdf
              $template = ob_get_contents();
              //creo instancia para crear pdf
              $html2pdf = new Html2Pdf();
              $html2pdf->writeHTML($template);
              
              $pdf_name = 'reporte_' . uniqid();
                                   
              $html2pdf->Output(PDF_REPORT. $pdf_name .".pdf", "F");
              $text['link'] = get_site_url() . '/' . str_replace(ABSPATH, '', PDF_REPORT . $pdf_name . ".pdf");
              ob_end_clean();

              die(json_encode($text));              
            
            }else{
              $text['ok'] = $body['respuesta'];

              die(json_encode($text));
            }
          }
        }
      }

      static function ajax_callback_procesar_orden() {
        $return = array();
        $return['status'] = 'ERROR';
        $return['text'] = 'LA ORDEN ESTA MAL REGISTRADA';
        if ($order = $_POST['order']) {
          $cancelar = isset($_POST['cancelacion']);

          $return['text'] = self::save_efactura_component($order, TRUE, $cancelar);

          $meta_field_data = get_post_meta($order, '_xml_signed_file', true);
          if (strlen($meta_field_data)) {
            $return['status'] = 'success';
            $tfile = file_exists(get_template_directory() . '/woocommerce/emails/fe-template-invoice-link.php') ? get_template_directory() . '/woocommerce/emails/fe-template-invoice-link.php' : dirname(WC_FE_FILE) . '/templates/fe-template-invoice-link.php';
            ob_start();
            load_template($tfile, TRUE);
            $template = ob_get_contents();
            ob_end_clean();
            $return['text'] = $template;
          }
        }

        die(json_encode($return));
      }
      static function ajax_callback_cargar_actividades(){
        $return = array();
        $return['actividades'] = '';
        $curl = curl_init();
        curl_setopt_array($curl, array(
          CURLOPT_URL => "https://api.hacienda.go.cr/fe/ae?identificacion=" . $_POST['cedula'],
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache"
          ),
        ));
        //se agregan dos lines porque se caia ala hora de consultar en hacienda.
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
        $response = curl_exec($curl);
        $err = curl_error($curl);
      
        curl_close($curl);
      
        if ($err) {
          return "cURL Error #:" . $err;
        }
        else {
          $activades_emisor = json_decode($response);
          $return['actividades'] = $activades_emisor->actividades;
           die(json_encode($return));
        }
      }
      static function admin_scripts() {
        wp_enqueue_style('woocommerce-factura-electronica', plugins_url('/css/woocommerce-factura-compra.css', WC_FE_FILE));
        wp_register_script('woocommerce-factura-electronica', plugins_url('/js/woocommerce-factura-electronica.js', WC_FE_FILE));
        wp_register_script('dropzone', plugins_url('/js/dropzone.js', WC_FE_FILE));
        wp_enqueue_style('dropzonecss', plugins_url('/css/dropzone.css', WC_FE_FILE));
        wp_enqueue_style('woocommerce-factura-electronica', plugins_url('/css/woocommerce-factura-electronica.css', WC_FE_FILE));
        wp_enqueue_script('woocommerce-factura-electronica');
        wp_enqueue_script('dropzone');
      }
      
      static function set_fe_error_values($data) {
        $hacienda = array(
          'receptor' => $data['receiver_data'],
          'emisor' => $data['transmitter_data'],
          'consecutivo' => $data['consecutive_number'],
          'tipo_cambio' => $data['resumen_factura']['tipo_cambio'],
          'ticket' => $data['ticket'],
          'medio_de_pago' => '02',
          'condicion_venta' => '01'
        );
        update_post_meta($data['order_id'], '_xml_hacienda', $hacienda);
      }

      static function set_fe_values($data, $api_data) {
       
          $hacienda = array(
            'receptor' => $data['receiver_data'],
            'emisor' => $data['transmitter_data'],
            'consecutivo' => $api_data['consecutivo'],
            'nota_credito' => $data['nota_credito'],
            'tipo_cambio' => $data['resumen_factura']['tipo_cambio'],
            'tipo_comprobante' => $data['tipo_comprobante'],
            'nota_credito' => null,
            'ticket'  => $data['ticket'],
            'clave' => $api_data['clave'],
            'fecha' => $api_data['fecha'],
            'medio_de_pago' => '02',
            'condicion_venta' => '01'
          );
        
        $hash = str_replace('Factura', '', substr($api_data['file_name'], 0, -4));
        QRcode::png(get_home_url() . '/fe-order/' . $data['order_id'] . "?hash=$hash", QR_PATH . $data['order_id'] . ".png");
        update_post_meta($data['order_id'], '_xml_hacienda', $hacienda);
        update_post_meta($data['order_id'], '_xml_signed_file', $api_data['file_name']);
        update_post_meta($data['order_id'], '_xml_remote_status', 'Pendiente');
      }
      
      static function set_nc_error_values($data){
        $hacienda = get_post_meta($data['order_id'], '_xml_hacienda', true);
        $hacienda['ticket']   = false;
        $hacienda['nota_credito'] =  $data['consecutive_number'];
        update_post_meta($data['order_id'], '_xml_hacienda', $hacienda);
      }
      
      static function set_nc_values($data, $api_data) {
        $hacienda = get_post_meta($data['order_id'], '_xml_hacienda', true);

        $hacienda['nc_clave'] = $api_data['clave'];
        $hacienda['nc_fecha'] = $api_data['fecha'];
        $hacienda['ticket']   = false;
        $hacienda['nota_credito'] =  $api_data['consecutivo'];
        update_post_meta($data['order_id'], '_nc_remote_status', 'Enviada');
        update_post_meta($data['order_id'], '_xml_remote_status', 'Nota');
        update_post_meta($data['order_id'], '_xml_hacienda', $hacienda);
      }

      static function attach_hacienda_to_mail($attachments, $email_id, $order) {
        if (!is_null($order) && $order instanceof WC_Order) {
        $settings = get_option('wc_settings_tab_efactura_fe_emails', true);
        $file_name = get_post_meta($order->get_id(), '_xml_signed_file', true);
        if (strlen($file_name) && is_array($settings) && in_array($email_id, $settings)) {
          $attachments[] = XML_SIGNED_PATH . $file_name;
          if (file_exists(CONFIRM_PATH . $file_name)) {
            $confirmation_name = str_replace('Factura', 'Confirmacion', $file_name);
            copy(CONFIRM_PATH . $file_name, get_temp_dir() . $confirmation_name);
            $attachments[] = get_temp_dir() . $confirmation_name;
          }
        }
      }
        return $attachments;
      }

      static function salvar_respuesta_xml($base_64_xml, $order_id) {
        $xml = base64_decode($base_64_xml);
        
        $name = get_post_meta($order_id, '_xml_signed_file', true);
        file_put_contents(CONFIRM_PATH . $name, $xml);
        return true;
      }

      static function existe_confirmacion($order_id) {
        $name = get_post_meta($order_id, '_xml_signed_file', true);
        return file_exists(CONFIRM_PATH . $name);
      }

      static function existe_cancelacion($order_id) {
        $name = get_post_meta($order_id, '_xml_signed_file', true);
        return file_exists(CREDIT_PATH . $name);
      }

      static function get_confirmation_url($order_id) {
        $factura = get_post_meta($order_id, '_xml_signed_file', true);
        return get_home_url() . '/wp-content/uploads/fefiles/xml_confirm/'."$factura";
      }

      static function get_cancelacion_url($order_id) {

        $factura = get_post_meta($order_id, '_xml_signed_file', true);
        return get_home_url() . '/wp-content/uploads/fefiles/xml_nota_credito/'."$factura";
      }

      static function get_hacienda_info($order_id) {
        $hacienda = get_post_meta($order_id, '_xml_hacienda', true);
        $ticket = !get_post_meta($order_id, '_billing_is_fe', true);

        $valores = array(
          'emisor',
          'consecutivo',
          'tipo_cambio',
          'clave',
          'fecha',
          'medio_de_pago',
          'condicion_venta'
        );

        if (!$ticket) {
          $valores['receptor'] = 'receptor';
        }

        foreach ($valores as $value) {
          if (!isset($hacienda[$value])) {
            return false;
          }
        }
        return $hacienda;
      }

      static function fe_wpo_wcpdf_template_paths($template_paths) {
        $template_paths['factura_electronica'] = dirname(WC_FE_FILE) . '/templates/factura_electronica';
        return $template_paths;
      }

      static function fe_scripts() {

        wp_register_script('woocommerce-factura-electronica', plugins_url('/js/woocommerce-factura-electronica-gs.js', WC_FE_FILE));
        wp_register_style('woocommerce-factura-electronica', plugins_url('/css/woocommerce-factura-compra.css', WC_FE_FILE));
        wp_register_style('woocommerce-factura-electronica', plugins_url('/css/woocommerce-factura-electronica.css', WC_FE_FILE));

        wp_enqueue_style('woocommerce-factura-electronica');

        wp_enqueue_script( 'jquery-ui-autocomplete');
        wp_enqueue_script('woocommerce-factura-electronica');

      }


      static function  ajax_callback_obtener_codigo_catalogo(){

        $data['producto'] = $_POST['producto'];

          $text['ok'] = "Exitoso";
          $api_response = wp_remote_post(FE_SERVER . '/wp-json/facturaelectronica/v1/obtener-codigo-cabys/', array(
            'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
            'body' => json_encode($data)
              )
          );
          //validar con el servidor:
          if(is_array($api_response)){

            $body = json_decode($api_response['body'], TRUE);
            
            if ($body) {

              if(is_array($body['respuesta']) && count($body['respuesta'] ) > 0  ){
                
                $titles = [];

                foreach( $body['respuesta'] as $r ){
                  $titles[] = addslashes($r['title']);
                }

                die(json_encode($titles));
              }
            }

          }
      }
    }

    // end class.
    // finally instantiate our plugin class and add it to the set of globals
    $GLOBALS['wc_efactura'] = new WC_Efactura();
  }
}
