<?php

if (!defined('ABSPATH'))
  exit; // Exit if accessed directly

/**
 * WooComerce Facturacion Electronica 
 * custom placeholders.
 */
if (!class_exists('WC_Fe_Placeholders')) {

  class WC_Fe_Placeholders {

    public  function __construct() {
      add_filter('woocommerce_email_format_string_replace', __CLASS__ . '::fe_email_replace', 20, 2);
      add_filter('woocommerce_email_format_string_find', __CLASS__ . '::fe_email_find', 20, 2);
      add_filter('woocommerce_email_footer_text', __CLASS__ . '::fe_email_footer');
    }

    public function fe_email_find($find, $mail) {
      $find['fe-link'] = '{fe_link}';
      $find['fe-link-raw'] = '{fe_link_raw}';
      $find['fe-confirm-link'] = '{fe_confirm_link}';
      $find['fe-confirm-raw'] = '{fe_confirm_raw}';
      $find['fe-qr'] = '{fe_qr}';
      $find['fe-valores'] = '{fe_valores}';
      return $find;
    }

    public function fe_email_replace($replace, $mail) {
      $order = $mail->object;
      $replace['fe-link'] = self::get_xml_file_link($order);
      $replace['fe-link-raw'] = self::get_xml_file_url($order);
      $replace['fe-confirm-raw'] = self::get_xml_confirm_file_url($order);
      $replace['fe-confirm-link'] = self::get_xml_confirm_file_url($order);
      $replace['fe-qr'] = 'link al qr';
      $replace['fe-valores'] = self::fe_hacienda_valores();
      return $replace;
    }

    public function get_xml_file_url($order) {
      $url = '';
      if (!is_null($order) && $order instanceof WC_Order) {
        $file_name = get_post_meta($order->get_id(), '_xml_signed_file', true);
        if (strlen($file_name)) {
          $hash = str_replace('Factura', '', substr($file_name, 0, -4));
          $url = get_home_url() . "/fe-order/{$order->get_id()}?hash={$hash}";
        }
      }
      return $url;
    }

    public function get_xml_file_link($order) {
      $url = '';
      if (!is_null($order) && $order instanceof WC_Order) {
        $url = self::get_xml_file_url($order);
        return strlen($url) ? "<a href='{$url}'>" . __('Descargar Factura Electrónica') : '</a>';
      }
      return $url;
    }

    public function get_xml_confirm_file_url($order) {
      $url = '';
      if (!is_null($order) && $order instanceof WC_Order) {
        $file_name = get_post_meta($order->get_id(), '_xml_signed_file', true);
        if (strlen($file_name)) {
          $hash = str_replace('Factura', '', substr($file_name, 0, -4));
          $url = get_home_url() . "/fe-confirmacion/{$order->get_id()}?hash={$hash}";
        }
      }
      return $url;
    }

    public function get_xml_confirm_file_link($order) {
      if (!is_null($order) && $order instanceof WC_Order) {
        $url = self::get_xml_confirm_file_url($order);
        return strlen($url) ? "<a href='{$url}'>" . __('Descargar Confirmacion') : '</a>';
      }
      return '';
    }

    public function fe_email_footer($string) {
      global $post;

      if (isset($post->ID) && $post->post_type == 'shop_order') {
        $order = wc_get_order($post->ID);
        $string = str_replace('{fe_link}', self::get_xml_file_link($order), $string);
        $string = str_replace('{fe_link_raw}', self::get_xml_file_url($order), $string);
        $string = str_replace('{fe_qr}', self::get_xml_file_url($order), $string);
        $string = str_replace('{fe_valores}', self::fe_hacienda_valores(), $string);
      }

      return $string;
    }

    public function fe_hacienda_valores() {
      global $post;

      $html = '';
      if (isset($post->ID)) {
        $template = file_exists(get_template_directory() . '/woocommerce/emails/fe-hacienda-info.php') ? get_template_directory() . '/woocommerce/emails/fe-hacienda-info.php' : dirname(WC_FE_FILE) . '/templates/fe-hacienda-info.php';
        ob_start();
        load_template($template, FALSE);
        $html = ob_get_contents();
        ob_end_clean();
      }
      return $html;
    }

    public function fe_email_variables() {
      global $post;
      $hacienda = array();
      if (isset($post->ID) && $post->post_type == 'shop_order') {
        $hacienda = get_post_meta($post->ID, '_xml_hacienda', true);
      }
      return $hacienda;
    }

  }

}