<?php
/**
 * Factura electronica Settings
 *
 */
if (!defined('ABSPATH'))
  exit; // Exit if accessed directly

if (!class_exists('WC_Efactura_Settings')) :

  function e_factura__add_settings() {

    class WC_Efactura_Settings extends WC_Settings_Page {

      public function __construct() {

        $this->id = 'settings_tab_efactura';
        $this->label = __('Factura Electrónica', 'woocommerce-settings-tab-efactura');

        add_filter('woocommerce_settings_tabs_array', array($this, 'add_settings_page'), 20);
        add_action('woocommerce_settings_' . $this->id, array($this, 'output'));
        add_action('woocommerce_settings_save_' . $this->id, array($this, 'save'));
        add_action('woocommerce_sections_' . $this->id, array($this, 'output_sections'));
        self::includes();
      }

      public function includes() {
        include_once plugin_dir_path(__FILE__) . 'efactura-aceptacion.php';
      }

      /**
       * Get sections
       *
       * @return array
       */
      public function get_sections() {

        $sections = array(
          '' => __('Hacienda Settings', 'woocommerce-settings-tab-efactura'),
          'consecutivo_section' => __('Consecutivos', 'woocommerce-settings-tab-efactura'),
          'email_section' => __('Email Settings', 'woocommerce-settings-tab-efactura'),
          'accept_interface' => __('Facturas Aceptadas', 'woocommerce-settings-tab-efactura'),
          'report_invoice' => __('Reporte de facturas', 'woocommerce-settings-tab-efactura')

        );

        return apply_filters('woocommerce_get_sections_' . $this->id, $sections);
      }

      public function get_settings($current_section = '') {
        $license = woocommerce_factura_electronica_get_option();
        $settings = array(
          'section_title' => array(
            'name' => $current_section == 'email_section' ? __('Efactura Emails', 'woocommerce-settings-tab-efactura') : __('', 'woocommerce-settings-tab-efactura'),
            'type' => 'title',
            'desc' => $license ? '<br />' : '<div id="message" class="error inline"><p><strong>"Licencia Inactiva"</strong></p></div>',
            'id' => 'wc_settings_tab_efactura_section_title'
          )
        );

        if ($license) {
          if ('consecutivo_section' == $current_section) {
            $settings['consecutivo_fe'] = array(
              'name' => __('Consecutivo factura eletrónica', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'order_consecutive_fe',
              'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
              . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
              . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_fe', '0000000001'),
            );
            $settings['consecutivo_tikect'] = array(
              'name' => __('Consecutivo tiquete electrónico', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'order_consecutive_ticket',
              'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
              . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
              . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_ticket', '0000000001'),
            );
            $settings['consecutivo_nota_credito'] = array(
              'name' => __('Consecutivo nota crédito', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'order_consecutive_nota_credito',
              'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
              . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
              . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_nota_credito', '0000000001'),
            );
            $settings['consecutivo_mensaje'] = array(
              'name' => __('Consecutivo mensaje', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'order_consecutive_mensaje',
              'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
              . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
              . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_mensaje ', '0000000001'),
            );
            $settings['consecutivo_exportacion'] = array(
              'name' => __('Consecutivo factura de exportación', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'order_consecutive_exportacion',
              'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
              . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
              . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_exportacion ', '0000000001'),
            );
            $settings['consecutivo_fe_compra'] = array(
               'name' => __('Consecutivo factura de compra', 'woocommerce-settings-tab-efactura'),
               'type' => 'number',
               'id' => 'order_consecutive_compra',
               'desc' => '<br/>Punto de inicio del consecutivo de 8 digitos, <br/> no tiene que ser '
               . 'igual al consecutivo que se lleva en hacienda, este nos dice de donde comenzar '
               . '. EJ: 0000000001<br>Actual Consecutivo: ' . get_option('order_consecutive_compra ', '0000000001'),
             );
          }
          else
          if ('email_section' == $current_section) {
            $settings['fe_emails'] = array(
              'name' => __('Emails para adjuntar el xml', 'woocommerce-settings-tab-efactura'),
              'type' => 'multiselect',
              'id' => 'wc_settings_tab_efactura_fe_emails',
              'css' => "height: 200px",
              'options' => $this->get_fe_woocomerce_emails(),
              'desc_tip' => true,
            );
          }
          else if ('accept_interface' == $current_section) {
            self::fe_contacts_page_handler();
            exit;
          }
          else if ('report_invoice' == $current_section){
            self::fe_report_with_range();
            exit;
          }
          else {
            $e = new WC_Efactura();
            $fecha = date('d/m/Y');
            $dollar = $e->tipo_cambio($fecha);

            $settings['full_name'] = array(
              'name' => __('Nombre del emisor', 'woocommerce-settings-tab-efactura'),
              'type' => 'text',
              'id' => 'wc_settings_tab_efactura_full_name'
            );
            $settings['username'] = array(
              'name' => __('Usuario', 'woocommerce-settings-tab-efactura'),
              'type' => 'text',
              'desc' => __('Usuario creado desde hacienda', 'woocommerce-settings-tab-efactura'),
              'id' => 'wc_settings_tab_efactura_username'
            );
            $settings['password'] = array(
              'name' => __('Contraseña', 'woocommerce-settings-tab-efactura'),
              'type' => 'password',
              'desc' => __('Contraseña creada desde hacienda', 'woocommerce-settings-tab-efactura'),
              'id' => 'wc_settings_tab_efactura_password'
            );
            $settings['pin'] = array(
              'name' => __('Pin', 'woocommerce-settings-tab-efactura'),
              'type' => 'password',
              'desc' => __('Pin creado de 4 digitos en hacienda', 'woocommerce-settings-tab-efactura'),
              'id' => 'wc_settings_tab_efactura_pin'
            );
            $settings['office'] = array(
              'name' => __('Número de Sucursal', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'desc' => '<br/>Sucursal donde se ejecuta la venta. Inicia en 1. Tiene que ser de 3 digitos Ej: 001',
              'id' => 'wc_settings_tab_efactura_office'
            );
            $settings['pointsale'] = array(
              'name' => __('Punto de Venta', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'desc' => '<br/>Punto de venta donde se ejecuta la facturacion. Inicia en 1. Tiene que ser de 5 digitos Ej: 00001',
              'id' => 'wc_settings_tab_efactura_pointsale'
            );
            $settings['id_type'] = array(
              'name' => __('Tipo de identificación', 'woocommerce-settings-tab-efactura'),
              'type' => 'select',
              'id' => 'wc_settings_tab_efactura_id_type',
              'options' => array(
                '01' => __('Cédula Física', 'woocommerce'),
                '02' => __('Cédula Jurídica', 'woocommerce'),
                '03' => __('DIMEX', 'woocommerce'),
                '04' => __('NITE', 'woocommerce')
              ),
              'desc_tip' => true,
            );
            $settings['id_number'] = array(
              'name' => __('Número de identificación', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'desc' => '<br/>Cedula juridica, residencia,etc. Tiene que ser de 12 digitos Ej: 003101672825',
              'id' => 'wc_settings_tab_efactura_id_number'
            );
            $settings['cod_actividad'] = array(
              'name'    => __('Codigo de actividad', 'woocommerce-settings-tab-efactura' ),
              'id'      => 'wc_settings_tab_efactura_cod_actividad',
              'class'   => 'wc-enhanced-select',
              'type'    => 'select',
              'options' => self::get_cod_actividad(),
               
            );
            $settings['store_email'] = array(
              'name' => __('Correo Electrónico', 'woocommerce-settings-tab-efactura'),
              'type' => 'email',
              'id' => 'wc_settings_tab_efactura_store_email'
            );
            $settings['store_phone'] = array(
              'name' => __('Teléfono', 'woocommerce-settings-tab-efactura'),
              'type' => 'text',
              'id' => 'wc_settings_tab_efactura_store_phone'
            );
            $settings['store_enviroment'] = array(
              'name' => __('Ambiente', 'woocommerce-settings-tab-efactura'),
              'type' => 'radio',
              'options' => array(
                'api-prod' => __('Production', 'woocommerce'),
                'api-stag' => __('Staging', 'woocommerce'),
              ),
              'id' => 'wc_settings_tab_efactura_store_enviroment'
            );
            $settings['sec_code'] = array(
              'name' => __('Codigo de seguridad', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'wc_settings_tab_efactura_sec_code',
              'desc' => '<br/>Codigo de seguridad de ocho digitos. EJ: 22999905',
            );
            $settings['tipo_cambio'] = array(
              'name' => __('Tipo De Cambio', 'woocommerce-settings-tab-efactura'),
              'type' => 'radio',
              'options' => array(
                'compra' => __('Compra ' . '(' . $dollar['compra'] . ')', 'woocommerce'),
                'venta' => __('Venta ' . '(' . $dollar['venta'] . ')', 'woocommerce'),
                'texto' => __('Digitar Valor', 'woocommerce'),
              ),
              'desc' => '<br/>Siempre se usara el tipo de cambio proveido por el BCCR automaticamente al escoger las dos primeras opciones ',
              'id' => 'wc_settings_tab_efactura_tipo_cambio'
            );
            $settings['dollar_custom'] = array(
              'name' => __('Tipo de cambio deseado', 'woocommerce-settings-tab-efactura'),
              'type' => 'number',
              'id' => 'wc_settings_tab_efactura_dollar_custom',
              'desc' => '<br/>En caso de la opcion anterior tener "Digitar Valor" se usara este '
            );

            $settings['cert_file'] = array(
              'name' => __('Llave Criptográfica', 'woocommerce-settings-tab-efactura'),
              'type' => 'file',
              'desc' => file_exists(CERTS_PATH . '/cert.pfx') ? '<br />Archivo subido' : '<br/>archivo proveido por hacienda, se desarga del atv',
              'id' => 'wc_settings_tab_efactura_cert_file'
            );
              
          }
          $settings['section_end'] = array(
            'type' => 'sectionend',
            'id' => 'wc_settings_tab_efactura_section_end'
          );
        }

        return apply_filters('woocommerce_get_settings_' . $this->id, $settings, $current_section);
      }
      /**
 @return array string
 --devuelve todas las actividades del MH
 */
       
public function get_cod_actividad(){
  $array = array(
    '751301' =>  'ACTIVIDADES DEL SECTOR PUBLICO RELACIONADAS CON LA INFRAESTRUCTURA',
    
    '014001' => 
    
       'SERVICIO DE JARDINERIA Y/O DISEÃ‘O PAISAJISTA',
    
    '011218' => 
  
       'PRODUCCION DE MINIVEGETALES',
     
    '930302' => 
  
     'CEMENTERIO PUBLICO (JUNTA ADMINISTRATIVA)',
    
    '743005' => 
    
      'ALQUILER DE ESPACIOS PUBLICITARIOS',
    
    '803002' => 
    
       'ENSEÃ‘ANZA SUPERIOR PRIVADA (UNIVERSIDADES)',
    
    '749914' => 
  
       'SERVCICIO DE COBRANZA DE RECIBOS PUBLICOS Y OTROS',
      
    '851101' => 
    
       'CLINICA, CENTROS MEDICOS, HOSPITALES PRIVADOS Y OTROS',
     
    '152001' => 
 
       'PRODUCCION DE HELADOS Y OTROS PRODUCTOS SIMILARES',
    
    '191101' => 

      'FABRICACION DE MALETAS, BOLSOS DE MANO Y ARTICULOS SIMILARES',
    
    '292701' => 
    
       'FABRICACION DE ARMAS Y MUNICIONES',
    
    '453002' => 
    
   'REPARACION DE ASCENSORES(ELEVADORES)',
     
    '924102' => 

      'EXPLOTACION DE PISCINAS O ALBERCAS DE BAÃ‘O',
    
    '749917' => 
  
       'MODELAJE PROFESIONAL (MODELO)',
     
    '749902' => 
    
       'SERVICIOS DE LEVANTADO DE TEXTO Y/O CORRECCION DE TEXTOS Y OTROS',
    
    '291201' => 
   
       'FABRICACION DE BOMBAS, COMPRESORES, GRIFOS Y VALVULAS',
    
    '711101' => 
    
       'ALQUILER DE AUTOMOVILES DE TODO TIPO',
     
    '522009' => 
      'VENTA DE FRUTAS GRAVADAS CON VENTAS',
    
    '749101' => 
       'SERVICIOS DE ADMINISTRACION DE PERSONAL',
     
    '501001' => 
       'VENTA AL POR MAYOR Y MENOR DE VEHICULOS NUEVOS Y/O USADOS',
    
    '515002' => 
    'VENTA AL POR MAYOR DE REPUESTOS Y/O ACCESORIOS PARA MAQUINARIA Y EQUIPO AGROPECUARIO',
     
    '512237' => 
       'COMERCIO AL POR MAYOR DE ALIMENTOS Y PRODUCTOS N.C.P. EXCENTOS DE VENTAS',
    
    '513904' => 
       'VENTA AL POR MAYOR DE SUMINISTROS Y ARTICULOS DE LIBRERIA',
      
    '519016' => 
       'VENTA AL POR MAYOR DE ARTICULOS Y ACCESORIOS ELECTRONICOS',
    
    '602302' => 
    
       'SERVICIOS DE GRUA',
    
    '369201' => 
    
       'FABRICACION DE INSTRUMENTOS MUSICALES, PARTES Y PIEZAS Y SUS ACCESORIOS',
    
    '289202' => 
    
       'SERVICIO DE TRATAMIENTO Y REVESTIMIENTO DE TODO TIPO DE MATERIALES',
    
    '232002' => 
    
       'FABRICACION DE PRODUCTOS DIVERSOS DERIVADOS DEL PETROLEO',
    
    '371001' => 
 
       'RECICLAJE DE DESPERDICIOS Y DESECHOS METALICOS',
    
    '642005' => 
    
       'SERVICIOS TELEFONICOS, TELEGRAFICOS Y POR TELEX',
    
    '851912' => 
  
       'NUTRICIONISTA',
    
    '172101' => 
    
       'FABRICACION DE ARTICULOS CONFECCIONADOS DE MATERIALES TEXTILES, EXEPTO PRENDAS DE VESTIR',
    
    '513901' => 
   
       'VENTA AL POR MENOR DE ARTICULOS DEPORTIVOS',
    
    '333001' => 
    
       'FABRICACION DE RELOJES DE TODO TIPO',
    
    '523906' => 
    
       'VENTA AL POR MENOR CELULARES ACCESORIOS EQUIPO Y ART PARA COMUNICACIONES INCLUYE LA REPARACION',
    
    '712911' => 
    
       'ALQUILER DE RAMPAS Y SIMILARES',
     
    '702001' => 
    
       'AGENTES O CORREDORES DE BIENES RAICES',
    
   '526012' => 
    
       'INSTALACION Y MANTENIMIENTO DE PISCINAS,JACUZZIS Y SIMILARES',
     
    '712901' => 
    
       'ALQUILER DE MAQUINAS EXPENDEDORAS DE ALIMENTOS Y OTROS',
      
    '919101' => 
   
       'ACTIVIDADES DE ORGANIZACIONES RELIGIOSAS',
     
    '525303' => 
    
       'EMISION DE LA LOTERIA NACIONAL Y SIMILARES',
    
    '300001' => 
    
       'FABRICACION DE MAQUINARIA DE OFICINA, CONTABILIDAD E INFORMATICA',
    
    '050001' => 
 
       'PESCADORES ARTESANALES EN PEQ. ESCALA',
    
    '351103' => 
  
       'VENTA DE EMBARCACIONES DE MOTOR Y VELA',
     
    '602002' => 
   
       'TRANSPORTE DE PRODUCTOS DERIVADOS DEL PETROLEO',
      
    '921403' => 
    
       'PERIODISTA POR CUENTA PROPIA',
    
    '154908' => 
  
       'ELABORACION DE PRODUCTOS DE MAIZ GRAVADOS CON VENTAS',
     
    '011802' => 
        'CULTIVO DE CEREALES (INCLUSIVE EL ARROZ LEGUMBRES Y GRANOS BASICOS',
     
    '741408' => 
    
       'CORRESPONSAL DE RADIO, TELEVISION Y PRENSA ESCRITA',
    
    '453003' => 
    
       'VENTA E INSTALACION DE ALARMAS Y OTROS SISTEMAS ELECTRICOS',
    
    '651905' => 
    
       'SERVICIO DE ENVIO Y RECIBO DE DINERO',
    
    '521901' => 
    
       'TIENDAS O ALMACENES POR DEPARTAMENTOS',
     
    '523405' => 
    
       'VENTA AL POR MENOR DE MATERIALES PARA LA CONSTRUCCION',
    
    '659908' => 
    
       'FONDOS DE LA LEY DEL SISTEMA DE BANCA PARA EL DESARROLLO',
     
    '924906' => 
    
       'GRABACION DE SONIDO (MUSICA, ETC.) EN DISCOS GRAMOFONICOS YEN CINTA MAGNETOFONICA',
     
    '852002' => 
    
       'SERVICIOS MEDICOS VETERINARIOS',
    
    '723001' => 
    
       'PROCESAMIENTO DE DATOS',
    
    '505003' => 
    
       'COMERCIO DE COMBUSTIBLE SIN PUNTO FIJO (PEDDLER)',
    
    '453005' => 
    
       'INSTALACION DE ALARMAS Y OTROS SISTEMAS DE SEGURIDAD',
    
    '504001' => 
    
       'VENTA DE MOTOCICLETAS',
    
    '292201' => 
    
       'FABRICACION DE MAQUINAS HERRAMIENTA',
    
    '050005' => 
    
       'PESCADORES EN GRAN ESCALA',
    
    '950001' => 
    
       'HOGARES PRIVADOS CON SERVICIO DOMESTICO',
    
    '290001' => 
    
       'SERVICIO DE REPARACION DE MAQUINARIA Y EQUIPO.',
     
    '232005' => 
    
       'ELABORACION DE BIOCOMBUSTIBLES',
    
    '930908' => 
    
       'SERVICIO DE HERRADURA',
     
    '202301' => 
    
       'FABRICACION DE TARIMAS',
     
    '851906' => 
    
       'PSIQUIATRIA',
     
    '519014' => 
    
       'VENTA AL POR MAYOR DE POLEN Y/O SEMILLAS PARA USO AGRICOLA',
     
    '751201' => 
    
       'JUNTAS DE EDUCACION, COMEDORES ESCOLARES, PATRONATOS, COOPERATIVAS ESCOLARES, COLEGIALES, VOCACIONALES',
     
    '252001' => 
    
       'FABRICACION DE ARTICULOS DE PLASTICO',
    
    '361004' => 
    
       'MAQUILA DE MUEBLES',
      
    '712902' => 
    
       'ALQUILER DE MAQUINAS DE ENTRETENIMIENTO (CON MONEDAS)',
    
    '712907' => 
    
       'ALQUILER MAQUINARIA Y EQUIPO PARA LA ELABORACION Y/O MANTENIMIENTO DE PRODUCTOS ALIMENTICIOS',
    
    '523909' => 
    
       'VENTA DE PRODUCTOS DE ARTESANIA Y SOUVENIR',
    
    '749904' => 
    
       'SERVICIO DE DISEÃ‘O O DECORACION DE INTERIORES (POR CUENTA PROPIA).',
    
    '513602' => 
    
       'COMERCIO AL POR MAYOR DE PRODUCTOS DE VETERINARIA',
    
    '924101' => 
    
       'GIMNASIOS',
    
    '731001' => 
    
       'INVESTIGACION Y DESARROLLO EXPERIMENTAL EN EL CAMPO DE LAS CIENCIAS NATURALES Y LA INGENIERIA',
    
    '523306' => 
    
       'VENTA AL POR MENOR DE ANTIGUEDADES',
    
    '011501' => 
    
       'CULTIVO DE BANANO',
    
    '741407' => 
    
       'ASESOR ADUANERO',
    
    '752301' => 
    
       'ACTIVIDADES DE MANTENIMIENTO DEL ORDEN PUBLICO Y DE SEGURIDAD',
    
    '11322' => 
    
       'CULTIVO DE SEMILLAS COMESTIBLES Y GERMINACION DE SEMILLAS OLEAGINOSAS',
    
    '809004' => 
    
       'PROFESOR POR CUENTA PROPIA',
    
    '369101' => 
    
       'FABRICACION DE JOYAS, BISUTERIA Y ARTICULOS CONEXOS',
    
    '512201' => 
    
       'COMERCIO AL POR MAYOR DE ALIMENTOS, GRANOS BASICOS, CARNES Y DEMAS COMESTIBLES Y ARTICULOS DE LA CANASTA BASICA',
    
    '523803' => 
    
       'VENTA AL POR MENOR DE MAQUINARIA Y EQUIPO DE TODO TIPO Y ARTICULOS CONEXOS',
    
    '523916' => 
    
       'COMERCIO AL POR MENOR DE ALIMENTOS Y PRODUCTOS N.C.P. EXENTOS DE VENTAS',
    
    '523309' => 
    
       'VENTA DE CUADROS PINTURAS HECHAS POR PINTORES NACIONALES Y EXTRANJEROS PRODUCIDOS EN EL PAIS',
    
    '289304' => 
    
       'TALLER DE MECANICA DE PRECISION',
    
    '960108' => 
    
       'COMERCIALIZADORES DE MERCANCIAS EXENTAS DEL IMPUESTO DE VENTAS',
    
    '919901' => 
    
       'ASOCIACION DE DESARROLLO COMUNAL Y/O SERVICIOS COMUNITARIOS',
    
    '701004' => 
    
       'ALQUILER DE EDIFICIOS Y PROPIEDADES DIFERENTES A CASAS DE HABITACION',
    
    '853103' => 
    
       'RESIDENCIAS UNIVERSITARIAS CON SERVICIOS INTEGRADOS',
    
    '512210' => 
    
       'COMERCIO AL POR MAYOR DE BEBIDAS CON CONTENIDO ALCOHOLICO (IMPORTADORES)',
    
    '369301' => 
    
       'FABRICACION DE ARTICULOS DE DEPORTE',
    
    '522015' => 
    
       'VENTA AL POR MENOR DE CAFE (EXCEPTO EL ENVASADO, ENLATADO, SOLUBLE, DESCAFEINADO)',
    
    '701005' => 
    
       'ALQUILER DE MARCAS REGISTRADAS',
    
    '522018' => 
    
       'VENTA DE PESCADOS Y/O MARISCOS (PESCADERIAS O MARISQUERIAS) GRAVADOS EN VENTAS',
    
    '401004' => 
    
       'FABRICACION, ENSAMBLE Y VENTA DE SISTEMAS PARA EL APROVECHAMIENTO DE ENERGIAS RENOBABLES',
    
    '713053' => 
    
       'ALQUILER DE ANIMALES (CABALLOS, SERPIENTES, ETC)',
    
    '522016' => 
    
       'COMERCIO AL POR MENOR DE PRODUCTOS DE TABACO',
    
   '505001' => 
    
       'VENTA DE COMBUSTIBLES (CONOCIDAS COMO GASOLINERAS O BOMBAS)',
    
    '930906' => 
    
       'SERVICIO DE TATUAJE Y PIERCING',
    
    '361002' => 
    
       'REPARACION DE TAPICERIA',
    
    '453008' => 
    
       'INSTALACION Y MANTENIMIENTO DE CABLEADO DE COMUNICACIONES Y/O ENERGIA ELECTRICA',
    
    '321001' => 
    
       'MANUFACTURA O FABRICACION DE OTROS COMPONENTES ELECTRONICOS',
     
    '281101' => 
    
       'FABRICACION DE PRODUCTOS METALICOS PARA USO ESTRUCTURAL',
    
    '011231' => 
    
       'CULTIVO HORTALIZAS LEGUMBRES ESPECIALIDADES HORTICOLAS PRODUCTOS VIVERO GRABADOS VENTAS',
    
    '801001' => 
    
       'ENSEÑANZA PREESCOLAR Y PRIMARIA PRIVADA',
    
    '514304' => 

       'VENTA AL POR MAYOR DE EQUIPO DE AIRE ACONDICIONADO Y CALENTADORES (ELECTRICOS, SOLARES, ETC.)',
  
    '011340' => 
   
       'CULTIVO DE PIÑA',
    
    '924909' => 
    
       'OTRAS ACTIVIDADES DE ESPARCIMIENTO',
    
    '292301' => 
    
       'FABRICACION DE MAQUINARIA METALURGICA',
    
    '331102' => 
    
       'FABRICACION Y COMERCIALIZACION DE EQUIPO MEDICO',
    
    '222102' => 
    
       'TIPOGRAFIA Y/O LITOGRAFIA (IMPRENTA)',
    
    '221201' => 
    
       'EDICION DE PERIODICOS, REVISTAS Y OTRAS PUBLICACIONES PERIODICAS',
    
    '523205' => 
    
       'VENTA AL POR MENOR DE PRENDAS DE VESTIR, ROPA Y ZAPATOS (TIENDAS)',
    
    '502004' => 
    
       'SERVICIO DE ENDEREZADO Y PINTURA PARA TODA CLASE DE VEHICULO',
    
   '453011' => 
    
       'INSTALACION Y MANTENIMIENTO DE CONMUTADORES Y OTROS SISTEMAS PARA TELECOMUNICACIONES',
    
    '724001' => 
    
       'ACTIVIDADES RELACIONADAS CON BASES DE DATOS',
    
    '315001' => 
    
       'FABRICACION DE LAMPARAS ELECTRICAS Y EQUIPO DE ILUMINACION',
    
    '853102' => 
    
       'FUNDACIONES DE SERVICIO SOCIAL (PRIVADAS)',
    
    '671101' => 
    
       'ADMINISTRACION DE MERCADOS FINANCIEROS',
    
    '701006' => 
    
       'ALQUILER DE PATENTES (DE LICORES, UNICAMENTE)',
    
    '511001' => 
    
       'COMISIONISTAS, AGENTES DE VENTAS, ORGANIZADORES DE SUBASTAS,TIQUETERAS, ETC.',
    
    '452003' => 
    
       'MANTENIMIENTO, REPARACION Y AMPLIACIONES DE EDIFICIOS, APARTAMENTOS, CONDOMINIOS Y CASAS',
    
    '319001' => 
    
       'FABRICACION DE OTROS TIPOS DE EQUIPO ELECTRICO N.C.P.',
    
    '012101' => 
    
       'CRIA DE CABALLOS Y OTROS EQUINOS',
    
    '851915' => 
    
       'SERVICIOS DE ESTERILIZACION DE PRODUCTOS MEDICOS Y FARMACEUTICOS',
    
    '11349' => 
    
       'CULTIVO DE CACAO',
      'descripcion' => 'EN ESTA CLASE SE INCLUYE EL CULTIVO DE CACAO',
    
    '809001' => 
    
       'ESCUELAS COMERCIALES (NO ESTATALES)',
    
    '11907'=> 
    
       'PEQUENOS PRODUCTORES AGRICOLAS',
    
    '930907' => 
    
       'SERVICIO DE DISCO MOVIL',
    
    '919905' => 
   
       'ASOCIACION PROAYUDA A PERSONAS ADICTAS A DROGAS E INDIGENTES',
     
    '921201' => 
    
       'EXHIBICION DE FILMES Y VIDEOCINTAS (SALAS DE CINE)',
    
    '523601' => 
    
       'COMERCIO AL POR MENOR DE COMPUTADORAS, ACCESORIOS, MICROCOMPONENTES Y PAQUETES DE COMPUTO',
     
    '453010' => 
   
       'INSTALACION Y MANTENIMIENTO DE PORTONES ELECTRICOS',
     
    '151401' => 
  
       'ELABORACION DE ACEITES Y GRASAS DE ORIGEN VEGETAL Y ANIMAL',
    
    '11302' => 
    
       'CULTIVO DE FRUTAS',
    
    '642004' => 
   
       'SERVICIO DE RADIO FRECUENCIA',
     
    '151105' => 
    
       'PRODUCCION DE EMBUTIDOS EMPACADOS, ENVASADO Y ENLATADO',
   
    '154903' => 

       'FABRICACION DE HIELO',
     
    '522002' => 
    
       'VENTA DE CARNES (RES, POLLO,CERDO) INCLUIDAS EN LA CANASTA BASICA',
    
  '752101' => 
   
       'RELACIONES EXTERIORES',
    
    '742201' => 
    
       'ENSAYOS Y ANALISIS TECNICOS',
    
    '181001' => 
  
       'SERVICIOS DE COSTURA Y SASTRERIA (COSTURERAS Y SASTRES)',
    
    '410001' => 
  
       'CAPTACION, TRATAMIENTO Y DISTRIBUCION DE AGUA',
    
    '522012' => 
    
       'COMERCIO AL POR MENOR DE BEBIDAS GASEOSAS Y CARBONATADAS',
    
    '924103' => 
   
       'ENTRENADOR, INSTRUCTORES Y/O PREPARADORES FISICOS POR CUENTA PROPIA',
     
    '752201' => 
   
       'ACTIVIDADES DE DEFENSA',
      
    '154102' => 
    
       'PASTELERIA, REPOSTERIA O AMBAS',
    
    '519005' => 
   
       'VENTA AL POR MAYOR DE EQUIPO PARA CAMPO DE JUEGOS (PLAY)',
    
    '020005' => 
    
       'ACTIVIDADES DE CONSERVACION DE BOSQUES. (SERVICIOS AMBIENTALES, VENTA DE OXIGENO).',
     
    '742102' => 
   
       'ACTIVIDADES DE ARQUITECTURA E INGENIERIA',
     
    '743004' => 
    
       'SERVICIOS DE PUBLICIDAD',
    
    '512204' => 
   
       'COMERCIO AL POR MAYOR DE PRODUCTOS LACTEOS',
    
    '651903' => 
    
       'BANCOS ESTATALES (EXCEPTO EL BANCO CENTRAL)',
    
    '292901' => 
    
       'FABRICACION DE OTROS TIPOS DE MAQUINARIA DE USO ESPECIAL',
      
    '321002' => 
 
       'DISEÃ‘O DE COMPONENTES ELECTRONICOS',
    
    '512232' => 
    
       'COMERCIO AL POR MAYOR DE PRODUCTOS SUSTITUTOS DEL AZUCAR',
    
    '523303' => 
   
       'VENTA AL POR MENOR DE DISCOS, CDS Y OTROS SIMILARES',
     
    '523304' => 
   
       'VENTA AL POR MENOR DE ELECTRODOMESTICOS, MUEBLES Y ARTICULOS PARA EL HOGAR',
     
    '749201' => 
 
       'SERVICIOS DE INVESTIGACION, SEGURIDAD PRIVADA, AGENCIAS Y CONSULTORES',
    
    '241301' => 
    
       'FABRICACION DE PLASTICOS Y CAUCHO SINTETICO EN FORMAS PRIMARIAS',
     
    '222104' => 
   
       'IMPRESION DIGITAL',
      
    '261001' => 
      'FABRICACION DE VIDRIO Y PRODUCTOS DE VIDRIO',
    
    '523206' => 
    
       'SERVICIO DE FOTOCOPIADORA',
    
    '512217' => 
   
       'COMERCIO AL POR MAYOR DE VINOS. BEBIDAS FERMENTADAS Y NO FERMENTADAS',
     
    '454001' => 
   
       'SERVICIOS DE TERMINACION Y ACABADO DE EDIFICIOS',
    
    '526011' => 
    
       'REPARACION Y MANTENIMIENTO DE EQUIPO PARA TELECOMUNICACIONES',
    
    '923202' => 
   
       'SERVICIOS DE RESTAURACION DE OBRAS DE ARTE',
     
    '151304' => 
  
       'PRODUCCION DE CONCENTRADOS PARA JUGOS (CITRICULTURA)',
    
    '622001' => 
    
       'SERVICIO DE TRANSPORTE AEREO NO REGULAR DE PASAJEROS',
    
    '523926' => 
    
       'MANTENIMIENTO DE EXTINTORES',
    
   '502007' => 

       'AUTODECORACION',
      
    '521201' => 

       'ABASTECEDORES, PULPERIAS O MINI-SUPER',
    
    '352001' => 
    
       'FABRICACION DE LOCOMOTORAS Y MATERIAL RODANTE PARA FERROCARRILES Y TRANVIAS',
    
    '221901' => 
   
       'ARTES GRAFICAS',
      
    '741401' => 
    
       'ECONOMISTAS',
    
    '523401' => 
   
       'VENTA AL POR MENOR DE DEPOSITO DE MADERA',
    
    '243001' => 
 
       'FABRICACION DE FIBRAS SINTETICAS',
     
    '503006' => 
   
       'ACTIVIDADES DE DESARME DE VEHICULOS Y VENTA DE REPUESTOS',
     
    '552002' => 
    
       'CAFETERIAS',
     
    '851207' => 

       'SERVICIOS DE ODONTOLOGO Y CONEXOS',
     
    '924105' => 
    
       'EXPLOTACION DE INSTALACIONES Y CAMPOS DEPORTIVOS',
    
    '141001' => 
  
       'EXTRACCION DE PIEDRA, ARENA Y ARCILLA',
    
    '515004' => 
    
       'VENTA AL POR MAYOR DE EXTINTORES Y EQUIPO SIMILAR',
     
    '742110' => 
   
       'BIOLOGO POR CUENTA PROPIA',
    
    '11103' => 
   
       'CULTIVO Y COMERCIALIZACION DE CESPED',
    
   ' 602102' => 
  
       'SERVICIO DE TRANSPORTE REGULAR DE PERSONAS POR VIA TERRESTRE',
    
    '269901' => 
    
       'FABRICACION DE OTROS PRODUCTOS MINERALES NO METALICOS N.C.P.',
    
    '526010' => 
    
       'INSTALACION Y MANTENIMIENTO DE PERSIANAS Y CORTINAS',
    
    '504004' => 
   
       'SERVICIO DE REPARACION DE TODA CLASE DE MOTOCICLETAS Y SUS PARTES',
    
    '281201' => 
    
       'FABRICACION DE TANQUES, DEPOSITOS Y RECIPIENTES DE METAL',
      
    '702003' => 
    
       'ADMINISTRACION, MANTENIMIENTO, REPARACION Y LIMPIEZA DE LOSSERVICIOS Y BIENES COMUNES DE LA PROPIEDAD EN CONDOMINIO',
     
    '924107' => 
    
       'ESPECTACULOS DEPORTIVOS',
     
    '711103' => 
    
       'ALQUILER DE EQUIPO DE TRANSPORTE POR VIA TERRESTRE, ACUATICA O AEREA',
  
    '642001' => 
    
       'SERVICIOS DE RADIO-MENSAJES, RADIOLOCALIZADORES Y SIMILARES',
    
    '513910' => 
    
       'VENTA AL POR MAYOR DE TODO TIPO DE ARTICULOS POR CATALOGO',
    
    '154905' => 
   
       'ELABORACION DE PRODUCTOS DE MAIZ EXENTOS DE VENTAS',
     
    '523208' => 
    
       'VENTA AL POR MENOR DE PRODUCTOS TEXTILES (TELAS)',
    
    '671201' => 
    
       'CORREDORES DE BOLSA',
    
    '523922' => 
    
       'VENTA AL POR MENOR DE POLEN Y SEMILLAS',
    
    '012204' => 
    
       'CRIA Y VENTA DE OTROS ANIMALES SEMIDOMESTICADOS O SALVAJES',
    
    '671202' => 
    
       'PUESTOS DE BOLSA',
    
    '155403' => 
  
       'ELABORACION DE BEBIDAS NO ALCOHOLICAS / GASEOSAS / AGUA MINERAL Y DE MANANTIAL',
    
    '672001' => 
    
       'ACTIVIDADES AUXILIARES DE LA FINANCIACION DE PLANES DE SEGUROS Y DE PENSIONES',
    
    '154904' => 
    
       'FABRICACION DE CAFE (EXCEPTO EL ENVASADO, ENLATADO, SOLUBLEY EL DESCAFEINADO)',
    
    '291901' => 
    
       'REPARACION DE EQUIPO DE REFRIGERACION Y CONGELACION',
     
    '551004'=> 
 
       'MOTEL Y/O SERVICIO DE HABITACION OCASIONAL, ALBERGUES, POSADAS Y SIMILARES',
     
    '513906' => 
    
       'VENTA AL POR MAYOR DE ARTICULOS, ARTEFACTOS, DISCOS Y MUEBLES PARA EL HOGAR',
    
    '749907' => 
 
       'SERVICIO DE INSPECCION DE TODO TIPO DE MERCADERIAS (INCLUYEDROGAS)',
    
    '741302' => 
    
       'ENCUESTAS DE OPINION PUBLICA',
    
    '011102' => 
    
       'CULTIVO DE PALMA AFRICANA Y OTROS FRUTOS OLEAGINOSOS',
    
    '911102' => 
   
       'CAMARAS DE COMERCIO',
    
    '852001' => 
   
       'SERVICIOS VETERINARIOS CON VENTA DE PRODUCTOS GRAVADOS CON VENTAS',
    
    '291903' => 
    
       'INSTALACION Y/O MANTENIMIENTO DE EQUIPO DE REFRIGERACION Y CONGELACION',
    
    '921401' => 
   
       'TEATROS (EXPLOTACION)',
    
    '241101' => 
           'FABRICACION DE SUSTANCIAS QUIMICAS Y GASES INDUSTRIALES EXCEPTO ABONOS',
     
    '523501' => 

       'VENTA AL POR MENOR REALIZADA DENTRO DEL DEPOSITO LIBRE COMERCIAL DE GOLFITO.',
     
    '924908' => 
    
       'ESPECTACULOS PUBLICOS EN GENERAL EXCEPTO LOS DEPORTIVOS Y EL TEATRO',
    
    '155101' => 
 
       'ELABORAC. RECTIFICAC MEZCLA BEBIDAS ALCOHOLICAS/PROD. DE ALCOHOL ETILICO (SUSTAN FERMENTADAS)',
     
    '851910' => 
 
       'TRANSPORTE EN AMBULANCIA TERRESTRE Y AEREO (SERVICIO PRIVADO)',
    
    '802101' => 

       'ENSEÃ‘ANZA SECUNDARIA PRIVADA',
    
    '960104' => 
    
       'IMPUESTO A LAS PERSONAS JURIDICAS',
    
    '153104' => 
    
       'FABRICACION DE HARINAS FINAS Y GRUESAS',
    
    '331201' => 
  
       'FABRICACION DE EQUIPOS PARA MEDIR, VERIFICAR, Y NAVEGAR Y DE EQUIPOS DE CONTROL',
     
    '630405' => 
    
       'EXPLOTACION DE ACTIVIDADES TURISTICAS(CUYA PRES TACION NO SEA REALIZADA POR UN CENTRO DE RECREO)',
    
    '152002' => 
 
       'ELABORACION DE PRODUCTOS LACTEOS GRAVADOS CON VENTAS',
    
    '512214' => 
   
       'VENTA AL POR MAYOR DE CAFE (EXCEPTO EL ENVASADO, ENLATADO, SOLUBLE, DESCAFEINADO)',
     
    '753001' => 
    
       'ACTIVIDADES DE PLANES DE SEGURIDAD SOCIAL DE AFILIACION OBLIGATORIA',
    
    '900002' => 
    
       'LIMPIEZA DE TANQUES SEPTICOS Y AGUAS REDISUALES',
    
    '369902' => 

       'FABRICACION DE VELAS (CANDELAS) EXCEPTO LAS PERFUMADAS, COLOREADAS Y DECORADAS.',
    
    '451001' => 
    
       'DEMOLICION DE EDIFICIOS Y OTRAS ESTRUCTURAS',
    
    '311001' => 
    
       'FABRICACION DE MOTORES, GENERADORES Y TRANSFORMADORES ELECTRICOS, PARTES Y ACCES',
    
    '523915' => 
    
       'DISTRIBUCION Y VENTA DE GAS EN CILINDRO',
      
    '921301' => 
    
       'RADIOEMOSORAS',
    
    '142101' => 
    
       'EXTRACCION DE MINERALES Y SUSTANCIAS PARA LA FABRICACION DE ABONOS',
    
    '331103' => 
    
       'FABRICACION Y COMERCIALIZACION DE ZAPATOS ORTOPEDICOS',
    
    '151301'=> 
    
       'ELABORACION Y CONSERVACION DE FRUTAS, LEGUMBRES Y HORTALIZAS',
    
    '012201' => 
    
       'CRIA DE CERDOS',
     
    '523912' => 
    
       'VENTA AL POR MENOR Y MAYOR DE PRODUCTOS E INSUMOS AGROPECUARIOS',
    
    '523403' => 
    
       'VENTA DE PINTURAS',
     
    '851913' => 
  
       'PROFESIONALES EN EDUCACION ESPECIAL',
     
    '523806' => 
    
       'VENTA AL POR MENOR DE MAQUINARIA INDUSTRIAL USADA',
     
    '749302' => 
 
       'SERVICIOS DE LIMPIEZA (INTERIORES Y EXTERIORES)',
    
    '154104' => 
    
       'VENTA DE PAN Y OTROS PRODUCTOS SIMILARES EXENTOS DE VENTAS',
     
    '342001' => 
    
       'FABRICACION DE CARROCERIAS PARA VEHICULOS AUTOMOTORES, REMOLQUES Y SEMIREMOLQUES',
    
    '153301' => 
    
       'ELABORACION DE ALIMENTOS PARA ANIMALES DESTINADOS AL CONSUMO HUMANO',
      
    '526008' => 
    
       'REPARACION Y MANTENIMIENTO DE PERSIANAS Y CORTINAS',
    
    '523921' => 
  
       'VENTA AL POR MENOR DE CAJAS REGISTRADORAS, CALCULADORAS O MAQUINAS DE CONTABILIDAD',
    
    '359901' => 
  
       'FABRICACION DE OTROS TIPOS DE EQUIPO DE TRANSPORTE N.C.P.',
    
    '351101' => 

       'SERVICIO DE REPARACION DE EMBARCACIONES Y SUS PARTES Y/O ESTRUCTURAS FLOTANTES',
    
    '525904' => 

       'TRABAJOS DE MANUALIDADES',
     
    '519011' => 
    
       'COMERCIO AL POR MAYOR DE EQUIPO Y ACCESORIOS PARA PESCA DEPORTIVA O ARTESANAL',
    
    '853101' => 
    
       'FUNDACIONES DE BIEN SOCIAL',
    
    '512233' => 
  
       'COMERCIO AL POR MAYOR DE CARNES DE TODO TIPO (PREPARADAS SAZONADAS CONDIMENTADAS EMPANIZADAS)',
      
    '155103' => 
    
       'ELABORACION DE BEBIDAS CON PORCENTAJE DE ALCOHOL POR VOLUMEN MENOR AL 15%.',
    
    '014002' => 
    
       'RECOLECCION DE COSECHAS Y ACTIVIDADES CONEXAS',
    
    '523402' => 
  
       'VENTA AL POR MENOR ARTICULOS DE FERRETERIA PINTURAS MADERA Y MATERIALES PARA LA CONSTRUCCION',
    
    '930904' => 
    
       'CENTRO O SALA DE BRONCEADO',
    
    '751101' => 
  
       'ACTIVIDADES DE LA ADMINISTRACION PUBLICA EN GENERAL',
    
    '351102' => 
  
       'CONSTRUCCION DE EMBARCACIONES Y ESTRUCTURAS FLOTANTES',
    
    '453007' => 
    
       'INSTALACION Y MANTENIMIENTO DE ASCENSORES (ELEVADORES)',
     
    '514903' => 
   
       'VENTA AL POR MAYOR DE PRODUCTOS PARA USO AGROPECUARIO Y VENTA DESECHOS ORGANICOS E INORGANICOS',
    
    '221101' => 

       'EDICION DE LIBROS DE TEXTOS',
    
    '853301' => 
  
       'ASOCIACIONES DECLARADAS DE UTILIDAD PUBLICA POR EL PODER EJECUTIVO, INCLUYE LAS ASADAS',
    
    '523301' => 
    
       'COMERCIO AL POR MENOR DE OBJETOS DE CERAMICA Y PORCELANA',
    
    '930102' => 

       'SERVICIO DE TEÃ‘IDO DE PRENDAS DE VESTIR',
     
    '526002' => 
  
       'REPARACION DE MUEBLES Y ACCESORIOS DOMESTICOS',
    
    '525903' => 
    
       'VENTA DE LIBROS A DOMICILIO',
    
    '912002' => 
    
       'ACTIVIDADES DE SINDICATOS',
     
    '242903' => 
           'REPARACION DE ARMAS DE FUEGO',
    
    '242401' => 
    
       'FABRICACION JABONES, DETERGENTES, PREPARADOS DE TOCADOR, PARA LIMPIAR PULIR Y PERFUMES',
    
    '513710' => 
    
       'VENTA AL POR MAYOR DE LIBROS Y TEXTOS EDUCATIVOS',
    
    '851208' => 
   
       'OTORRINOLARINGOLOGIA, AUDIOLOGIA Y SERVICIOS CONEXOS.',
     
    '659905' => 
    
       'FONDOS DE INVERSION',
    
    '271001' => 
    
       'INDUSTRIAS BASICAS DE HIERRO Y ACERO',
    
    '514201' => 
 
       'VENTA AL POR MAYOR DE METALES Y MINERALES METALIFEROS',
   
    '522014' => 
 
       'VENTA DE FRUTAS INCLUIDAS EN LA NUEVA CANASTA BASICA',
   
   ' 523702' => 
   
       'VENTA AL POR MENOR DE REVISTAS/PERIODICOS EN PUESTOS DE VENTAS O MERCADOS',
      
    '512213' => 
    
       'COMERCIO AL POR MAYOR DE BEBIDAS GASEOSAS Y CARBONATADAS',
    
    '513601' => 
 
       'VENTA AL POR MAYOR EQUIPO MEDICO ACCESORIOS MEDICAMENTOS PRODUC FARMACEUTICO EXENTOS DE VENTAS',
     
    '522007' => 
  
       'LICORERIAS Y/O DEPOSITO DE LICORES (VENTA AL POR MENOR)',
     
    '552003' => 

       'SERVICIOS DE CATERING SERVICE',
    
    '552004' => 

       'SERVICIO DE RESTAURANTE, CAFETERIAS, SODAS Y OTROS EXPENDIOS DE COMIDA',
    
    '331101' => 
       'FABRICACION Y COMERCIALIZACION DE PROTESIS',
  
    '153302' => 
 
       'ELABORACION DE PACAS DE HENO',
    
    '522013' => 
    
       'COMERCIO AL POR MENOR DE AGUA EMBOTELLADA',
     
    '101001' => 
    
       'EXTRACCION Y AGLOMERACION DE CARBON DE PIEDRA',
    
    '452006' => 
   
       'ACTIVIDADES DE CONSTRUCCION ESPECIALES',
    
    '601001' => 
    
       'SERVICIO DE TRANSPORTE POR VIA FERREA',
     
    '011301' => 
    
       'CULTIVO DE ESPECIAS DE TODO TIPO',
    
    '521101' => 
 
       'SUPERMERCADOS Y ALMACENES DE ABARROTES EN CADENA',
    
    '523808' => 
    
       'VENTA AL POR MENOR DE REPUESTOS USADOS PARA MAQUINARIA, EQUIPO Y OTROS',
    
    '924901' => 
    
       'NIGHT CLUB/CABARETTE',
     
    '012103' => 
  
       'CRIA DE ANIMALES DOMESTICOS COMO: GANADO VACUNO , OVEJAS Y CABRAS',
     
    '242101' => 
    
       'FABRICACION DE PLAGUICIDAS Y OTROS PRODUCTOS QUIMICOS DE USO AGROPECUARIO',
      
    '232001' => 
   
       'REFINERIAS DE PETROLEO',
     
    '523404' => 
    
       'VENTA AL POR MENOR DE VIDRIO PARA LA CONSTRUCCION',
    
    '630001' => 
   
       'PRESTACION DE SERVICIOS DE TRANSITO AEREO',
    
    '142201' => 
    
       'EXTRACCION DE SAL',
    
    '851901' => 
    
       'MEDICINA ALTERNATIVA',
    
    '503005' => 
    
       'COMERCIALIZACION DE LLANTAS (NEUMATICAS) PARA VEHICULOS AUTOMOTORES',
    
    '515003' => 
    
       'VENTA AL POR MAYOR DE MAQUINARIA Y EQUIPO AGROPECUARIO',
     
    '523807' => 
    
       'VENTA AL POR MENOR DE REPUESTOS NUEVOS PARA MAQUINARIA, EQUIPO Y OTROS',
    
    '210201' => 
 
       'FABRICACION DE OTROS ARTICULOS DE PAPEL Y CARTON',
     
    '523913' => 
    
       'VENTA AL POR MENOR DE ANIMALES DOMESTICOS PARA MASCOTAS.',
     
    '522011' => 
    
       'PREPARACION, SERVICIO Y VENTA DE FRUTAS PICADAS Y BEBIDAS DE FRUTAS Y/O LEGUMBRES.',
   
    '521102' => 
 
       'VENTA AL POR MENOR DE ESPECIAS, SALSAS Y CONDIMENTOS',
     
    '153201' => 
  
       'ELABORACION DE PRODUCTOS DERIVADOS DE ALMIDON',
    
    '020001' => 
    
       'VENTA DE ARBOLES EN PIE (ARBOLES DE REFORESTACION)',
      
    '012401' => 
   
       'PRODUCCION Y VENTA DE HUEVOS DE CUALQUIER TIPO, EXCEPTO LOS DE GALLINA',
    
    '014005' => 

       'MANEJO E INSTALACION DE SISTEMAS DE RIEGO',
     
    '851210' => 
  
       'CARDIOLOGOS',
    
    '242902' => 
       'FABRICACION DE PEGAMENTOS Y ADHESIVOS',
    
    '741301' => 
    
       'ASESORES EN MERCADEO Y VENTAS',
    
    '269301' => 
  
       'FABRICACION DE PRODUCTOS DE ARCILLA Y CERAMICA NO REFRACTARIAS PARA USO ESTRUCTURAL',
     
    '014004' => 

       'SERVICIO DE PLANTACION DE ARBOLES Y SIMILARES',
     
    '809007' => 
   
       'ESCUELA Y AGENCIA DE MODELOS',
     
    '202902' => 
    
       'FABRICACION Y/O VENTA DE ATAUDES (CAJAS MORTUORIAS, FERETROS).',
    
    '660101' => 
    
       'PLANES DE SEGUROS DE VIDA',
      'descripcion' => 'EN ESTA CLASE SE INCLUYEN LA CONCERTACION DE CONTRATOS DE ANUALIDADES Y POLIZAS DE SEGUROS DE VIDA(INCLUSO EL REASEGURO SEGUROS DE INVALIDEZ Y SEGUROS DE DOBLE INDEMNIZACION, CONTENGAN O NO UN ELEMENTO IMPORTANTE DE AHORRO, MEDIANTE LOS CUALES SE CAPTAN E INVIERTEN FONDOS.',
    
    '921904' => 
    
       'SALON DE PATINES',
     
    '289306' => 
   
       'SERVICIOS DE TORNO (MECANICA DE PRECISION)',
    
    '519015' => 
    
       'VENTA AL POR MAYOR DE OTROS PRODUCTOS NO ESPECIALIZADOS',
    
    '922001' => 
    
       'ACTIVIDADES DE AGENCIAS DE NOTICIAS',
    
    '241201' => 

       'FABRICACION DE ABONOS Y COMPUESTOS DE NITROGENO',
     
    '725001' => 
    
       'REPARACION DE EQUIPO DE COMPUTO',
    
    '713013' => 
    
       'ALQUILER DE EQUIPO Y UTENSILIOS PARA EVENTOS ESPECIALES',
    
    '712302' => 
    
       'ALQUILER DE MAQUINARIA Y EQUIPO DE OFICINA',
    
    '515001' => 
 
       'VENTA AL POR MAYOR DE MAQUINARIA Y EQUIPO INDUSTRIAL, DE CONSTRUCCION, INGENIERIA CIVIL Y OTROS, ASI COMO SUS ACCESORIOS',
     
    '523901' => 
    
       'VENTA AL POR MENOR DE OTROS PRODUCTOS EN ALMACENES ESPECIALIZADOS',
     
    '611001' => 
   
       'SERVICIO DE TRANSPORTE DE CARGA Y/O CABOTAJE DE PASAJEROS POR VIA ACUATICA',
    
    '012202' => 
    
       'CRIA DE ANIMALES DOMESTICADOS (AVES DE CORRAL)',
     
    '525301' => 
    
       'VENTA DE LOTERIA EN AGENCIAS O AL DETALLE',
    
    '012303' => 
    
       'PRODUCCION DE QUESO Y OTROS PRODUCTOS LACTEOS EXENTOS DEL IMPUESTO SOBRE LAS VENTAS',
    
    '172201' => 
   
       'FABRICACION DE TAPICES Y ALFOMBRAS',
     
    '523101' => 
    
       'FARMACIAS',
    
    '725004' => 
           'MANTENIMIENTO DE CAJAS REGISTRADORAS, CALCULADORAS, MAQUINAS DE CONTABILIDAD',
     
    '630301' => 
 
       'PARQUEOS/ESTACIONAMIENTO DE VEHICULOS',
    
    '242201' => 

       'FABRICACION DE PINTURAS, BARNICES Y PRODUCTOS DE REVESTIMI-TO SIMILARES',
    
    '202101' => 
    
       'FABRICACION DE HOJAS DE MADERA PARA ENCHAPADO Y TABLEROS COMO PLYWOOD, DURPANEL Y SIMILARES',
    
    '722003' => 
    
       'DISEÃ‘ADOR GRAFICO, DE SOFWARE Y PAGINAS WEB',
    
    '523204' => 
  
       'PASAMANERIAS',
    
    '523929' => 
   
       'VENTA DE PERROS ENTRENADOS PARA SEGURIDAD',
    
    '153101' => 
    
       'SERVICIO DE MOLIENDA',
    
    '181002' => 
       'FABRICACION DE PRENDAS DE VESTIR (ROPA DE TODO TIPO)',
     
    '050003' => 
       'CRIADERO DE PECES, CRUSTACEOS O MOLUSCOS. COMERCIALIZACION DE LARVAS DE ESPECIES MARINAS',
     
    '659909' => 
    
       'FIDEICOMISOS DE LA LEY DEL SISTEMA DE BANCA PARA EL DESARROLLO',
   
    '930303' => 
    
       'CEMENTERIO O CAMPOSANTO PRIVADO',
      
    '353001' => 
    
       'FABRICACION DE AERONAVES Y NAVES ESPACIALES Y MAQUINARIA CONEXA',
     
    '732001' => 
  
       'INVESTIGACIONES Y DESARROLLO EXPERIMENTAL EN EL CAMPO DE LAS CIENCIAS SOCIALES Y LAS HUMANIDADES',
     
    '921402' => 
    
       'ACTIVIDADES MUSICALES Y ARTISTICAS (SERVICIOS)',
     
   '752302' => 
    
      
       'SERVICIOS DE VIGILANCIA O CONTROL PORTUARIA, COSTERA, AEREA,Y FRONTERIZA',
    
    '851203' => 
   
       'NEUROLOGOS',
     
    '513907' => 
    
       'VENTA AL POR MAYOR DE DISCOS COMPACTOS Y OTROS DISPOSITIVOSDE GRABACION',
    
    '523203' => 

       'VENTA AL POR MENOR DE CALZADO (ZAPATAERIAS)',
    
    '602101' => 
   
       'TRANSPORTE DE ESTUDIANTES Y EMPLEADOS',
    
    '671901' => 

       'PUESTOS Y/O CASAS DE CAMBIO DE MONEDA EXTRANJERA',
    
    '630903' => 

       'AGENCIA DE TRANSPORTE (NAVIERA)',
     
    '523308' => 
 
       'VENTA AL POR MENOR DE INSTRUMENTOS MUSICALES, PARTES Y ACCESORIOS',
      
    '749908' => 

       'SERVICIO DE RECOLECCION DE MONEDAS DE LOS TELEFONOS PUBLICOS',
    
    '512211' => 
  
       'COMERCIO AL POR MAYOR DE OTROS ALIMENTOS N.C.P. GRABADOS CON VENTAS',
    
    '853201' => 
   
       'GUARDERIAS/CENTROS INFANTILES Y SERVICIOS SOCIALES',
    
    '930903' => 
    
       'OTRAS ACTIVIDADES DE SERVICIOS PERSONALES N.C.P.',
    
    '451002' => 
    
       'PREPARACION DE TERRENOS',
    
    '851905' => 

       'PSICOLOGO',
    
    '453009' => 
   
       'REPARACION DE PORTONES ELECTRICOS',
     
    '749301' => 
    
       'SERVICIOS DE FUMIGACION (NO AGRICOLA)',
      
    '741402' => 

       'ASESORAMIENTO EMPRESARIAL Y EN MATERIA DE GESTION (HANDLING)',
    
    '452002' => 
   
       'CONSTRUCCION DE EDIFICIOS, APARTAMENTOS, CONDOMINIOS Y CASAS DE HABITACION',
    
    '013001' => 
    
       'CULTIVO DE PRODUCTOS AGRICOLAS EN COMBINACION CON LA CRIA DE ANIMALES (EXPLOTACI',
    
    '523302' => 
       'VENTA AL POR MENOR DE CRISTALERIA',
    
    '701002' => 
 
       'ALQUILER DE LOCALES COMERCIALES Y CENTROS COMERCIALES',
    
    '172902' => 
    
       'SERVICIO DE BORDADO A MANO O A MAQUINA',
    
    '523918' => 
    
       'VENTA AL POR MENOR DE EQUIPO DE AUDIO Y VIDEO',
    
    '519003' => 
    
       'VENTA AL POR MAYOR DE EQUIPO, ARTICULOS Y ACCESORIOS DE BELLEZA, COSMETICOS E HIGIENE PERSONAL',
    
    '900001' => 
    
       'ASESORAMIENTO Y ELIMINACION DE DESPERDICIOS, SANEAMIENTO (EXCEPTO LIMPIEZA DE TANQUES SEPTICOS)',
    
    '721001' => 
    
       'CONSULTORES INFORMATICOS',
    
    '523202'=> 
    
       'VENTA AL POR MENOR DE ROPA (BOUTIQUE)',
     
    '291202' => 
 
       'VENTA DE BOMBAS DE AGUA',
      
    '851902' => 
  
       'FISICO TERAPISTA',
    
    '343001' => 
    
       'FABRICACION DE PARTES, PIEZAS Y ACCESORIOS PARA VEHICULOS AUTOMOTORES',
    
    '523925' => 
   
       'VENTA AL POR MENOR DE EXTINTORES',
     
    '142901' => 
   
       'EXPLOTACION DE OTRAS MINAS Y CANTERAS N.C.P.',
    
    '551005' => 
    
       'SERVICIO DE SALAS VIP Y PREMIUN EN AEROPUERTOS',
  
    '660202' => 
    
       'FONDOS DE OPERADORAS DE PENSIONES EXENTOS DEL 8%',
    
    '751102' => 
    
       'ACTIVIDADES DE LA ADMINISTRACION PUBLICA EN GENERAL, NO SUJETAS AL IMPUESTO SOBRE LA RENTA',
    
    '151201' => 

       'ELABORACION Y CONSERVACION DE PESCADO Y SUS DERIVADOS',
     
    '289302' => 
    
       'HOJALATERIA',
    
    '514101' => 
  
       'VENTA AL POR MAYOR DE COMBUSTIBLES SOLIDOS (LEÃ‘A Y SIMILARES)',
     
    '960105' => 

       'ACTIVIDADES PREOPERATIVAS O DE ORGANIZACION',
    
    '292401' => 
  
       'FABRICACION DE MAQUINARIA PARA LA EXPLOTACION DE MINAS Y CANTERAS Y PARA OBRAS DE CONSTRUCCION',
     
    '851911' => 

       'PROFESIONALES EN SALUD OCUPACIONAL',
    
    '011201' => 
  
       'CULTIVO HORTALIZAS LEGUMBRES ESPECIALIDADES HORTICOLAS PRODUCTOS DE VIVERO EXCENTO DE VENTAS',
    
    '552001' => 
 
       'BARES, CANTINAS O TABERNAS',
     
    '011903' => 
    
       'VIVEROS',
    
    '523908' => 
   
       'COMERCIO AL POR MENOR DE ANIMALES DOMESTICOS PARA CONSUMO HUMANO',
    
    '701007' => 
    
       'EXPLOTACION DE FRANQUICIAS',
    
    '851204' => 
 
       'SERVICIOS DE OFTALMOLOGO U OCULISTA, OPTOMETRISTA Y/O OPTICO',
    '711102' => 
 
       'ALQUILER DE MOTOCICLETA/SERVICIO',
    
    '851201' => 
  
       'GINECOLOGO',
     
    '242202' => 
    
       'FABRICACION DE TINTAS DE IMPRENTAS',
    
    '671902' => 
 
       'ASESORES FINANCIEROS Y ACTIVIDADES AUXILIARES DE LA INTERMEDICIACION FINANCIERA',
     
    '921902' => 
    
       'VENTA AL POR MENOR DE DISCOS, GRABACIONES DE MUSICA Y DE VIDEO',
     
    '960106' => 
   
       'EXPORTADORES DE MERCANCIAS EXENTAS DEL IMPUESTO DE VENTAS',
    
    '015001' => 
    
       'CAZA ORDINARIA MEDIANTE TRAMPAS, Y REPOBLACION DE ANIMALES DE CAZA, INCLUSO LAS ACTIVIDADES DE SERVICIOS CONEXAS.',
    
    '924907' => 
    
       'SERVICIO DE ENLACE DE LLAMADAS Y CASAS DE APUESTAS ELECTRONICAS (SPORTBOOKS)',
    
    '291401' => 
  
       'FABRICACION DE HORNOS Y QUEMADORES',
    
    '651906' => 
   
       'INSTITUCIONES DE AHORRO Y CREDITO PARA VIVIENDA',
    
    '154401' => 
   
       'ELABORACION DE MACARRONES, FIDEOS, ALCUZCUZ Y PRODUCTOS FARINACEOS SIMILARES',
    
    '751302' => 
    
       'FONDOS PUBLICOS (ACTIVIDADES DEL SECTOR PUBLICO)',
    
    '749102' => 
  
       'SERVICIOS DE BIBLIOTECOLOGIA',
   
    '014003' => 
   
       'FUMIGACION DE CULTIVOS',
    
    '749905' => 
    
       'SERVICIO DE CONTESTACION DE TELEFONOS (CALL CENTER)',
     
    '372001' => 
   
       'RECICLAJE DE OTRO TIPO DE MATERIALES N.C.P.',
    
    '523907' => 
 
       'FLORISTERIA',
    
    '809006' => 
    
       'ENSEÃ‘ANZA DE LA DE SEGURIDAD PRIVADA',
    
    '402001' => 
  
       'FABRICACION DE GAS; DISTRIBUCION DE COMBUSTIBLES GASEOSOS POR TUBERIAS',
    
    '712909' => 
 
       'ALQUILER DE OTROS TIPOS DE MAQUINARIA Y EQUIPO PARA USO COMERCIAL',
     
    '525901' => 
    
       'VENTA DE ALFOMBRAS Y TAPICES',
     
    '160001' => 
    
       'ELABORACION DE PRODUCTOS DE TABACO',
     
    '155404' => 
    
       'ELABORACION DE CONCENTRADO PARA BEBIDAS NATURALES Y GASEOSAS',
    
    '155301' => 

       'ELABORACION DE BEBIDAS MALTEADAS Y DE MALTA NO ARTESANALES',
    
    '523930' => 
  
       'VENTA AL POR MENOR DE PIEZAS DE BAMBU',
     
    '269101' => 

       'FABRICACION PRODUCTOS DE CERAMICA BARRO LOZA Y/O PORCELANANO REFRACTARIA USO NO ESTRUCTURAL',
    
    '630901' => 
    
       'AGENCIAS ADUANALES, ALMACENES FISCALES Y ESTACIONAMIENTOS TRANSITORIOS',
     
    '930103' => 
 
       'SERVICIO DE LIMPIEZA Y LAVADO DE MUEBLES',
    
    '172103' => 
 
       'DISEÃ‘O ARTISTICO PARA COSTURA (MOLDES)',
      
    '269601' => 
    
       'CORTE, TALLADO Y ACABADO DE LA PIEDRA',
     
    '522010' => 
  
       'VENTA DE EMBUTIDOS Y CARNES (RES, POLLO, CERDO)INCLUIDAS ENLA CANASTA BASICA',
     
    '523902' => 
 
       'VENTA AL POR MENOR DE SUMINISTROS Y/0 EQUIPO DE OFICINA',
    
    '154202' => 
 
       'ELABORACION DE PRODUCTOS DERIVADOS DE LA CAÃ‘A DE AZUCAR',
    
    '713001' => 
   
       'ALQUILER DE MENAJE',
    
    '743001' => 
    
       'PUBLICIDAD',
    
    '369901' => 
   
       'FABRICACION DE ESCOBAS',
    
    '630904' => 
    
       'AGENTE ADUANERO FISICO O JURIDICO',
    
    '524004' => 
    
       'VENTA DE LIBROS USADOS',
     
    '924905' => 
    
       'SALA DE VIDEO JUEGOS',
     
    '660301' => 
    
       'AGENTES DE SEGUROS',
     
    '359101' => 
    
       'FABRICACION DE MOTOCICLETAS, PARTES, PIEZAS Y SUS ACCESORIOS',
     
    '526006' => 
    
       'REPARACION DE ARTICULOS ELECTRICOS',
    
    '526003' => 
    
       'REPARACION DE BICICLETAS',
     
    '924903' => 
  
       'PESCA DEPORTIVA',
     
    '751401' => 
    
       'ACTIVIDADES DE SERVICIOS AUXILIARES PARA LA ADMINISTRACION PUBLICA EN GENERAL',
     
    '513101' => 
 
       'VENTA AL POR MAYOR DE CALZADO, PRODUCTOS TEXTILES; PRENDASDE VESTIR;',
    
    '851213' => 
   
       'REUMATOLOGO',
    
    '930915' => 
  
       'ALBERGUE Y CUIDO DE ANIMALES A DOMICILIO',
    
    '201002' => 
    
       'ASERRADO Y ACEPILLADURA DE MADERA',
    
    '990001' => 
       'ACTIVIDADES DE ORGANIZACIONES Y ORGANOS EXTRATERRITORIALES, INCLUYE LAS EMBAJADAS DE OTROS PAISES',
    
    '154902' => 
    
       'FABRICACION DE PRODUCTOS ALIMENTICIOS PREPARADOS N.C.P. (NOCONTEMPLADOS EN OTRA PARTE)',
     
    '630201' => 
   
       'SERVICIOS DE ALMACENAJE',
      
    '712905' => 
    
       'ALQUILER DE EQUIPO PARA RADIO, TELEVISION Y COMUNICACIONES',
    
    '702002' => 
    
       'ARRENDAMIENTO O ALQUILER DE BIENES INMUEBLES MEDIANTE CONTRATO VERBAL O ESCRITO.',
    
    '523923' => 
    
       'VENTA AL POR MENOR DE REPUESTOS PARA ELECTRODOMESTICOS',
    
    '551001' => 
   
       'ALQUILER DE BIENES INMUEBLES DE USO HABITACIONAL POR PERIODOS INFERIORES A UN MES (CASAS DE ESTANCIA TRANSITORIA, CASA DE HUESPEDES, CABINAS, CAMPAMENTOS, ENTRE OTROS)',
    
    '659202' => 
   
       'EMISORAS Y PROCESADORAS DE TARJETAS DE CREDITO',
    
    '731003' => 
    
       'METEOROLOGO POR CUENTA PROPIA',
    
    '930101' => 
    
       'SERVICIOS DE LAVANDERIA DE TODO TIPO',
    
    '502001' => 
  
       'LAVADO, ENCERADO Y PULIDO DE AUTOMOVILES (LAVA CAR)',
      
    '602201' => 
    
       'SERVICIOS DE BUS/MICROBUS (EXCURSIONES)',
    
    '155302' => 
    
       'ELABORACION ARTESANAL DE BEBIDAS MALTEADAS Y DE MALTA.',
      
    '519013' => 
    
       'COMERCIALIZACION AL POR MAYOR DE SUPLEMENTOS ALIMENTICIOS',
     
    '621003' => 
    
       'SERVICIO DE PILOTAJE',
    
    '919903' => 
       'ASOCIACIONES O ENTIDADES CON FINES CULTURALES, SOCIALES, RECREATIVOS, ARTESANALES, ETC.',
    
    '290003' => 
    
       'SERVICIO DE MANTENIMIENTO DE MAQUINARIA Y EQUIPO.',
    
    '642002' => 
    
       'SERVICIO DE TELEVISION POR CABLE, SATELITE U OTROS SISTEMA S SIMILARES',
    
    '729001' => 
   
       'OTRAS ACTIVIDADES DE INFORMATICA',
    
    '522004' => 
   
       'VENTA DE VERDURAS Y FRUTAS EXENTOS DE VENTAS',
      
    '504003' => 
    
       'VENTA DE PARTES O ACCESORIOS DE MOTOCICLETAS',
     
    '630403' => 
    
       'GUIA DE TURIMO Y SERVICIO DE ASESORIA EN VIAJES Y TURISMO',
    
    '522001' => 
   
       'COMERCIO AL POR MENOR DE CONFITES Y OTROS PRODUCTOS RELACIONADOS (CONFITERIA)',
    
    '242406' => 
  
       'FABRICACION DE CERAS, ABRILLANTADORES, LUSTRADORES Y PREPARACIONES SIMILARES',
    
    '552007' => 
    
       'OTROS EXPENDIOS DE COMIDAS',
    
    '359201' => 
   
       'FABRICACION DE BICICLETAS Y SILLAS DE RUEDAS, PARTES, PIEZAS Y SUS ACCESORIOS',
    
    '522017' => 
    
       'VENTA DE EMBUTIDOS Y CARNES (RES, POLLO, CERDO, CABALLO,ETC) GRAVADAS EN VENTAS',
    
    '182001' => 
         'ADOBO Y TEÃ‘IDO DE PIELES; FABRICACION DE ARTICULOS DE PIEL NATURAL O ARTIFICIAL',
    
    '802201' => 
       'ENSEÃ‘ANZA SECUNDARIA DE FORMACION TECNICA Y PROFESIONAL',
    
    '749901' => 
    
       'SERVICIO DE FOTOCOPIADO Y OTROS',
    
    '192001' => 
    
       'FABRICACION DE TODO TIPO DE ZAPATOS EXEPTO EL ORTOPEDICO',
     
    '523927' => 
    
       'REPARACION EQUIPO DE AUDIO Y VIDEO',
     
    '020002' => 
    
       'EXTRACCION Y/O VENTA DE MADERA',
     
    '403001' => 

       'SUMINISTRO DE VAPOR Y AIRE ACONDICIONADO',
     
    '749402' => 
  
       'SERVICIO DE FOTOGRAFIA (FOTOGRAFO)',
     
    '741101' => 
    
       'BUFFETE DE ABOGADO, NOTARIO, ASESOR LEGAL',
    
    '523905' => 
    
       'VENTA AL POR MENOR DE JOYERIA, RELOJERIA Y BISUTERIA',
   
    '725002' => 
  
       'MANTENIMIENTO DE EQUIPO DE COMPUTO',
      
    '742112' => 
    
       'DIBUJANTE ARQUITECTONICO Y/O PLANOS DE CONSTRUCCION',
    
    '523209' => 
    
       'VENTA AL POR MENOR DE MATERIALES PARA CALZADO',
     
    '523201' => 
  
       'BAZARES',
      
    '453001' => 
    
       'REPARACION DE AIRE ACONDICIONADO',
    
    '341001' => 
    
       'FABRICACION DE VEHICULOS AUTOMOTORES',
    
    '519010' => 

       'COMERCIALIZACION Y DISTRIBUCION AL POR MAYOR DE ALIMENTOS PREPARADOS PARA ANIMALES.',
    
    '919902' => 
  
       'ASOCIACIONES DE CLUBES SOCIALES',
     
    '153103' => 
    
       'PROCESAMIENTO DEL ARROZ',
   
    '402002' => 
    
       'INSTALACION Y VENTA DE TANQUES PARA GAS',
  
    '401002' => 
    
       'GENERACION Y/O DISTRIBUCION DE ENERGIA ELECTRICA (HIDRAULICA,CONVENCIONAL, TERMICO, ETC)',
    
    '012102' => 
   
       'PRODUCCION DE SEMEN BOVINO, VENTA DE SEMEN CONGELADO Y DILUYENTE PARA SEMEN',
     
    '526005' => 
    
       'ELECTRICISTA, SERVICIOS',
    
    '369904' => 
 
       'FABRICACION DE SELLOS DE MANO, METAL O HULE (CAUCHO)',
    
    '512235' => 
   
       'VENTA AL POR MAYOR DE LEGUMBRES Y HORTALIZAS GRABADOS EN VENTAS',
    
    '930905' => 
 
       'PELUQUERIA Y SALA DE ESTETICA PARA ANIMALES',
    
    '154303' => 
    
       'ELABORACION DE DULCES, GOLOSINAS Y CONSERVAS EN AZUCAR',
     
    '713012' => 
    
       'ALQUILER DE TRAJES DE TODO TIPO',
     
    '502008' => 
    
       'SERVICIO DE REVISION TECNICA VEHICULAR (DIAGNOSTICOS POR ESCANNER)',
    
    '749401' => 
    
       'ESTUDIOS FOTOGRAFICOS',
    
    '742202' => 
  
       'PROFESIONALES EN TECNOLOGIA DE ALIMENTOS',
    
    '712301' => 
    
       'SERVICIOS DE INTERNET EN LOCALES PUBLICOS ( CAFE INTERNET )',
    
    '269401' => 
  
       'FABRICACION Y/O VENTA DE CEMENTO, CAL Y YESO',
     
    '292501' => 
    
       'FABRICACION DE MAQUINARIA PARA LA ELABORACION DE ALIMENTOS,BEBIDAS Y TABACO',
    
    '851909' => 
   
       'SERVICIOS DE RADIOLOGIA, ANESTESIOLOGIA Y OTROS',
    
    '741203' => 
    
       'ACTVIDAD DE CONTABILIDAD (CONTADORES TENEDURIA DE LIBROS,AUDITORIA Y ASESOR FISCAL',
     
    '525902' => 
    
       'VENTA AMBULANTE DE ARTICULOS PARA EL HOGAR',
    
    '011329' => 
    
       'CULTIVO DE PLANTAS PARA PREPARAR BEBIDAS Y MEDICINAS',
     
    '273101' => 
   
       'FUNDICION DE HIERRO Y ACERO',
     
    '503002' => 
    
       'VENTA DE REPUESTOS USADOS PARA AUTOMOVILES',
    
    '749204' => 
    
       'INSTRUCTOR DE TIRO (MANEJO DE ARMAS).',
   
    '523801' => 
    
       'VENTA AL POR MENOR DE ARMAS (ARMERIAS)',
     
    '515203' => 
   
       'VENTA AL POR MAYOR DE EQUIPO Y SUMINISTROS DE OFICINA',
    
    '522008' => 
 
       'VENTA AL POR MENOR DE HUEVOS DE GALLINA INCLUIDOS EN LA CANASTA BASICA',
   
    '712910' => 
    
       'ALQUILER DE MAQUINARIA Y/O EQUIPO PARA USO INDUSTRIAL',
    
    '659901' => 
    
       'SERVICIO DE PRESTAMO (PRESTAMISTAS)',
     
    '524005' => 
    
       'VENTA DE MONEDAS, BILLETES,ESTAMPILLAS (NUEVAS Y USADAS) PARA COLECCION Y ESPECIES FISCALES',
     
    '803003' => 
  
       'ENSENANZA SUPERIOR PUBLICA (UNIVERSIDADES)',
    
    '651101' => 
    
       'BANCA CENTRAL',
     
    '621001' => 
    
       'SERVICIO DE TRANSPORTE DE CARGA POR VIA AEREA',
    
    '851212' => 
    
       'ONCOLOGOS',
    
    '523703' => 
       'VENTA AL POR MENOR DE LIBROS Y TEXTOS EDUCATIVOS EXENTOS DE VENTAS',
    
    '512102' => 
       'VENTA AL POR MAYOR DE FLORES Y PLANTAS DE TODO TIPO',
    
    '281301' => 
       'FABRICACION GENERADORES DE VAPOR, EXCEPTO CALDERAS DE AGUA CALIENTE PARA CALEFACION CENTRAL',
     
    '912001' => 
       'ASOCIACIONES SOLIDARISTAS',
     
    '525101' => 

       'VENTA AL POR MENOR DE TODO TIPO DE ARTICULOS POR CATALOGO',
      
    '921501' => 
  
       'CANALES DE TELEVISION',
      
    '851907' => 
   
       'SERVICIOS DE ENFERMERIA',
     
    '630401' => 
    
       'AGENCIAS DE VIAJES Y EXCURSIONES',
    
    '642008' => 
   
       'SERVICIO DE TRANSMISION DE DATOS, TEXTO, SONIDO, VOZ Y VIDEO POR MEDIO DE LA RED DE INTERNET',
     
    '742108' => 
   
       'ACTIVIDADES DE GEOGRAFIA Y/O GEOLOGIA',
    
    '851209' => 
  
       'FARMACEUTICO O BOTICARIO',
     
    '515201' => 
   
       'VENTA AL POR MAYOR DE EQUIPO DE COMPUTO, SUS PARTES Y ACCESORIOS',
     
    '512209' => 
  
       'COMERCIO AL POR MAYOR DE PRODUCTOS DE TABACO',
     
    '512225' => 
  
       'COMERCIO AL POR MAYOR DE BEBIDAS NO ALCOHOLICAS (JUGOS DEFRUTAS, VEGETALES) Y AGUA EMBOTELLADA',
    
    '222105' => 
   
       'SERIGRAFIA',
    
    '711105' => 
    
       'ALQUILER DE CARRITOS DE GOLF Y OTROS',
     
    '050002' => 
    
       'PESCADORES ARTESANALES MEDIOS',
    
    '120001' => 
   
       'EXTRACCION DE MINERALES DE URANIO Y TORIO',
    
    '502003' => 
    
       'SERVICIO DE REPARACION DE TODA CLASE DE VEHICULOS Y SUS PARTES',
    
    '742107' =>
       'QUIMICOS',
      
    '749501' => 
    
       'SERVICIOS DE ENVASE Y EMPAQUE',
    
    '712101' => 
    
       'ALQUILER DE MAQUINARIA Y EQUIPO PARA USO AGRICOLA',
    
    '621002' => 
   
       'SERVICIO DE TRANSPORTE AEREO REGULADO DE PASAJEROS (LINEAS AEREAS)',
    
    '921302' => 
    
       'PROGRAMACION Y TRANSMISIONES DE RADIO Y TELEVISION',
    
    '153102' => 
   'ELABORACION DE HARINAS PREMEZCLADAS Y PREPARADAS PARA LA FABRICACION DE PRODUCTOS DE PANADERIA Y REPOSTERIA',
      
    
    '151103' => 
    
       'PRODUCCION DE CUEROS Y PIELES SIN CURTIR',
    
    '911101' => 
  
       'ACTIVIDADES DE ORGANIZACIONES EMPRESARIALES Y DE EMPLEADORES',
    
    '523407' => 
    
       'VENTA AL POR MENOR DE PLYWOOD',
    
    '512208' => 
    
       'COMERCIO AL POR MAYOR DE PRODUCTOS DE CONFITERIA',
    
    '522003' => 
  
       'MACROBIOTICAS',
    
    '851916' => 
   
       'OTRAS ACTIVIDADES RELACIONADAS CON LA SALUD HUMANA(BANCO DE SANGRE, BANCO DE PIEL, ETC)',
      
    '659101' => 
  
       'ARRENDAMIENTO OPERATIVO EN FUNCION FINANCIERA CON OPCION DECOMPRA O RENOVACION (LEASING OPERATIVO)',
    
    '241302' => 
   
       'FABRICACION DE RESINAS',
    
    '725003' => 
   
       'REPARACION CAJAS REGISTRADORAS, CALCULADORAS, MAQUINAS DE CONTABILIDAD Y EQUIPO DE OFICINA',
     
    '602304' => 
 
       'SERVICIO DE MUDANZA INTERNACIONAL',
     
    '523919' => 
  
       'VENTA AL POR MENOR DE ARTICULOS Y ACCESORIOS ORTOPEDICOS',
     
    '111001' => 
   
       'EXTRACCION DE PETROLEO CRUDO Y GAS NATURAL',
    
    '172302' => 
    
       'FABRICACION DE HILOS Y CUERDAS PARA USO AGRICOLA Y PESCA',
    
    '522005' => 

       'VENTA DE MARISCOS Y/O PESCADO (PESCADERIAS O MARISQUERIAS)INCLUIDOS EN LA CANASTA BASICA',
     
    '202201' => 

       'FABRICACION DE PARTES Y PIEZAS DE CARPINTERIA PARA EDIFICIOS Y CONSTRUCCIONES',
    
    '513501' => 
 
       'VENTA AL POR MAYOR DE PREPARADOS Y/O ARTICULOS PARA LA LIMPIEZA DE USO GENERAL',
     
    '924902' => 

       'CASINOS Y SALAS DE JUEGOS',
      
    '505002' => 
       'VENTA DE LUBRICANTES, ACEITES, GRASAS Y PRODUCTOS DE LIMPIEZA PARA AUTOMOTORES',
      
    '273201' => 

       'FUNDICION DE METALES NO FERROSOS',
      
    '525201' => 
  
       'VENTA DE REVISTAS/PERIODICOS (PUESTOS CALLEJEROS)',
    
    '242301' => 
    
       'FABRICACION PRODUCTOS FARMACEUTICOS, SUSTANCIAS QUIMICAS YPRODUCTOS BOTANICOS',
     
    '154201' => 
    
       'ELABORACION DE AZUCAR Y SUS DERIVADOS',
    
    '712906' => 
  
       'ALQUILER DE EQUIPO PARA DISPENSAR BILLETES (CAJERO AUTOMA-TICO)',
     
    '171201' => 
    
       'MAQUILA DE PRODUCTOS TEXTILES',
    
    '314001' => 
    
       'FABRICACION DE PILAS, BATERIAS Y ACUMULADORES',
    
    '172301' => 
    
       'FABRICACION DE CUERDAS, CORDELES,BRAMANTES Y REDES',
    
    '924904' => 
  
       'ACTIVIDADES DE JUEGOS DE BILLAR, POOL Y OTROS SIMILARES',
      
    '523903' => 
   
       'VENTA AL POR MENOR DE FLORES',
    
    '242405' => 
    
       'FABRICACION DE JABONES PARA LAVAR (EXCEPTO PREPARACIONES TENSO ACTIVAS USADAS COMO JABON)',
    
    '292402' => 
  
       'PERFORACION DE POZOS',
  
    '919201' => 
 
       'ACTIVIDADES DE ORGANIZACIONES POLITICAS',
     
    '851914' => 
    
       'SERVICIOS DE PARAMEDICOS',
    
    '322001' => 
    
       'FABRICACION DE TRANSMISORES Y RECEPTORES DE RADIO Y TELEVISION',
    
    '523805' => 
    
       'VENTA AL POR MENOR DE PURIFICADORES DE AGUA, SUS PARTES Y REPUESTOS',
    
    '171101' => 
    
       'FABRICACION DE TODO TIPO DE TELAS Y/O HILOS',
    
    '272001' => 
   
       'FABRICACION DE PRODUCTOS PRIMARIOS DE METALES PRECIOSOS Y METALES NO FERROSOS',
      
    '011401' => 
   
       'CULTIVO DE CAFE',
    
    '242404' => 
    
       'FABRICACION DE JABONES DE TOCADOR',
      
    '911201' => 
    
       'ACTIVIDADES DE ORGANIZACIONES PROFESIONALES',
    
    '142902' => 
   
       'VENTA DE ASFALTO/MEZCLA ASFALTICA',
    
    '514103' => 
    
       'VENTA AL POR MAYOR DE COMBUSTIBLES SOLIDOS (CARBON)',
    
    '455001' => 
    
       'ALQUILER DE EQUIPO DE CONSTRUCCION O DEMOLICION CON OPERADORES',
   
    '851202' => 
   
       'SERVICOS DE MEDICO GENERAL',
    
    '453004' => 
  
       'REPARACION DE CABLEADO DE COMUNICACIONES',
   
    '809005' => 
   
       'ENSEÃ‘ANZA CULTURAL',
    
    '602202' => 
 
       'SERVICIO DE TAXI',
    
    '659903' => 
   
       'SOCIEDADES DE INVERSION MOBILIARIA',
    
    '223001' => 
   
       'REPRODUCCION DE GRABACIONES',
    
    '289305' => 
   
       'FABRICACION DE PIEZAS, ARTICULOS Y ACCESORIOS DE METAL INCLUYE LAS CERRAJERIAS.',
    
    '924106' => 
   
       'ACTIVIDADES DEPORTIVAS Y OTRAS POR CUENTA PROPIA',
    
    '659907' => 
    
       'INGRESOS POR INTERESES DIFERENTES AL COMERCIO DEL PRESTAMO',
    
    '519006' => 
   
       'DISTRIBUCION Y COMERCIALIZACION AL POR MAYOR DE MATERIAL DEEMPAQUE',
    
    '924104' => 
    
       'ACTIVIDADES DE ESCUELAS Y CLUBES DEPORTIVOS',
   
    '521202' => 
 
       'PULPERIAS ( MINI-SUPER)(SIN CANTINA)',
    
    '923301' => 
   
       'ACTIVIDADES DE JARDINES BOTANICOS, ZOOLOGICOS, PARQUES NACIONALES Y RESERVAS NACIONALES',
    
    '369401' => 
   
       'FABRICACION DE JUEGOS Y JUGUETES',
    
    '012403' => 
    
       'PRODUCCION DE HUEVOS DE GALLINA INCLUIDOS EN LA CANASTA BASICA',
    
    '660302' => 
    
       'COMERCIALIZADORES DE SEGUROS',
    
    '173001' => 
    
       'FABRICACION DE TEJDOS Y ARTICULOS DE PUNTO Y GANCHILLO',
    
    '923101' => 
   
       'ACTIVIDADES DE BIBLIOTECAS Y ARCHIVOS',
    
    '701003' => 
    
       'COMPRA Y VENTA DE PROPIEDADES (INVERSIONISTAS)',
    
    '523904' => 
    
       'SERVICIOS DE REPARACION DE JOYERIA Y RELOJERIA EN GENERAL',
    
    '251101' => 
    
       'FABRICACION DE LLANTAS Y CUBIERTAS PARA EQUIPO Y MAQUINARIAMOVIL',
    
    '512234' => 
   
       'COMERCIO AL POR MAYOR DE CERVEZA IMPORTADA',
    
    '630101' => 
   
       'SERVICIO DE CONSOLIDACION DE CARGA Y DESCARGA',
    
    '514308' => 
   
       'VENTA AL POR MAYOR DE MATERIALES PARA LA CONTRUCCION, ARTICULOS DE FERRETERIA, EQUIPO Y MATERIALES DE FONTANERIA Y CALEFACCION',
    
    '514901' => 
    
       'VENTA AL POR MAYOR Y AL POR MENOR DE CHATARRA',
    
    '743006' => 
    
       'PUBLICIDAD A TRAVES DE MEDIOS ELECTRONICOS',
    
    '453006' => 
    
       'INSTALACION Y MANTENIMIENTO DE AIRE ACONDICIONADO',
    
    '011901' => 
    
       'CULTIVO DE FLORES DE TODO TIPO',
    
    '742105' => 
   
       'SERVICIOS DE CONSULTORIA EN MANTENIMIENTO INDUSTRIAL Y MECANICO',
    
    '012402' => 
  
       'PRODUCCION Y VENTA DE MIEL DE ABEJA NATURAL',
    
    '251102' => 
    
       'REENCAUCHADORA DE LLANTAS',
    
    '293001' => 
    
       'FABRICACION DE APARATOS DE USO DOMESTICO N.C.P.',
    
    '659906' => 

       'ACTIVIDADES DE SOCIEDADES DE CARTERA (HOLDING)',
    
    '372003' => 
   
       'RECICLAJE DE PAPEL Y PLASTICO Y MATERIALES RELACIONADOS',
    
    '659201' => 
  
       'ENTIDADES FINANCIERAS DISTINTAS AL SISTEMA BANCARIO NACIONAL',
    
    '210101' => 
   
       'FABRICACION DE PAPEL Y CARTON Y ENVASES DE PAPEL Y CARTON',
   
    '525905' => 
    
       'VENTA AL POR MENOR DE TARJETAS TELEFONICAS, PINES, TIEMPO AIRE Y SIMILARES',
 
    '369903' => 

       'FABRICACION DE VELAS COLOREADAS, PERFUMADAS Y DECORADAS',
   
    '660303' => 
  
       'PLANES DE SEGUROS GENERALES',
     
    '713002' => 
  
       'ALQUILER DE EQUIPO RECREATIVO Y DEPORTIVO',
    
    '526001' => 
    
       'SERVICIOS DE REPARACION DE ZAPATOS',
    
    '602001' => 
  
       'SERVICIO DE TRANSPORTE DE CARGA POR VIA TERRESTRE',
    
   '503004' => 
    
       'VENTA DE REPUESTOS NUEVOS PARA AUTOMOVILES',
    
    '292101' => 
  
       'FABRICACION DE MAQUINARIA AGRICOLA',
    
    '524002' => 
 
       'CASA DE EMPEÃ‘O Y AFIN',
    
    '960109' => 
    
       'DISTRIBUIDORES DE MERCANCIAS EXENTAS DEL IMPUESTO DE VENTAS',
    
    '361001' => 
   
       'FABRICACION Y/O REPARACION DE MUEBLES Y ACCESORIOS (INCLUYECOLCHONES)',
    
    '291101' => 
    
       'FABRICACION DE MOTORES Y TURBINAS, EXCEPTO MOTORES PARA AERONAVES, VEHICULOS AUTOMOTORES Y MOTOCICLETAS',
    
   '930901' => 
    
       'SALAS DE MASAJES',
    
    '292601' => 
  
       'FABRICACION MAQUINARIA PARA LA ELABORACION DE PRODUCTOS TEXTILES, PRENDAS DE VESTIR Y CUEROS',
    
    '712904'=> 
    
       'ALQUILER DE PELICULAS CINEMATOGRAFICAS (VIDEO CLUB) Y/O VIDEO JUEGOS',
    
    '332001' => 
  
       'FABRICACION DE INSTRUMENTOS OPTICOS Y EQUIPO FOTOGRAFICO',
    
    '603001' => 
 
       'SERVICIO DE TRANSPORTE POR TUBERIAS',
    
    '642003' => 
    
       'MANTENIMIENTO DE REDES DE TELECOMUNICACION',
    
    '502002' => 
 
       'SERVICIOS DE ALINEAMIENTO Y REPARACION DE LLANTAS',
    
    '523911' => 
    
       'VENTA AL POR MENOR DE PAÃ‘ALES DESECHABLES, ARTICULOS DE LIMPIEZA Y OTROS (PAÃ‘ALERA)',
    
    '514701' => 
    
       'VENTA AL POR MAYOR DE PRODUCTOS,SUSTANCIAS O REACTIVOS QUIMICOS Y SOLVENTES EN GENERAL',
   
    '513604' => 
  
       'VENTA AL POR MAYOR EQUIPO MEDICO ACCESORIO MEDICAMENTO PRODUC FARMACEUTICO GRABADOS CON VENTAS',
    
    '749906' => 
  
       'SERVICIOS DE TRADUCTOR',
    
    '11701' =>
       'CULTIVO DE CAÃ‘A DE AZUCAR',
    
    '289201' => 
 
       'SERVICIOS DE SOLDADURA',
    
    '651904' => 
 
       'ENTIDADES FINANCIERAS PRIVADAS (BANCOS)',
    
    '551002' => 

       'HOTEL',
     
    '642007' => 
 
       'VENTA DE ESPACIO EN EL CABLE SUBMARINO',
    
    '630303' => 
   
       'PERMISO DE PASO POR PROPIEDADES PRIVADAS (PEAJE)',
    
    '602203' => 
   
       'SERVICIO ESPECIAL ESTABLE DE TAXI',
    
    '523102' => 
    
       'VENTA AL POR MENOR DE COSMETICOS Y PERFUMERIA',
    
    '630302' => 
    
       'FUNCIONAMIENTO DE INSTALACIONES TERMINALES COMO PUERTOS, MUELLES Y AEROPUERTOS',
    
    '523207' => 
    
       'VENTA AL POR MENOR DE ARTICULOS DE CUERO (EXCEPTO CALZADO)',
   
    '923201' => 
  
       'GALERIAS DE ARTE',
    
    '749918' => 
   
       'AGENCIAS DE COBRO Y CALIFICACION CREDITICIA',
    
    '930202' => 
    
       'SALONES DE BELLEZA, PELUQUERIA Y BARBERIA',
  
    '452005' => 
 
       'CONSTRUCCION Y MANTENIMIENTO DE CARRETERAS Y OTRAS VIAS',
    
    '651901' => 

       'COOPERATIVAS DE AHORRO Y CREDITO',
      
    '151101' => 
    
       'PRODUCCION, PROCESAMIENTO Y CONSERVACION DE CARNES Y EMBUTIDOS (SIN ENVASAR)',
    
    '713003' => 

       'ALQUILER DE EQUIPO MEDICO Y ARTICULOS CONEXOS',
    
    '171202' => 
   
       'HIDROFUGADO/IMPERMEABILIZADO',
     
    '154101' => 
  
       'VENTA DE PAN Y OTROS PRODUCTOS SIMILARES GRABADOS CON VENTAS',
     
    '921903' => 

       'SALON DE BAILE Y DISCOTECA',

    '659904' => 
  
       'SERVICIO RECUPERACION DE DEUDAS (FACTORE0)',
   
    '342002' => 
   
       'FABRICACION DE CARROCERIAS PARA CAMIONES DE LAS PARTIDAS ARANCELARIAS 870790002 Y 870790500029',
    
    '242901' => 
    
       'FABRICACION DE OTROS PRODUCTOS QUIMICOS.',
    
    '012203' => 
    
       'CRIA DE MARIPOSAS',
    
    '641101' => 
    
       'ACTIVIDADES POSTALES Y DE CORREO',
    
   '602301' => 
   
       'SERVICIO DE ACARREO Y DISTRIBUCION DE TODO TIPO DE MERCANCIA (INCLUYE LA MUDANZA NACIONAL)',
    
    '751202' => 
 
       'COMITES CANTONALES DE DEPORTES Y RECREACION',
    
    '154301' => 
    
       'ELABORACION DE CACAO',
    
    '524001' => 
   
       'VENTA DE TODO TIPO DE ARTICULOS USADOS',
    
    '154302' => 
   
       'ELABORACION DE CHOCOLATE',
     
    '712201' => 
  
       'ALQUILER DE MAQUINARIA Y EQUIPO DE CONSTRUCCION E INGENIERIA CIVIL',
      
    '960107' => 
    
       'PRODUCTORES DE MERCANCIAS EXENTAS DEL IMPUESTO DE VENTAS',
     
    '523406' => 
   
       'VENTA AL POR MAYOR Y MENOR DE ARTICULOS ELECTRONICOS, ELECTRICOS Y SIMILARES',
     
    '269501' => 
  
       'FABRICACION DE ARTICULOS DE CEMENTO, YESO Y HORMIGON PARA LA CONSTRUCCION',
    
    '749911' => 
   
       'AGENCIAS FOTOGRAFICAS POR CATALOGO',
    
    '269201' => 
  
       'FABRICACION DE PRODUCTOS DE CERAMICA REFRACTARIA',
     
    '289902' => 
    
       'FABRICACION DE OTROS PRODUCTOS ELABORADOS DE METAL N.C.P.',
      
    '749903' => 
    
       'SERVICIOS SECRETARIALES Y/O OFICINISTA',
    
    '921101' => 
   
       'ACTIVIDADES PRODUCCION POSTPRODUCCION Y DISTRIBUCION DE PELICULAS CINEMATOGRAFICAS (VIDEOS)',
    
    '659902' => 
 
       'FIDEICOMISOS Y/O ADMINISTRADORES DE FONDOS DE INVERSION',
      
    '523917' => 
   
       'VENTA AL POR MENOR DE BICICLETAS Y SUS ACCESORIOS',

    '930301' => 
       'SERVICIOS FUNEBRES Y ACTIVIDADES CONEXAS',
   
    '512202' => 
       'VENTA AL POR MAYOR DE FRUTAS, VERDURAS FRESCAS Y LEGUMBRES Y HORTALIZAS EXENTA DE VENTAS',
  
    '851206' => 

       'ORTOPEDISTA (CONSULTA PRIVADA)',
   
    '523910' => 
  
       'VENTA AL POR MENOR DE JUGUETES Y/O ARTICULOS DE ESPARCIMIENTO',
   
    '660201' => 
    
       'OPERADORA DE PENSIONES',
   
    '523920' => 
       'VENTA AL POR MENOR DE ANTEOJOS Y ARTICULOS OPTICOS (OPTICA)',
   
    '313001' => 'FABRICACION DE HILOS Y CABLES AISLADOS',
    
    '152003' =>  'ELABORACION DE PRODUCTOS LACTEOS EXENTOS DE IMP. VENTAS',
    
    '851908' =>  'LABORATORIOS MEDICOS - CLINICOS',
    
    '523701' => 
       'LIBRERIAS',
    
    '251901' =>  'FABRICACION DE OTROS PRODUCTOS DE CAUCHO',
    
  );
  return $array;
}
  

      public function output() {

        global $current_section;

        $settings = $this->get_settings($current_section);
        WC_Admin_Settings::output_fields($settings);
      }

      public function save() {
        global $current_section;
        if ($current_section == 'email_section') {
          $this->email_save();
        }
        else if ($current_section == 'consecutivo_section') {
          self::consecutivo_save();
        }
        else {
          $this->hacienda_save();
        }
      }

      public function hacienda_save() {
         // se eliminaran las lineas porque para actaulizar la informaciion de hacienda era requerida la llave criptografica.
          if (!function_exists('wp_handle_upload'))
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
          $file = wp_handle_upload($_FILES['Llave_Criptográfica'], array('test_form' => FALSE, 'unique_filename_callback' => __CLASS__ . '::fe_unique_name'), '9999/99');
          if (isset($file['file'])) {
            $cert = base64_encode(file_get_contents($file['file']));
            if (!copy($file['file'], CERTS_PATH . '/cert.pfx')) {
              if (!file_exists(CERTS_PATH . '/cert.pfx')) {
                WC_Admin_Settings::add_error('Error uploading "Llave Criptografica"');
              }
            }
          }
        
        

        $data = array(
          'post' => $_POST,
          'file' => $cert,
          'license' => woocommerce_factura_electronica_get_option(),
          'domain' => strstr($_SERVER['SERVER_SOFTWARE'], 'nginx') ? $_SERVER['HTTP_HOST'] : $_SERVER['SERVER_NAME'],
          'license_type' => woocommerce_factura_electronica_get_option(TRUE),
          'wp_flag' => true
        );

       //validaciones de campos en el emisor
        $validacion_texto = self::validar_campos($_POST);
        //entra solo si ES TRUE , Sino esque porque devuelve un texto con error. Si pongo si es === boolean me da error
        if ( $validacion_texto === TRUE ) {
          $settings = $this->get_settings();
          WC_Admin_Settings::save_fields($settings);
          //sent values to the server
          //no need to save this locally since we saved on the server to avoid hacking
          wp_remote_post(FE_SERVER . '/wp-json/registrar/v1/llave/', array(
            'headers' => array('Content-Type' => 'application/json; charset=utf-8'),
            'body' => json_encode($data)
              )
          );
        }
        else {
          WC_Admin_Settings::add_error($validacion_texto);
        }
      }
      /*
       * validar todos los campos referidos en los settings del emisor
       */
      public function validar_campos($post) {
        $data = true;
        if ($post['wc_settings_tab_efactura_full_name'] == NULL) {
          $data = __('El nombre completo es un nombre ');
        }
        else if ($post['wc_settings_tab_efactura_username'] == NULL) {
          $data = __('El usuario de hacienda es un campo requerido. ');
        }
        else if ($post['wc_settings_tab_efactura_password'] == NULL) {
          $data = __('El password de hacienda es un campo requerido.');
        }
        else if ($post['wc_settings_tab_efactura_pin'] == NULL) {
          $data = __('El pin es un campo requerido ');
        }
        else if ($post['wc_settings_tab_efactura_office'] == NULL) {
          $data = __('La sucursal es un campo requerido. ');
        }
        else if ($post['wc_settings_tab_efactura_pointsale'] == NULL) {
          $data = __('El punto de venta es un campo requerido. ');
        }
        else if ($post['wc_settings_tab_efactura_id_number'] == NULL) {
          $data = __('La cédula es un campo requerido.');
        }
        else if ($post['wc_settings_tab_efactura_store_email'] == NULL) {
          $data = __('El email es un campo requerido.');
        }
        else if ($post['wc_settings_tab_efactura_store_phone'] == NULL) {
          $data = __('El teléfono es un campo requerido.');
        }
        else if ($post['wc_settings_tab_efactura_sec_code'] == NULL) {
          $data = __('El código de segurida  es un campo requerido y es utilizado para tu claves electronicas.');
        }
        //si es true es porque todos los campos se cumplieron, entonces valida que los campos cumplan con el formato correcto.
        if ( $data === TRUE ) {
           //valida los campos numericos.
          $data = self::validar_campos_numericos($post);
        }
        return $data;
      }
      /*
       * Validar que cumpla con los formatos requeridos.
       */
      public function validar_campos_numericos($post) {
        $data = true;

        if (strlen(str_replace("-", "", $post['wc_settings_tab_efactura_id_number'])) != 12) {
          $data = __('El formato de la cédula debe de cumplir con 12 dígitos.');
        }
        else if (strlen(str_replace("-", "", $post['wc_settings_tab_efactura_store_phone'])) != 8) {
          $data = __('El formato del telefono es de 8 digitos.');
        }
        else if (strlen(str_replace("-", "", $post['wc_settings_tab_efactura_office'])) != 3) {
          $data = __('El formato de la  sucursal es de 3 dígitos númericos. ');
        }
        else if (strlen(str_replace("-", "", $post['wc_settings_tab_efactura_pointsale'])) != 5) {
          $data = __('El formato del punto de venta es de 5 dígitos númericos. ');
        }
        else if (strlen(str_replace("-", "", $post['wc_settings_tab_efactura_sec_code'])) != 8) {
          $data = __('El formato del codígo de seguridad es de 8 dígitos.');
        }
        return $data;
      }

      /**
       --Guarda los consecutivos por separado en cada evento efectuado.
       */
      public function consecutivo_save() {
        $settings = $this->get_settings('consecutivo_section');
        if ($_POST['wc_settings_tab_efactura_consecutivo_fe']) {
          update_option('order_consecutive_fe', $_POST['wc_settings_tab_efactura_consecutivo_fe']);
        }
        if ($_POST['wc_settings_tab_efactura_consecutivo_ticket']) {
          update_option('order_consecutive_ticket', $_POST['wc_settings_tab_efactura_consecutivo_ticket']);
        }
        if ($_POST['wc_settings_tab_efactura_consecutivo_nota_credito']) {
          update_option('order_consecutive_nota_credito', $_POST['wc_settings_tab_efactura_consecutivo_nota_credito']);
        }
        if ($_POST['wc_settings_tab_efactura_consecutivo_mensaje']) {
          update_option('order_consecutive_mensaje', $_POST['wc_settings_tab_efactura_consecutivo_mensaje']);
        }
        if ($_POST['wc_settings_tab_efactura_consecutivo_exportacion']) {
          update_option('order_consecutive_exportacion', $_POST['wc_settings_tab_efactura_consecutivo_exportacion']);

        }
        if ($_POST['wc_settings_tab_efactura_consecutivo_compra']) {
         update_option('order_consecutive_compra', $_POST['wc_settings_tab_efactura_consecutivo_compra']);

       }
        
        WC_Admin_Settings::save_fields($settings);
      }

      public function email_save() {
        $settings = $this->get_settings('email_section');
        WC_Admin_Settings::save_fields($settings);
      }
      public function fe_report_with_range(){
         ?>
         <br>
         <div> 
            <label>Desde: </label>
            <input type="date" id="from-date" class="form-control"  step="1"/>
            <label>Hasta: </label>
            <input class="form-control" id="to-date" type="date" step="1"/>
            <button class="button-primary" id="get-report-invoices">Consultar</button>
            <div hidden class="link-div">
               <a class="link-report" href="" download target="_blank">Descargar reporte</a>
            </div>
         </div>
         <br><br>
         <iframe frameborder="0" width="100%" height="680px" class="iframe-pdf" src=""></iframe>
         <?php
      }
      public function fe_contacts_page_handler() {
        $table = new WC_Efactura_Aceptacion();
        $table->prepare_items();

        $message = '';
        if ('delete' === $table->current_action()) {
          $message = '<div class="updated below-h2" id="message"><p>' . sprintf(__('Items deleted: %d', 'wpbc'), strlen($_REQUEST['clave'])) . '</p></div>';
        }
        ?>     
        <div id="message-holder"  style="visibility: hidden">
            <?php echo $message; ?>
        </div>
        <form id="contacts-table" method="POST">
            <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>"/>
            <?php $table->display() ?>
        </form>

        <?php
        $url = admin_url('admin-ajax.php');
        $nonce_files = wp_nonce_field('protect_content', 'my_nonce_field');
        ?>
        <div id="cargar-facturas">
            <a class="wc-update-now button-primary" ref='#' onClick="window.location.reload()">
                CARGAR FACTURAS</a>
        </div>
        <br />
        <div id="dropzone-wordpress"><form action="<?php echo $url ?>" class="dropzone needsclick dz-clickable" id="dropzone-wordpress-form">
                <?php echo $nonce_files ?>
                <div class="dz-message needsclick">
                    Drop files here or click to upload.<br>
                    <span class="note needsclick">(Files are uploaded)</span>
                </div>
                <input type='hidden' name='action' value='submit_dropzonejs_fe'>
            </form></div>

        <?php
      }

      /**
       * get all emails registered in WooCommerce
       * @param  boolean $remove_defaults switch to remove default woocommerce emails
       * @return array   $emails       list of all email ids/slugs and names
       */
      public function get_fe_woocomerce_emails() {
        // get emails from WooCommerce
        global $woocommerce;
        $mailer = $woocommerce->mailer();
        $wc_emails = $mailer->get_emails();

        $non_order_emails = array(
          'customer_note',
          'customer_reset_password',
          'customer_new_account'
        );

        $emails = array();
        foreach ($wc_emails as $class => $email) {
          if (!in_array($email->id, $non_order_emails)) {
            switch ($email->id) {
              case 'new_order':
                $emails[$email->id] = sprintf('%s (%s)', $email->title, __('Admin email'));
                break;
              case 'customer_invoice':
                $emails[$email->id] = sprintf('%s (%s)', $email->title, __('Manual email'));
                break;
              default:
                $emails[$email->id] = $email->title;
                break;
            }
          }
        }

        return $emails;
      }

    }

    return new WC_Efactura_Settings();
  }

  function fe_dropzonejs_upload() {

    if (!empty($_FILES) && wp_verify_nonce($_REQUEST['my_nonce_field'], 'protect_content')) {
      $file = ACCEPTED_PATH . $_FILES['file']['name'];
      $uploaded_bits = file_put_contents(
          $file, file_get_contents($_FILES['file']['tmp_name'])
      );

      if (!$uploaded_bits || !(isset($_FILES['file']['type']) && $_FILES['file']['type'] == 'text/xml')) {
        $error = $uploaded_bits['error'];
        return add_action('admin_notices', function() use ( $error ) {
          $msg[] = '<div class="error"><p>';
          $msg[] = '<strong>Factura Electronica XML Aceptacion</strong>: ';
          $msg[] = sprintf(__('wp_upload_bits failed,  error: "<strong>%s</strong>'), $error);
          $msg[] = '</p></div>';
          echo implode(PHP_EOL, $msg);
        });
      }



      //create record
      try {
        global $wpdb;
        $table_name = $wpdb->prefix . 'fe_aceptadas';
        $xml = simplexml_load_file($file);
        if ($xml) {
          $emisor = $xml->Emisor;
          $receptor = $xml->Receptor;
          $record = array(
            'clave' => (string)$xml->Clave,
            'emisor' => (string)$emisor->Nombre,
            'tipo_emisor' => (int)$emisor->Identificacion->Tipo,
            'cedula_emisor' => (string)$emisor->Identificacion->Numero,
            'receptor' => (string)$receptor->Nombre,
            'tipo_receptor' => (int)$receptor->Identificacion->Tipo,
            'cedula_receptor' => (string)$receptor->Identificacion->Numero,
            'file_name' => $_FILES['file']['name'],
            'fecha' => (string)$xml->FechaEmision,
          );          
          $wpdb->insert($table_name, $record);
        }
      }
      catch (Exception $ex) {
        error_log('Error creating record: ' . $ex->getMessage());
      }
    }
    die();
  }

  add_filter('woocommerce_get_settings_pages', 'e_factura__add_settings', 15);
  add_action('wp_ajax_nopriv_submit_dropzonejs_fe', 'fe_dropzonejs_upload');
  add_action('wp_ajax_submit_dropzonejs_fe', 'fe_dropzonejs_upload');

endif;